      subroutine catena(nrp,ngr,nspgr,box,x,panm,iprint,iknot)
C     Prepares for analyzing knotted structures.
C     Calls KNOT to check whether or not two molecules are knotted,
C        i.e. a bond (line L) crosses three atoms defining a plane (V).
C     B.P. van Eijck, october 2019.

C     Input through parameters:
C     NRP: number of atoms
C     NGR: number of groups
C     NSPGR: number of symmetry operations (E excluded, nop = 0...NSPGR)
C     BOX(1..6): ax, by, cz, bx, cx, cy
C     X(1..): atom coordinates,including symmetry-expanded coordinates.
C     PANM(1..): atom names
C     IPRINT regulates output to unit 16 and to screen.

C     Output through parameters:
C     X1(1..3*NGR): coordinates of first ring
C     X2(1..3*NGR): coordinates of second ring
C     IKNOT: number of knots (corrected for in-out situations)

C     OUT(1...): results coming back from KNOT and CROSS
C     OUT(1): number eliminated in first preselection
C     OUT(2): number eliminated in second preselection
C     OUT(3): number eliminated in final selection
C     OUT(4): number of hits: line P crosses plane V
C     OUT(10): number of sets investigated by KNOT

C     IDAT(1...): data for KNOT
C     IDAT(1): cell shift along a (-1,0,1)
C     IDAT(2): cell shift along b (-1,0,1)
C     IDAT(3): cell shift along c (-1,0,1)
C     IDAT(4): number of symmetry operation (0...NSPGR)
C     IDAT(5): natoms

      implicit none
      double precision ax,bx,by,cx,cy,cz,
     +     x(*),box(*),x1(1000),x2(1000),
     +     c,zw1(3),zw2(3),straal,dist,dmin
      integer kring,nring(10),iring(10,100),out(10),idat(10),ddat(10),
     +     natoms,nspgr,iprint,nop,i,j,k,k1,k2,m,n,n1,n2,n3,nk,ipr,
     +     nrp,ngr,igr1,igr2,iknot
      character regel*200
      character*8 panm(*)

      do i=1,10
         out(i)=0
         idat(i)=0
         ddat(i)=0
      enddo
      dmin=1.0d10
      natoms=nrp*ngr
      idat(5)=natoms
      ipr=abs(iprint)
      if (ipr.gt.2) then
         print 22, nrp,ngr,nspgr
         write(16,22) nrp,ngr,nspgr
      endif
      if (ipr.gt.4) then
         print*,' Coordinates with symmetry-expanded ones'
         do i=1,natoms*(nspgr+1)
            print 41,i,panm(i),(x(3*i-3+m),m=1,3)
         enddo
      endif

      do m=1,3
         if (box(m).lt.0.05d0) then
            print 90,(box(k),k=1,6)
            print*,'***** CATENA: Cell axes less than 0.5 Angstrom.'
            write(16,*)'***** CATENA: Cell axes less than 0.5 Angstrom.'
            stop
         endif
      enddo

      nk=0
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)

      read(49,*,end=110,err=110) kring
      goto 120
  110 continue
      print*,' CATENA: Read error in file RINGS.DEF'
      write(16,*) ' CATENA: Read error in file RINGS.DEF'
      stop
  120 continue
      if (kring.eq.0) then
         print*,' ***** No rings in RINGS.DEF'
         write(16,*) ' ***** No rings in RINGS.DEF'
         return
      endif
      if (ipr.gt.2) then
         print*,kring,' rings in each group of molecules'
         write(16,*) kring,' rings in each group of molecules'
      endif
      do i=1,kring
         read(49,21) regel
         call intstr(regel,1,nring(i))
         n=nring(i)+1
         do j=1,n
            call intstr(regel,j+1,iring(i,j))
         enddo
         if (ipr.gt.3) then
            print 23,i,nring(i)
            print 42,(iring(i,j),j=1,n)
         endif
C        Last atom must be equivalent to first
         if (iring(i,1).ne.iring(i,n)) then
            print*,' ***** First and last atoms are different'
            stop
         endif
         do j=1,n-1
            k1=iring(i,j)
            k2=iring(i,j+1)
            c=0
            do m=1,3
               c=c+(x(3*k1-3+m)-x(3*k2-3+m))**2
            enddo
            c=sqrt(c)
            if (c.lt.0.115 .or. c.gt.0.180) then
               print*,' ***** CATENA: Unrealistic bond distance:',
     +              k1,panm(k1),k2,panm(k2),c
               write(16,*)'CATENA: Unrealistic bond distance:',
     +              k1,panm(k1),k2,panm(k2),c
               iknot=-100
               return
            endif
         enddo
      enddo

      do igr1=0,ngr-1
         do i=1,nrp
ccc            panm(i+igr1*nrp)=panm(i)
         enddo
      enddo

      do igr1=0,ngr-1
C        Calculate largest distance from atom to center of molecule
C        This may not work well for several molecules in the group!!
         do i=1,3*nrp
            x1(i)=x(i+3*(igr1*nrp))
         enddo
         do m=1,3
            zw1(m)=0
            do i=1,nrp
               zw1(m)=zw1(m)+x1(3*i-3+m)
            enddo
            zw1(m)=zw1(m)/nrp
         enddo
         straal=0
         do i=1,nrp
            c=0
            do m=1,3
               c=c+(x1(3*i-3+m)-zw1(m))**2
            enddo
            if (c.gt.straal) straal=c
         enddo
         straal=sqrt(straal)
         if (ipr.gt.3) print*,' Largest distance from atom to',
     +        ' center of molecule',straal

C        Determine distances between molecules
         do igr2=0,ngr-1
            do n1=-2,2
               idat(1)=n1
               do n2=-2,2
                  idat(2)=n2
                  do n3=-2,2
                     idat(3)=n3
                     do nop=0,nspgr
                        idat(4)=nop
                        if (.not. (igr1.eq.igr2 .and. n1.eq.0 .and.
     +                       n2.eq.0 .and. n3.eq.0 .and. nop.eq.0)) then
                           do i=1,natoms
                              j=3*i-3
                              k=j+3*(igr2*nrp+nop*natoms)
                              x2(j+1)=x(k+1)+n1*ax+n2*bx+n3*cx
                              x2(j+2)=x(k+2)+n2*by+n3*cy
                              x2(j+3)=x(k+3)+n3*cz
                           enddo
                           do m=1,3
                              zw2(m)=0
                              do i=1,nrp
                                 zw2(m)=zw2(m)+x2(3*i-3+m)
                              enddo
                              zw2(m)=zw2(m)/nrp
                           enddo
                           c=0
                           do m=1,3
                              c=c+(zw1(m)-zw2(m))**2
                           enddo
                           dist=sqrt(c)
                           call knot(x1,x2,panm,kring,nring,iring,
     +                          iprint,out,idat,dist)
                           nk=nk+1
                           if (ipr.gt.3 .and. dist.lt.1.0) 
     +                          print *,igr1,igr2,n1,n2,n3,nop,dist
                           if (dist.lt.dmin) then
                              dmin=dist
                              do m=1,4
                                 ddat(m)=idat(m)
                              enddo
                           endif
                        endif
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo

      if (ipr.gt.3) print*,'Shortest distance',dmin,' straal', straal
cc      pause

      do n1=-2,2
         idat(1)=n1
         do n2=-2,2
            idat(2)=n2
            do n3=-2,2
               idat(3)=n3
               if (.not. (n1.eq.0 .and. n2.eq.0 .and. n3.eq.0)) then
                  do nop=0,nspgr
                     idat(4)=nop
                     do i=1,natoms
                        j=3*i-3
                        k=j+3*nop*igr2*nrp
                        x2(j+1)=x(k+1)+n1*ax+n2*bx+n3*cx
                        x2(j+2)=x(k+2)+n2*by+n3*cy
                        x2(j+3)=x(k+3)+n3*cz
                     enddo
                     if (ipr.gt.4) print*,' Analyzing equivalent posit',
     +                    'ion at',nop+1,' at translation',n1,n2,n3
ccc                     if (dist.lt.1.0) then
ccc                        call knot(x1,x2,panm,kring,nring,iring,
ccc     +                       iprint,out,idat,dist)
ccc                        nk=nk+1
ccc                     endif
                     if (dist.lt.dmin) then
                        dmin=dist
                        do m=1,4
                           ddat(m)=idat(m)
                        enddo
                     endif
                  enddo
               endif
            enddo
         enddo
      enddo

      if (ipr.gt.2) then
         print 24,dmin,(ddat(m),m=1,4),out(4)
         print*,' Subroutine KNOT called',nk,' times'
         print*,out(10),' sets investigated'
         print*,out(1),' sets rejected in first preselection'
         print*,out(2),' sets rejected in second preselection'
         print*,out(3),' sets rejected in final selection'
      endif
      if (ipr.gt.1) then
         if (out(4).eq.0) then
            print*,'          No knots found'
            write(16,*) '          No knots found'
         else
            write(16,*) out(4),' sets give crossing of line with plane'
            print*,out(4),' sets give crossing of line with plane'
            write(16,*) out(6),' sets give real knottings of rings' 
            print*,out(6),' sets give real knottings of rings' 
         endif
      endif
      iknot=out(6)
      
   21 format(a)
   22 format(' Starting CATENA',i5,' atoms',i3,' group(s)',
     +     i3,' equivalent positions')
   23 format(' Ring number',i3,' with',i3,' atoms')
   24 format('  Shortest distance between intermolecular centers',
     +     f10.4,' for shifts',3i3,' and symmetry operation',i3,
     +     ' - crossings',i4)
   41 format(i4,3x,a5,3f10.5)
   42 format(30i4)
   90 format(' Box: ',6f10.4)
      end


      subroutine knot(x,f,panm,kring,nring,iring,iprint,out,idat,dist)

C     Compares two rings in separate molecules, coordinares X and F.
C     Defines planes of three atoms in X and lines between two atoms in F.
C     Calls CROSS to perform the actual comparisons.

C     Input through parameters:
C     IDAT(1...): data from CATENA
C     IDAT(1): cell shift along a (-1,0,1)
C     IDAT(2): cell shift along b (-1,0,1)
C     IDAT(3): cell shift along c (-1,0,1)
C     IDAT(4): number of symmetry operation (0...NSPGR)
C     IDAT(5): natoms

C     Input coming back from CROSS
C     OUT(4): number of hits: line P crosses plane V
C             OUT(4) is incremented cumulatively
C     OUT(5): +1 or -1 if this run gives a hit, else 0
C             The sign gives the direction of the crossing
C     OUT(6): number of hits corrected for in-out situations
C     OUT(10): number of sets investigated by KNOT

      implicit none
      integer kring,nring(10),iring(10,100),out(*),idat(*),ipr,
     +     i,ii,jj,i1,i2,i3,i4,i5,m,j,n,k,nc,nat,idir,iprint
      double precision x(*),f(*),r(5,3),c,zw1(3),zw2(3),straal,dist
      character*4 filenm*20
      character*8 panm(*),temp

      data nc /0/
      save nc

      ipr=abs(iprint)
      nat=idat(5)
      if (nat.gt.0) then
C        Preselection: Compare positions of molecule 1 and equivalent ones.
         do m=1,3
            zw1(m)=0
            zw2(m)=0
            do i=1,nat
               zw1(m)=zw1(m)+x(3*i-3+m)
               zw2(m)=zw2(m)+f(3*i-3+m)
            enddo
            zw1(m)=zw1(m)/nat
            zw2(m)=zw2(m)/nat
         enddo
         straal=0
         do i=1,nat
            c=0
            do m=1,3
               c=c+(x(3*i-3+m)-zw1(m))**2
            enddo
            if (c.gt.straal) straal=c
         enddo
         straal=sqrt(straal)
         c=0
         do m=1,3
            c=c+(zw1(m)-zw2(m))**2
         enddo
         dist=sqrt(c)
         if (dist.gt.2.0*straal) return
      endif
      if (ipr.gt.3) then
         print 83,dist,(idat(k),k=1,4)
      endif

C     Test phase: write SPF files for Pluton.
      if (ipr.gt.100 .and. dist.lt.0.9) then
         do n=1,kring
C        Consider all rings in the first molecule (coordinates X)
            do m=1,kring
C              Consider all rings in the second molecule (coordinates F)
               write(filenm,84) n,m
               open(unit=33,file=filenm)
               write(33,80) filenm
               do ii=1,nring(n)
                  jj=iring(n,ii)
                  write(33,81)panm(jj),(10.0*x(3*jj-3+k),k=1,3)
               enddo
C              For clarity the C atoms in the second ring are red
               do ii=1,nring(m)
                  jj=iring(m,ii)
                  temp=panm(jj)
                  if (temp(1:1).eq.'C') temp(1:1)='O'
                  write(33,81) temp,(10.0*f(3*jj-3+k),k=1,3)
               enddo
               close (33)
            enddo
         enddo
      endif
C     end test phase

      do n=1,kring
C        Consider all rings in the first molecule (coordinates X)
C        Find center of ring
         do m=1,3
            zw1(m)=0
            do i=1,nring(n)
               i3=iring(n,i)
               zw1(m)=zw1(m)+x(3*i3-3+m)
            enddo
            r(3,m)=zw1(m)/nring(n)
         enddo
         if (ipr.gt.4 .and. dist.lt.0.7) then
            print*,' Center for ring',n,(r(3,m),m=1,3)
            i3=iring(n,nring(1))
            print*,' First atom     ',iring(n,1),(x(3*i3-3+m),m=1,3)
         endif
         do m=1,kring
C           Consider all rings in the second molecule (coordinates F)
            idir=0
cc          print*,' reset idir' 
            i3=iring(n,nring(n))
C           Consider all atoms in the first ring
            do i=1,nring(n)
               i1=iring(n,i)
               i2=iring(n,i+1)
C              i1,i2,i3 define a triangle in a plane V
c              print*,' Triangle V',n,i1,i2,i3
C              Consider all atoms in the second ring
               do j=1,nring(m)
                  i4=iring(m,j)
                  i5=iring(m,j+1)
C                 i4 and i5 define a line; does it cross V in a point P?
                  do k=1,3
                     r(1,k)=x(3*i1-3+k)
                     r(2,k)=x(3*i2-3+k)
c                     r(3,k)=x(3*i3-3+k)
                     r(4,k)=f(3*i4-3+k)
                     r(5,k)=f(3*i5-3+k)
                  enddo
                  call cross(r,out,0)
                  nc=nc+1

                  if (out(5).ne.0) then
C                    A crossing is found
                     out(6)=out(6)+1
                     idir=idir+out(5)
ccc                 print*,' idirchange',idir,out(5)
                     if (ipr.gt.2) then
                        print 85,out(4),(idat(k),k=1,4),
     +                       n,i1,i2,i3,m,i4,i5,out(5)
ccc                 print*,n,m,idir,out(5)
                        write(16,85) out(4),(idat(k),k=1,4),
     +                       n,i1,i2,i3,m,i4,i5,out(5)
                        if (idir.eq.0) then
                           print 86,n,m
                           write(16,86) n,m
                        endif
                     endif
                     if (idir.eq.0) out(6)=out(6)-2
                     if (iprint.gt.1) then
C                       Prepare SPF-files for Pluton.
                        write(filenm,82) out(4)
                        open(unit=33,file=filenm)
                        write(33,80) filenm
                        do ii=1,nring(n)
                           jj=iring(n,ii)
                           write(33,81)panm(jj),(10.0*x(3*jj-3+k),k=1,3)
                        enddo
C                       For clarity the C atoms in the second ring are red
                        do ii=1,nring(m)
                           jj=iring(m,ii)
                           temp=panm(jj)
                           if (temp(1:1).eq.'C') temp(1:1)='O'
                           write(33,81) temp,(10.0*f(3*jj-3+k),k=1,3)
                        enddo
                        close (33)
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo
      out(10)=nc
c      print*,'Netto',out(6)

   80 format('TITL  ',a)
   81 format('ATOM  ',a4,3f10.5)
   82 format('Knot',i2.2,'.spf')
   83 format(' Short distance ',f8.3,' to molecule with shifts',3i3,
     +     ',  symmetry operation',i3)
   84 format('Test',2i1,'.spf')
   85 format('Knot',i4,'  Shifts',3i3,',  symmetry ',
     +     'operation',i3, '.  Ring number',i3,', atoms',3i4,
     +     ', crossed by ring ',i3, ', atoms', 2i4,', sign',i3)
   86 format('Rings',i3,' and',i3,' have crossings,'
     +     ' but probably no real knots')
      end



      subroutine cross(r,out,ipri)
C     Performs the final test.
C     B.P. van Eijck, october 2019.

C     INPUT: r(5,3) defining five points.
C     Points 1,2,3 define a plane V.
C     Perpendicular to this plane is the vector a = r1  r3.
C     Points 4 and 5 define a line L.
C     Problem: does L cross plane V within triangle 1,2,3?

C     Rotate these 5 points until plane V is perpendicular to the z-axis.
C     (Is the handedness of 1,2,3 relevant?)

C     Input through parameters:
C     R(1:5,1:3): coordinates

C     Output through parameters:
C     OUT(1): number eliminated in first preselection
C     OUT(2): number eliminated in second preselection
C     OUT(3): number eliminated in final selection
C     OUT(4): number of hits: line P crosses plane V
C     OUT(1) ... OUT(4) are incremented cumulatively

C     OUT(5): +1 or -1 if this run gives a hit, else 0
C             The sign gives the direction of the crossing

      implicit none
      double precision r(5,3),a(3),s(5,3),t(5,3),u(5,3),c,
     +     alfa,beta,gamma,xp,yp,x1,x3,y3,xq,yq,c3,cp
      integer i,m,out(*),ipri

      out(5)=0
C     Put point 2 in the origin.
      do i=1,5
         do m=1,3
            t(i,m)=r(i,m)-r(2,m)
         enddo
      enddo
      a(1)=t(1,2)*t(3,3)-t(1,3)*t(3,2)
      a(2)=t(1,3)*t(3,1)-t(1,1)*t(3,3)
      a(3)=t(1,1)*t(3,2)-t(1,2)*t(3,1)

C     Rotate around x-axis until a(2)=0
      alfa=atan2(a(2),a(3))
      do i=1,5
         s(i,1)=t(i,1)
         s(i,2)=t(i,2)*cos(alfa) - t(i,3)*sin(alfa)
         s(i,3)=t(i,2)*sin(alfa) + t(i,3)*cos(alfa)
      enddo
      a(1)=a(1)
      a(3)=a(2)*sin(alfa) + a(3)*cos(alfa)
      a(2)=0

C     Rotate around y-axis until a(1)=0
      beta=atan2(a(1),a(3))
      do i=1,5
         t(i,1)=s(i,1)*cos(beta) - s(i,3)*sin(beta)
         t(i,2)=s(i,2)
         t(i,3)=s(i,1)*sin(beta) + s(i,3)*cos(beta)
      enddo
      a(3)=a(1)*sin(beta) + a(3)*cos(beta)
      a(1)=0
      a(2)=0

C     Now all points 1, 2, 3 are in the x,y-plane.
C     So t(1,3) and t(3,3) must be very nearly zero.
      if (abs(t(1,3)).ge.1.0d-10 .or. abs(t(3,3)).ge.1.0d-10) then
         print*,' ***** Atoms 1 and 3 must be in the x,y plane.',
     +        ' Error in algorithm. The t-coordinates:'
         do i=1,5
            print*,(t(i,m),m=1,3)
         enddo
         stop
      endif
C     And a(3) is expected to be positive
      if (a(3).le.0) then
         print*,' ***** Vector a = r1  r3 must be along the positive',
     +        'z axis. Error in algorithm. The t-coordinates:'
         do i=1,5
            print*,(t(i,m),m=1,3)
         enddo
         stop
      endif

C     First preselection:
C     If t(4,3) and t(5,3) have the same sign, rhere is no point P.
      if (t(4,3)*t(5,3).gt.1.0d-12) then
         out(1)=out(1)+1
         return
      endif

C     Rotate around z-axis until y(1)=0
      gamma= -atan2(t(1,2),t(1,1))
      do i=1,5
         u(i,1)=t(i,1)*cos(gamma) - t(i,2)*sin(gamma)
         u(i,2)=t(i,1)*sin(gamma) + t(i,2)*cos(gamma)
         u(i,3)=t(i,3)
      enddo
      x1=u(1,1)
      x3=u(3,1)
      y3=u(3,2)

C     Check signs of u-coordinates
      if (abs(u(1,2)).ge.1.0d-10)then
         print*,' ***** The y coordinate of atom 1 is nonzero.',
     +        ' Error in algorithm.'
         do i=1,5
            print*,(u(i,m),m=1,3)
         enddo
         stop
      endif
      if (x1.lt.0) then
         print*,' ***** The x coordinate of atom 1 is negative.',
     +        ' Error in algorithm.'
         do i=1,5
            print*,(u(i,m),m=1,3)
         enddo
         stop
      endif
      if (y3.lt.0) then
         print*,' ***** The y coordinate of atom 3 is negative.',
     +        ' Error in algorithm.'
         do i=1,5
            print*,(u(i,m),m=1,3)
         enddo
         stop
      endif

C     Calculate point P where L crosses V.
      c=u(5,3)/(u(5,3)-u(4,3))
      xp=u(5,1)+c*(u(4,1)-u(5,1))
      yp=u(5,2)+c*(u(4,2)-u(5,2))

      if (ipri.gt.2) then
         do i=1,5
            print 12,(u(i,m),m=1,3)
         enddo
         print 12,xp,yp
      endif

C     Second preselection:
C     P should be within the square surrounding the triangle 1-2-3
      if (yp.lt.0 .or. yp.gt.y3) goto 210
      if (x3.lt.0) then
         if (xp.lt.x3 .or. xp.gt.x1) goto 210
      elseif (x3.gt.x1) then
         if (xp.lt.0 .or. xp.gt.x3) goto 210
      else
         if (xp.lt.0 .or. xp.gt.x1) goto 210
      endif

C     Final check P-coordinates: should be within triangle 1-2-3
      cp=yp/xp
      xq=y3*x1/(y3-cp*(x3-x1))
      yq=cp*xq
      if (yp.gt.yq) goto 220
      c3=y3/x3
      if (ipri.gt.2) print 12,xq,yq
      if (x3.lt.0) then
         if (xp.lt.0 .and. cp.gt.c3) goto 220
      else
         if (cp.gt.c3) goto 220
      endif

C     Crossing found!
      out(4)=out(4)+1
      if (u(4,3).gt.0.0) out(5)=1
      if (u(4,3).lt.0.0) out(5)=-1
      return

  210 continue
      out(2)=out(2)+1
      return
  220 continue
      out(3)=out(3)+1

   12 format(3f10.4)
      end



      subroutine rings(nat,nbond,ib,jb,iprint)
C     B. P. van Eijck, November 2017.

C     Calls subroutines TSTRNG and INDEX

      implicit none
      integer i,j,k,m,n,ii,jj,iprint,nbond,ib,jb,
     +     nat,tmax,nsteps,maxstp,ntr,kk,mm,nr,n1,n2,mtree,einde,
     +     k1,k2,icyc,i1,j1,ico,ntrf,maxbon,maxat,nar,ntree
      parameter(tmax=10000,maxstp=50,maxbon=200,maxat=1000)
      integer nb(maxat),iat(maxat,4),train(100,100),ntrain(100),
     +     tree(tmax,maxstp),first(maxat),last(0:maxat),
     +     ind(tmax),ring(tmax,maxstp),nring(tmax),temp(3),
     +     test(100),natr(tmax),atring(tmax,500)
      dimension ib(*),jb(*)

      nsteps=40
      ntrf=0

C     Count number of atoms bound to each atom: NB(I)
C     The numbers of these atoms are IAT(I,J=1..NB(I))
      do i=1,nat
         nb(i)=0
      enddo
      do i=1,nat
         do j=1,nbond
            if ((ib(j).eq.i)) then
               nb(i)=nb(i)+1
               iat(i,nb(i))=jb(j)
            endif
            if ((jb(j).eq.i)) then
               nb(i)=nb(i)+1
               iat(i,nb(i))=ib(j)
            endif
         enddo
      enddo
      if (iprint.gt.2) then
         do i=1,nat
            print*,i,'  ',nb(i),(iat(i,m),m=1,nb(i))
         enddo
      endif

C     Eliminate atoms that are in dead ends, not part of a ring
  150 continue
      do i=1,nat
         if (nb(i).eq.1) then
            j=iat(i,1)
            nb(i)=nb(i)-1
            do k=1,nb(j)
               if (iat(j,k).eq.i) then
                  do m=k,nb(j)-1
                     iat(j,m)=iat(j,m+1)
                  enddo
               endif
            enddo
            nb(j)=nb(j)-1
            goto 150
         endif
      enddo
      if (iprint.gt.1) then
         print*,' Atoms remaining after elimination of dead ends'
         do i=1,nat
            if (nb(i).gt.1) then
               print*,i,nb(i),' connections'
            endif
         enddo
      endif
      do i=1,nat
         if (nb(i).eq.4) then
            print*,' 4 links to atom',i,'. Not programmed.'
            stop
         endif
      enddo

C     Set arrays to zero        
      nr=0
      nar=0
      last(0)=0
      do i=1,tmax
         nring(i)=0
         natr(i)=0
         do j=1,maxstp
            ring(i,j)=0
            atring(i,j)=0
         enddo
      enddo

C     Find unconnected rings
      do ii=1,nat
         if (nb(ii).eq.2) then
            i=ii
            test(1)=ii
            j=iat(test(1),1)
            test(2)=j
            mm=2
  190       continue
            if (nb(j).gt.2) then
               if (iprint.gt.1) print*,
     +              ' Overconnected ring starting at',j
               goto 195
            else
               do k=1,2
                  if (iat(j,k).ne.i) then
                     m=iat(j,k)
                     mm=mm+1
                     test(mm)=m
                  endif
               enddo
            endif
            if (m.eq.ii) then
               if (iprint.gt.0) print*,' Unconnected ring found'
               nar=nar+1
               natr(nar)=mm
               do kk=1,mm
                  atring(nar,kk)=test(kk)
                  atring(nar,kk+mm)=test(kk+1)
               enddo
               do k=1,mm
                  nb(test(k))=0
               enddo
               if (iprint.gt.3) print 15,mm,(atring(nar,k),k=1,mm)
            else
               i=j
               j=m
               goto 190
            endif
         endif
  195    continue
      enddo

C     Find "trains" of atoms connected by only two bonds (nb=2).
C     First and last atoms are "knots" with nb>2.
      ntr=0
      do i=1,100
         do j=1,100
            train(i,j)=0
         enddo
      enddo
      do i=1,nat
         if (nb(i).gt.2) then
            do k=1,nb(i)
               ntr=ntr+1
               train(ntr,1)=i
               j=iat(i,k)
               train(ntr,2)=j
               m=2
  200          continue
               if (nb(j).eq.2) then
                  if (iat(j,1).eq.train(ntr,m-1)) jj=iat(j,2)
                  if (iat(j,2).eq.train(ntr,m-1)) jj=iat(j,1)
                  m=m+1
                  train(ntr,m)=jj
                  j=jj
                  goto 200
               endif
               ntrain(ntr)=m
               first(ntr)=train(ntr,1)
               last(ntr)=train(ntr,ntrain(ntr))

C              Look for trains which are in fact rings with one link
               if (first(ntr).eq.last(ntr)) then
                  print*,' Ring forming train found',ntr,
     +                 first(ntr),last(ntr)
                  nar=nar+1
                  natr(nar)=ntrain(ntr)
                  do kk=1,natr(nar)
                     atring(nar,kk)=train(ntr,kk)
                  enddo
                  if (iprint.gt.3) print 15,
     +                 natr(nar)-1,(atring(nar,kk),kk=1,natr(nar))
                  do i1=1,nar-1
                     if (first(ntr).eq.atring(i1,1) .and.
     +                    last(ntr).eq.atring(i1,natr(i1))) then
                        print*,' &&&&&&&',first(ntr),last(ntr)
                        nar=nar-1
                     endif
                  enddo
                  ntr=ntr-1
               endif
            enddo
         endif
      enddo
      if (iprint.gt.0) then
         do n=1,ntr
            print 12,n,(train(n,jj),jj=1,ntrain(n))
         enddo
      endif
C     There are now trains i=1..NTR of NTRAIN(i) atoms.
C     These atoms are numbered TRAIN(i,m) with m=1...NTRAIN(i)
C     First and last atoms ar given in correponding arrays.

C     Look for sets of trains with identical first and last atoms
      do i=1,nat
         if (nb(i).eq.3) then
            k=0
            do n=1,ntr
               if (first(n).eq.i) then
                  if (k.eq.0) j=last(n)
                  if (j.lt.i) goto 198
                  if (last(n).eq.j) then
                     k=k+1
                     temp(k)=n
                  endif
               endif
  198          continue
            enddo

            do ii=1,k-1
               do jj=ii+1,k
                  print*,' Simple ring found',i,j,ii,jj
                  n1=temp(ii)
                  n2=temp(jj)
                  nar=nar+1
                  mm=0
                  do kk=1,ntrain(n1)
                     mm=mm+1
                     atring(nar,mm)=train(n1,kk)
                  enddo
                  do kk=ntrain(n2)-1,1,-1
                     mm=mm+1
                     atring(nar,mm)=train(n2,kk)
                  enddo
                  natr(nar)=mm
                  if (iprint.gt.3) print 15,mm,(atring(nar,kk),kk=1,mm)
               enddo
            enddo 
         endif
      enddo              

      if (iprint.gt.0) print*,' Begin cycle of train combinations'
      do icyc=1,nat
C        Build a tree of train links
C        Start with any atom (ICYC) with at least 3 connections: nb>2.
         ntree=0
         if (nb(icyc).gt.2) then
            do i=1,tmax
               do j=1,maxstp
                  tree(i,j)=0
               enddo
            enddo
            do n=1,ntr
               if (icyc.eq.first(n)) then
c              print*,n,last(n)
                  do k=1,tmax
                     if (tree(k,1).eq.0) then
                        ntree=ntree+1
                        tree(ntree,1)=first(n)
                        tree(ntree,2)=last(n)
                        goto 210
                     endif
                  enddo
               endif
  210          continue
            enddo


            mtree=2
C           Expand the tree recursively.
  250       continue
            mtree=mtree+1
            do i=1,ntree
               if (tree(i,mtree-1).eq.0) goto 280
               do n=1,ntr
                  if (first(n).eq.tree(i,mtree-1) .and.
     +                 last(n).ne.tree(i,mtree-2)) then
                     do j=1,tmax
                        if (tree(j,1).eq.0) then
                           ntree=ntree+1
                           if (ntree.gt.tmax) then
                              print*,' ***** Increase TMAX!',ntree
                              stop
                           endif
                           do kk=1,mtree-1
                              tree(ntree,kk)=tree(i,kk)
                           enddo
                           tree(ntree,mtree)=last(n)
                           goto 270
                        endif
                     enddo
                  endif
  270             continue
               enddo
  280          continue
            enddo
C           Tree grown

C           Write tree information for the first links
            write(16,*) ' Cycle',icyc,mtree,ntree
            do k=1,ntree
               write(16,13) k,(tree(k,kk),kk=1,25)
            enddo

C           Look for rings
            do i=1,ntree
               do k=1,mtree-1
                  if (tree(i,mtree-1).ne.0 .and.
     +                 tree(i,k).eq.tree(i,mtree)) then
C                    Ring found
                     if (iprint.gt.4) print*,' found ',
     +                    (tree(i,kk),kk=1,mtree),100,i
                     do kk=k,mtree
                        test(kk-k+1)=tree(i,kk)
                     enddo
                     mm=mtree-k+1
C                    MM: number of ring elements+1 (element mm = element 1)
                     do kk=1,maxstp
                        tree(i,kk)=0
                     enddo

C                    Ring found, corresponding tree eliminated.
C                    Check whether the ring is already known.
                     call tstrng(test,ring,nring,nr,mm,0,jj)
C                    Output JJ=0: new ring
C                           JJ>0: old ring with number IKNOWN.
                     if (jj.gt.0) goto 500

C                    Test if the new ring has an internal connection
                     if (mtree.gt.7) then
                        do j=1,ntree
                           do k1=1,mtree
                              do k2=k1+1,mtree
                                 do n1=3,mm
                                    do n2=1,n1-2
                                       i1=n1-n2
                                       j1=n2+mm-n1-1
                                       if (j1.lt.i1) i1=j1
                                       if (tree(j,k1).eq.test(n1) .and.
     +                                      tree(j,k2).eq.test(n2)
     +                                      .and.i1.gt.k2-k1) then
                                          ico=ico+1
                                          goto 500
                                       endif
                                    enddo
                                 enddo
                              enddo
                           enddo
                        enddo
                     endif
C                    Add new ring
                     nr=nr+1
                     nring(nr)=mm
                     do kk=1,mm
                        ring(nr,kk)=test(kk)
                        ring(nr,kk+mm)=test(kk+1)
                     enddo
  500                continue
                  endif
               enddo
            enddo
            do i=1,nr
               if (iprint.gt.3) print 17,i,(ring(i,kk),kk=1,nring(i))
            enddo

            einde=0
            do k=1,ntree
               if (tree(k,mtree).ne.0) then
                  einde=1
               endif
            enddo
            if (mtree.lt.nsteps .and. einde.eq.1) goto 250
            if (iprint.gt.1) print 20,mtree,nsteps,icyc,nb(icyc)
         endif
      enddo
      if (iprint.gt.0) print*,' All steps performed.'

      if (nr.lt.1) goto 800
C     Sort rings to increasing numbers of knots
      call indexx(nr,nring,ind)
C     Ring k krijgt in het schema plaats ind(k)
      do k=1,nr
         i=ind(k)
         print 14,i,nring(i)-1,(ring(i,kk),kk=1,nring(i))
      enddo
      if (iprint.gt.0) print*,'Rings sorted to knots'

C     Expand trains to atoms, count number of atoms in each ring
      do i=1,nr
         nar=nar+1
         n=0
         do k=1,nring(i)-1
            do j=1,ntr
               if (first(j).eq.ring(i,k).and.
     +              last(j).eq.ring(i,k+1)) goto 520
            enddo
            print*,' Error 1',i
            stop
  520       continue
            do kk=1,ntrain(j)-1
               n=n+1
               atring(nar,n)=train(j,kk)
            enddo
         enddo
         atring(nar,n+1)=train(j,ntrain(j))
         natr(nar)=n+1
      enddo

  800 continue

C     Sort rings to increasing numbers of atoms
      if (nar.lt.1) then
         print*,' No rings found'
         stop
      endif
      call indexx(nar,natr,ind)
C     Ring k krijgt in het schema plaats ind(k)
      if (iprint.gt.0) print*,'Rings sorted to atoms'
C     Output to file RINGS.DEF
      open(unit=17,file='rings.def')
      write(17,18)nar
      print*,nar
      do k=1,nar
         i=ind(k)
         print 16,natr(i)-1,(atring(i,kk),kk=1,natr(i))
         write(17,16) natr(i)-1,(atring(i,kk),kk=1,natr(i))
      enddo

   12 format(' Train',i5,'  junctions ',20i4)
   13 format(i5,'  tree ',30i4)
   14 format(' Ring',i4,' has',i4,' knots: ',20i4)
   15 format(' Ring   ',i4,4x,100i4)
   16 format(i4,4x,100i4)
   17 format(i4,' Ring   ',30i4)
   18 format(i4,' rings')
   20 format(' End of cycle',i3,' with',i3,' steps in trial',
     +     i4,2x,i3,' bonds')
      end


      subroutine tstrng(test,ring,nring,nr,mm,iprint,iknown)
C     Test if the ring TEST has already been found in any RING.
C     NR: number of rings
C     MM: number od atoms in ring TEST
C     Output IKNOWN=0: new ring
C            IKNOWN>0: old ring with number IKNOWN.
      implicit none
      integer nr,mm,iprint,kk,j,is,n1,n2,nn,tmax,maxstp,iknown
      parameter(tmax=10000,maxstp=30)
      integer test(*),ring(tmax,maxstp),nring(*)


      if (iprint.gt.2) print*,' Starting tstrng',nr
      iknown=0
      if (nr.eq.0) return
      do kk=1,mm-1
         test(kk+mm)=test(kk+1)
      enddo
      if (iprint.gt.3) print 15,(test(kk),kk=1,2*mm-1)
      do j=1,nr
         if (nring(j).eq.mm) then
            do is=0,mm-1
               n1=0
               n2=0
               do kk=1,mm
                  nn=test(kk+is)
                  n1=n1+abs(ring(j,kk)-nn)
                  n2=n2+abs(ring(j,mm+1-kk)-nn)
               enddo
               if (n1.eq.0 .or. n2.eq.0) then
C                 This ring is already known
                  iknown=j
                  if (iprint.gt.3) print 18,n1,n2,j
                  return
               endif
            enddo
         endif
      enddo

      if (iprint.gt.3) print 19,nr,(test(kk),kk=1,mm)

   15 format(' Ring   ',20i4)
   18 format(' This ring is already known',2i5,' as number',i5)
   19 format(' Possible ring, number',i5,',  atoms',20i4)
      end


      subroutine indexx(n,arrin,indx)
C     Numerical Recipes p. 233
      integer arrin(n),indx(n)
      do 11 j=1,n
         indx(j)=j
   11 continue
      if (n.eq.1) return
      l=n/2+1
      ir=n
   10 continue
      if (l.gt.1) then
         l=l-1
         indxt=indx(l)
         q=arrin(indxt)
      else
         indxt=indx(ir)
         q=arrin(indxt)
         indx(ir)=indx(1)
         ir=ir-1
         if (ir.eq.1) then
            indx(1)=indxt
            return
         endif
      endif
      i=l
      j=l+l
   20 continue
      if (j.le.ir) then
         if (j.lt.ir) then
            if (arrin(indx(j)).lt.arrin(indx(j+1))) j=j+1
         endif
         if (q.lt.arrin(indx(j))) then
            indx(i)=indx(j)
            i=j
            j=j+j
         else
            j=ir+1
         endif
         goto 20
      endif
      indx(i)=indxt
      goto 10
      end
