      program codes
C     B. P. van Eijck, October 1997.
C     Makes coded topology file for UPACK programs:
C        every force field parameter is coded (codes 100-899).
C     In the UPACK programs these code numbers specify force field parameters
C        to be found from a given force field file.

C     Input files: IN.CON (unit 18), molecular connectivity
C                  IN.COD (unit 19), standard list of codes
C     Output file: OUT.TP (unit 20), framework for topology file: editing
C                  may be required.
C---------------------------------------------------------------------------
C     IN.CON (unit 18): molecular connectivity and coded charges, free format
C     - TITLE: text (a80)
C     - NRP: number of atoms in molecule
C        For i=1...NRP
C         - PANM(i): atom name (freely chosen, but starting with C, H, O...)
C           TYPE(i): atom code, corresponding with NB.COD
C           MCG(i): atomic charge codes (500-599)
C                   A negative MCG(i) enotes the end of a charge group.
C                   If MCG is simply +1 or -1, the codes are sought in IN.COD.
C     - I1,I2,I3.....: numbers of connected atoms
C           Repeat ad libitum: each line forms a chain of connected atoms.
C           Bonds may occur more than once.
C     - NDIHZ: number of dihedral angles that determine the separate
C              conformations
C       For every i=1...NDIHZ:
C        -  I1,I2,I3,I4: atomic numbers to define the dihedral
C     - NDIHH: number of less important dihedrals
C       For every i=1...NDIHH:
C       -  I1,I2,I3,I4: atomic numbers to define the dihedral
C               convention: rotate I1, not I4.
C----------------------------------------------------------------------------
C     IN.COD (unit 19): force field codes, free format
C     The following blocks (=) must be separated by a blank line.
C     Atom types are denoted by A1, A2,...
C     =   text
C     =   A1, atomic mass, polarizability code
C            Codes 800-899
C     =   A1, A2, bond code
C            Codes 100-199
C     =   A1, A2, A3, angle code
C            Codes 200-299
C     =   A1, improper dihedral code
C            Codes 300-399; chirality choices must be made manually!
C     =   A1, A2, A3, A4, torsional code (symbol X allowed for A1 and A4)
C            Codes 400-499
C---------------------------------------------------------------------------
C     OUT.TP (unit 20):
C     Hopefully the contents are self-explaining.
C     Situations requiring manual adjustment are indicated by ***** on
C        the screen and partly also in the output file:
C     For united atoms chirality choices must be made.
C     In aromatic rings the proper dihedrals must be changed into
C        improper ones.
C     The program tries to find reference atoms C1, C2, C3 to define
C        rotational matrices; if necessary, the numbers of missing
C        reference atoms must be provided.
C***************************************************************************
      implicit none
      integer i,i1,i2,ii1,ii2,ine,ine14,ios,ioutd,it,j,jt,k,ki,kj,
     +     kne,kne14,kt,l,l1,l2,l3,l4,lnang,lnat,lnbo,lncg,lndih,
     +     lnimp,lnvdw,lt,m,maxang,maxbon,maxcga,maxcgl,maxdih,
     +     maximp,maxnrp,maxnrt,maxome,mul,mxangl,mxbonl,
     +     mxdihl,mximpl,n,n1,n12,n2,nang,nbo,nbonh,ncag,ndef,ndih,
     +     ndihh,ndihz,nerr,nhit,nimp,nphih,nqhih,nratt,nrp,nt,ntheh,
     +     nword
      parameter(maxbon=500,maxang=1500,maximp=100,maxdih=3000,
     +     maxnrp=200,maxome=50,maxnrt=15)
      parameter(mxbonl=100,mxangl=250,mximpl=80,mxdihl=200,
     +     maxcga=20,maxcgl=100)
      double precision amass(maxnrp)
      integer bo(0:3,maxbon),ang(0:4,maxang),dih(0:5,maxdih),
     +     jsne(maxnrp),jsne14(maxnrp),boma(maxnrp,maxnrp),
     +     iac(maxnrp),icg(maxnrp),imp(0:5,maximp),ntype(maxnrt),
     +     lbo(2,mxbonl),lang(3,mxangl),limp(4,mximpl),ldih(4,mxdihl),
     +     mb(mxbonl),mh(mxangl),mi(mximpl),mt(mxdihl),
     +     mult(maxdih),inc(0:maxnrp),iexcl(maxnrp,maxnrp),scr(maxcga),
     +     def(2,maxdih),mtout(maxdih),mcg(maxnrp),inb(40),ireftr(12),
     +     mvdw(maxnrt,maxnrt),mpol(maxnrp),imol(maxnrp),
     +     iome(maxome),jome(maxome),kome(maxome),lome(maxome),
     +     cgimp(maxcga,maxcgl),mcgl(maxcga,maxcgl),ncg(maxcgl)
      character*5 naam,type(maxnrt),panm(maxnrp),
     +     lnaam(0:maxnrp),atna,code(maxnrp)
      character*80 regel,titel

      do i=1,maxnrp
         do j=1,maxnrp
            boma(i,j)=0
         enddo
      enddo
      inc(0)=0
      lnat=0
      lnbo=0
      lnang=0
      lnimp=0
      lndih=0
      lncg=0
      lnvdw=0
      lnaam(0)='X    '
      naam='     '

      open(18,file='in.con',status='OLD',iostat=ios)
      if (ios.ne.0) then
         print*,' ***** No file IN.CON'
         stop
      endif
      open(19,file='in.cod',status='OLD',iostat=ios)
      if (ios.ne.0) then
         print*,' ***** No file IN.COD'
         stop
      endif



C     Read connectivity file IN.CON
      print*,' Reading connectivity file:'
      read(18,10) titel
      print 80,titel

C     Read atom codes
      nratt=0
      read(18,*) nrp
      print 25,nrp,'atoms'
      if (nrp.gt.maxnrp) then
         print*,' ***** Too many atoms',maxnrp
         stop
      endif
      do i=1,nrp
         imol(i)=i
         read(18,10) regel
         print 10, regel
         if (nword(regel).lt.3) print*,' **** Error '
         call strext(regel,1,panm(i))
         call strext(regel,2,code(i))
         call intstr(regel,3,mcg(i))
      enddo
      mcg(nrp)=-abs(mcg(nrp))
C     Read bond connectivity
      nbo=0
      nbonh=0
  140 read(18,10,end=900) regel
      n=nword(regel)
      if (n.eq.0) goto 900
      if (n.eq.1) goto 160
      call intstr(regel,1,inb(1))
      do k=2,n
         call intstr(regel,k,inb(k))
         if (inb(k).lt.0 .or. inb(k).gt.nrp) then
            print*,' ***** Error reading atom numbers for bonds'
            stop
         endif
         i=inb(k-1)
         j=inb(k)
         if (boma(i,j).eq.1) goto 150
         nbo=nbo+1
         if (nbo.gt.maxbon) then
            print*,' ***** Too many bonds', nbo
            stop
         endif
         if (i.eq.j) then
            print*,' ***** Equal atoms in bond list',i
            stop
         endif
         boma(i,j)=1
         boma(j,i)=1
         bo(0,nbo)=0
         bo(1,nbo)=i
         bo(2,nbo)=j
         if (code(i)(1:1).eq.'H' .or. code(j)(1:1).eq.'H') bo(0,nbo)=1
         nbonh=nbonh+bo(0,nbo)
  150    continue
      enddo
      goto 140
C     boma(i,j)=1 if bond i-j is present

  160 continue
C     Read special dihedrals
      call intstr(regel,1,ndihz)
      do i=1,ndihz
         read(18,*) iome(i),jome(i),kome(i),lome(i)
      enddo
      read(18,*) ndihh
      do i=ndihz+1,ndihz+ndihh
         read(18,*) iome(i),jome(i),kome(i),lome(i)
      enddo



C     Read code file IN.COD
      print*,' Reading list of codes:'
      read(19,10,err=950,end=950) regel
      print 80,regel
  172 continue
      read(19,10,err=950,end=950) regel
      n=nword(regel)
      if (n.eq.0) goto 172

C     Read atom types, masses and polarization codes
  175 continue
      lnat=lnat+1
      call strext(regel,1,lnaam(lnat))
      call realst(regel,2,amass(lnat))
      call intstr(regel,3,mpol(lnat))
      read(19,10) regel
      n=nword(regel)
      if (n.eq.0) goto 180
      if (n.ne.3) then
         print*,' ***** Inconsistent atom type parameters'
         stop
      endif
      goto 175
  180 continue

      do i=1,maxnrt
         ntype(i)=0
      enddo
      do i=1,nrp
         do k=1,lnat
            if (code(i).eq.lnaam(k)) goto 200
         enddo
         naam(1:2)=code(i)(1:2)
         print*,' ***** Atom type ',naam,' not found, available are:'
         do k=1,lnat
            print*,k,'   ---',lnaam(k),'---'
         enddo
         print*,' Change atom type or add type ',naam,' to .COD file'
         stop
  200    continue
         do j=1,nratt
            if (ntype(j).eq.k) then
               iac(i)=j
               goto 220
            endif
         enddo
         nratt=nratt+1
         if (nratt.gt.maxnrt) then
            print*,' ***** Too many atom types',nratt
            stop
         endif
         type(nratt)=code(i)
         ntype(nratt)=k
         iac(i)=nratt
  220    continue
      enddo
      print 25,nratt,'atom types in this molecule'
      print*,'      Reference data in list of codes:'
      print*,lnat,' atoms'

C     Read van der Waals parameters
  250 continue
      read(19,10) regel
      if (nword(regel).eq.0) goto 250
      call intstr(regel,3,m)
      ioutd=0
      if (m/100.eq.6) then
         print*,' ***** Outdated version of file IN.COD'
         write(16,*) ' ***** Outdated version of file IN.COD' 
         ioutd=1
         goto 260
      endif
      if (m/100.eq.1) goto 310
      print*,' Error reading IN.COD',m
      stop
  260 continue
      n=nword(regel)
      if (n.eq.0) then
         print*,lnvdw,' nonbonded pairs'
         goto 300
      endif
      if (n.ne.3) then
         print*,' ***** Inconsistent van der Waals parameters'
         print*,regel
         stop
      endif
      lnvdw=lnvdw+1
      do m=1,2
         call strext(regel,m,atna)
         do k=1,nratt
            if (atna.eq.type(k)) then
               scr(m)=k
               goto 270
            endif
         enddo
         read(19,10) regel
         goto 260
  270    continue
      enddo
      ii1=scr(1)
      ii2=scr(2)
      if (ii1.lt.ii2) then
         ii1=scr(2)
         ii2=scr(1)
      endif
      call intstr(regel,3,mvdw(ii1,ii2))
      read(19,10) regel
      goto 260
      
C     Read bond parameters
  300 continue
      read(19,10) regel
      if (nword(regel).eq.0) goto 300
      call intstr(regel,3,m)
      if (m/100.eq.1) goto 310
      print*,' Error reading IN.COD',m
      stop
  310 continue
      n=nword(regel)
      if (n.eq.0) then
         print*,lnbo,' bonds'
         goto 330
      endif
      if (n.ne.3) then
         print*,' ***** Inconsistent bond type parameters'
         print*,regel
         stop
      endif
      lnbo=lnbo+1
      do m=1,2
         call strext(regel,m,atna)
         do i=0,lnat
            if (atna.eq.lnaam(i)) then
               lbo(m,lnbo)=i
               goto 320
            endif
         enddo
         print*,' ***** Code not found: ',atna
         print*,regel
         stop
  320    continue
      enddo
      call intstr(regel,3,mb(lnbo))
      read(19,10) regel
      goto 310

C     Read angle parameters
  330 continue
      read(19,10) regel
      if (nword(regel).eq.0) goto 330
      call intstr(regel,4,m)
      if (m/100.eq.2) goto 340
      print*,' Error reading IN.COD',m
      stop
  340 continue
      n=nword(regel)
      if (n.eq.0) then
         print*,lnang,' angles'
         goto 360
      endif
      if (n.ne.4) then
         print*,' ***** Inconsistent angle type parameters'
         print*,regel
         stop
      endif
      lnang=lnang+1
      do m=1,3
         call strext(regel,m,atna)
         do i=0,lnat
            if (atna.eq.lnaam(i)) then
               lang(m,lnang)=i
               goto 350
            endif
         enddo
         print*,' ***** Code not found: ',atna
         print*,regel
         stop
  350    continue
      enddo
      call intstr(regel,4,mh(lnang))
      read(19,10) regel
      goto 340


C     Read improper dihedral parameters
  360 continue
      read(19,10) regel
      n=nword(regel)
      if (n.eq.0) goto 360
      call intstr(regel,n,m)
      if (m/100.eq.3) goto 370
      print*,' Error reading IN.COD',m
      stop
  370 continue
      n=nword(regel)
      if (n.eq.0) then
         print*,lnimp,' improper dihedrals'
         goto 390
      endif
      if (n.ne.5 .and. n.ne.2) then
         print*,' ***** Inconsistent improper type parameters'
         print*,regel
         stop
      endif
      lnimp=lnimp+1
      do m=1,n-1
         call strext(regel,m,atna)
         do i=0,lnat
            if (atna.eq.lnaam(i)) then
               limp(m,lnimp)=i
               goto 380
            endif
         enddo
         print*,' ***** Code not found: ',atna
         print*,regel
         stop
  380    continue
      enddo
      call intstr(regel,n,mi(lnimp))
      read(19,10) regel
      goto 370

C     Read proper dihedral-parameters
  390 continue
      read(19,10) regel
      if (nword(regel).eq.0) goto 390
      call intstr(regel,5,m)
      if (m/100.eq.4) goto 400
      print*,' Error reading IN.COD',m
      stop
  400 continue
      n=nword(regel)
      if (n.eq.0) goto 420
      if (n.ne.5) then
         print*,' ***** Inconsistent torsional type parameters'
         print*,regel
         stop
      endif
      lndih=lndih+1
      do m=1,4
         call strext(regel,m,atna)
         do i=0,lnat
            if (atna.eq.lnaam(i)) then
               ldih(m,lndih)=i
               goto 410
            endif
         enddo
         print*,' ***** Code not found: ',atna
         print*,regel
         stop
  410    continue
      enddo
      call intstr(regel,5,mt(lndih))
      read(19,10,err=420,end=420) regel
      goto 400
  420 continue
      print*,lndih,' torsional angles'

C     Read charge groups
  430 continue
      read(19,10,err=460,end=460) regel
      if (nword(regel).eq.0) goto 430
  440 continue
      n1=nword(regel)
      if (n1.gt.maxcga) then
         print*,' ***** Increase MAXCGA'
         stop
      endif
      if (n1.gt.0) then
         lncg=lncg+1
         if (lncg.gt.maxcgl) then
            print*,' ***** Increase MAXCGL'
            stop
         endif
         do m=1,n1
            call strext(regel,m,atna)
            do i=0,lnat
               if (atna.eq.lnaam(i)) then
                  cgimp(m,lncg)=i
                  goto 450
               endif
            enddo
            print*,' ***** Code not found: ',atna
            print*,regel
            stop
  450       continue
         enddo
         read(19,10,end=460) regel
         n2=nword(regel)
         if (n2.ne.n1) then
            print*,' ***** Wrong charge group information on ALL.COD'
            print*,regel
            stop
         endif
         ncg(lncg)=n1
         do m=1,n1
            call intstr(regel,m,mcgl(m,lncg))
         enddo
         read(19,10,end=460) regel
         goto 440
      endif
  460 continue
      print*,lncg,' charge groups'
      print*,' End reading list of codes'
      close(19)
C     End reading file IN.COD


C     Make angles
      nang=0
      ntheh=0
      do i=1,nrp
         do l=1,nrp
            if (boma(i,l).eq.1) then
               do j=l+1,nrp
                  if (boma(i,j).eq.1) then
                     nang=nang+1
                     if (nang.gt.maxang) then
                        print*,' ***** Too many angles',nang,maxang
                        stop
                     endif
                     ang(1,nang)=l
                     ang(2,nang)=i
                     ang(3,nang)=j
                     ang(0,nang)=0
                     if (type(iac(l))(1:1).eq.'H' .or. type(iac(j))(1:1)
     +                    .eq.'H') ang(0,nang)=1
                     ntheh=ntheh+ang(0,nang)
                  endif
               enddo
            endif
         enddo
      enddo

C     Make improper dihedrals
      nimp=0
      nqhih=0
      do i=1,nrp
         if (nimp.ge.maximp) then
            print*,' ***** Too many improper dihedrals',nimp
            stop
         endif
         imp(0,nimp+1)=0
         imp(1,nimp+1)=i
         n=0
         do j=1,nrp
            if (boma(i,j).eq.1) then
               n=n+1
cc               print*,i,j,n
               imp(n+1,nimp+1)=j
               if (type(iac(j))(1:1).eq.'H') imp(0,nimp+1)=1
            endif
         enddo
cc         if (n.gt.3) print 23,n,i
         if (n.eq.3) then
            nimp=nimp+1
            nqhih=nqhih+imp(0,nimp)
         endif
      enddo

C     Make proper dihedrals
      ndih=0
      nphih=0
      ndef=0
      do i=1,nrp
         do l=i,nrp
            mul=0
            if (boma(i,l).eq.1) then
               do j=1,nrp
                  if (boma(i,j).eq.1 .and. j.ne.l) then
                     do k=1,nrp
                        if (boma(k,l).eq.1 .and. k.ne.i
     +                       .and. k.ne.j) then
                           nhit=0
                           ndef=ndef+1
                           mul=mul+1
                           def(1,ndef)=j
                           def(2,ndef)=k
                           kt=ntype(iac(k))
                           lt=ntype(iac(l))
                           it=ntype(iac(i))
                           jt=ntype(iac(j))
                           if (type(iac(k))(1:1).eq.'H' .or.
     +                          type(iac(j))(1:1).eq.'H') nphih=nphih+1
                           do m=1,lndih
                              l1=ldih(1,m)
                              l2=ldih(2,m)
                              l3=ldih(3,m)
                              l4=ldih(4,m)
                              if ((l1.eq.kt .and. l2.eq.lt .and. 
     +                             l3.eq.it .and. l4.eq.jt) .or.
     +                            (l1.eq.jt .and. l2.eq.it .and.
     +                             l3.eq.lt .and. l4.eq.kt)) then
                                 nhit=nhit+1
                                 ndih=ndih+1
                                 if (ndih.gt.maxdih) goto 940
                                 mtout(ndih)=mt(m)
                                 dih(1,ndih)=k
                                 dih(2,ndih)=l
                                 dih(3,ndih)=i
                                 dih(4,ndih)=j
                              endif
                           enddo
C                          Search for X  ATOM  ATOM  X
                           if (nhit.eq.0) then
                              do m=1,lndih
                                 l1=ldih(1,m)
                                 l2=ldih(2,m)
                                 l3=ldih(3,m)
                                 l4=ldih(4,m)
                                 if ((l1.eq.0  .and. l2.eq.lt .and. 
     +                                l3.eq.it .and. l4.eq.0) .or.
     +                                (l1.eq.0  .and. l2.eq.it .and. 
     +                                l3.eq.lt .and. l4.eq.0)) then
                                    nhit=nhit+1
                                    ndih=ndih+1
                                    if (ndih.gt.maxdih) goto 940
                                    mtout(ndih)=mt(m)
                                    dih(1,ndih)=k
                                    dih(2,ndih)=l
                                    dih(3,ndih)=i
                                    dih(4,ndih)=j
                                 endif
                              enddo
                           endif
                           if (nhit.eq.0) then
C                             Dihedral not found, but still registered
                              ndih=ndih+1
                              if (ndih.gt.maxdih) goto 940
                              dih(1,ndih)=k
                              dih(2,ndih)=l
                              dih(3,ndih)=i
                              dih(4,ndih)=j
                              mult(ndih)=0
                              mtout(ndih)=0
                           endif
                        endif
                     enddo
                  endif
               enddo
               do j=1,ndih
                  if (mult(j).eq.0) mult(j)=mul
               enddo
            endif
         enddo
      enddo
C     The force constant can later be divided by the multiplicity MULT.

C     Count excluded neighbors
      kne=0
      kne14=0
      ncag=0
      do i=1,nrp
         ine=0
         do j=i,nrp
            iexcl(i,j)=3
            if (boma(i,j).eq.1) then
               ine=ine+1
               jsne(ine)=j
            endif
         enddo
         do j=1,nang
            k=ang(3,j)
            l=ang(1,j)
            if (k.eq.i .and. i.lt.l) then
               ine=ine+1
               jsne(ine)=l
            endif
            if (l.eq.i .and. i.lt.k) then
               ine=ine+1
               jsne(ine)=k
            endif
            l=ine-1
            do m=1,l
               if (jsne(m).eq.jsne(ine)) ine=l
            enddo
         enddo
         icg(i)=0
         if (mcg(i).lt.0) icg(i)=1
         kne=kne+ine
         if (icg(i).eq.1) then
            ncag=ncag+1
            inc(ncag)=i
         endif
         do m=1,ine
            iexcl(i,jsne(m))=1
         enddo

C        Count 1..4 interacties
         ine14=0
         do j=1,ndef
            k=def(1,j)
            l=def(2,j)
            m=ine14
            if (k.eq.i .and. i.lt.l) then
               ine14=ine14+1
               jsne14(ine14)=l
            endif
            if (l.eq.i .and. i.lt.k) then
               ine14=ine14+1
               jsne14(ine14)=k
            endif
            if (m.eq.ine14) goto 510
            do l=1,m
               if (jsne14(l).eq.jsne14(ine14)) then
                  ine14=m
                  goto 510
               endif
            enddo
            do l=1,ine
               if (jsne(l).eq.jsne14(ine14)) then
                  ine14=m
                  goto 510
               endif
            enddo
  510       continue
         enddo
         kne14=kne14+ine14
         do m=1,ine14
            iexcl(i,jsne14(m))=2
         enddo
      enddo
      
C     Check for disconnected sets of atoms
      do i=2,nrp
         do j=1,i-1
            if (boma(i,j).eq.1) goto 610
         enddo
         do k=i+1,nrp
            do j=1,i-1
               if (boma(j,k).eq.1) goto 610
            enddo
         enddo
C        This atom is not connected to previous ones and starts a new molecule.
         print 51,i
         imol(i-1)=-(i-1)
C        Change IEXCL to zero for intermolecular atom pairs
         do k=i,nrp
            do j=1,i-1
               iexcl(j,k)=0
            enddo
         enddo
  610    continue
      enddo
c      imol(nrp)=-nrp

C     Search for known charge groups
      if (icg(nrp).ne.1) print*,' ***** Charge group not closed'
      if (lncg.gt.0) then
         do j=1,ncag
            i1=inc(j-1)
            i2=inc(j)
            n12=i2-i1
            do k=1,lncg
               if (ncg(k).eq.n12) then
                  do m=1,n12
                     if (cgimp(m,k).ne.ntype(iac(i1+m))) goto 620
                     scr(m)=mcgl(m,k)
                  enddo
                  goto 630
  620             continue
                  do m=1,n12
                     if (cgimp(n12+1-m,k).ne.ntype(iac(i1+m))) goto 640
                     scr(m)=mcgl(n12+1-m,k)
                  enddo
  630             continue
C                 Charge group found in IN.COD
                  scr(n12)=-scr(n12)
                  do m=1,n12
                     ii1=mcg(i1+m)
                     if (abs(ii1).ge.500 .and. abs(ii1).lt.600) then
                        if (scr(m).ne.ii1) then
C                          Replace charge group code
                           print 24,scr(m),i1+m,panm(i1+m),j,ii1
c                           print*,(lnaam(cgimp(m,k)),m=1,n12)
c                           print*,(scr(m),m=1,n12)
c                           print*,(mcg(i1+m),m=1,n12)
c                           pause
                        endif
                     else
                        mcg(i1+m)=scr(m)
                     endif
                  enddo
                  goto 650
  640             continue
               endif
            enddo
C           Charge group not found in IN.COD
            print 26,j,(code(i),i=i1+1,i2)
c            pause
            do m=1,n12
               ii1=mcg(i1+m)
               if (abs(ii1).lt.500 .or. abs(ii1).gt.600) then
                  print 28,ii1,i1+m,panm(i1+m)
cc                  stop
               endif
            enddo
  650       continue
         enddo
      endif
      
      do i=1,nrp
         m=abs(mcg(i)/100)
         if (m.ne.5) mcg(i)=599
      enddo

C     Write output to topology file
      open(20,file='out.tp')
      write(20,80) titel
      if (ioutd.eq.1) then
         write(20,25) nratt,'atom types:'
         write(20,15) (type(i),i=1,nratt)
         write(20,*) '  Van der Waals codes:'
         do i=1,nratt
            write(20,72) (mvdw(i,j),j=1,i)
         enddo
      endif
      write(20,25) nrp,'atoms in a molecule'
      write(20,74)
      do i=1,nrp
         j=iac(i)
         nt=ntype(j)
         write(20,76) imol(i),panm(i),j,type(j),
     +        amass(nt),abs(mcg(i)),mpol(nt)
      enddo

      write(20,*) ' Exclusion matrix: 0: intermolecular interaction'
      write(20,*) '   Intramolecular: 1: 1-2,1-3;   2: 1-4;   3: 1-5,..'
      do i=1,nrp
         iexcl(i,i)=1
         do j=1,i
            iexcl(i,j)=iexcl(j,i)
         enddo
      enddo
      do i=1,nrp
         write(20,'(25i3)') (iexcl(j,i),j=1,i)
      enddo
      write(20,25) ncag,'charge groups, ending at:'
      write(20,'(15i5)') (inc(i),i=1,ncag)

C     Write bonds
      write(20,25) nbo,'bonds'
      print*,nbonh,'   nbonh = bond with H'
      print*,nbo-nbonh,'   nbon  = bonds without H'
      naam=' '
      nerr=0
      do k=1,nbo
         ki=ntype(iac(bo(1,k)))
         kj=ntype(iac(bo(2,k)))
         do l=1,lnbo
            if (ki.eq.lbo(1,l) .and. kj.eq.lbo(2,l)) then
               write(20,'(2i5,i10,a5)') bo(1,k),bo(2,k),mb(l),naam
               goto 690
            endif
            if (ki.eq.lbo(2,l) .and. kj.eq.lbo(1,l)) then
               write(20,'(2i5,i10,a5)') bo(2,k),bo(1,k),mb(l),naam
               goto 690
            endif
         enddo
         nerr=nerr+1
         write(20,'(2i5,i10,a7)') (bo(m,k),m=1,2),0,'  *****'
  690    continue
      enddo
      if (nerr.ne.0)
     +     print*,' ***** ',nerr,' bond-types not found'

C     Write angles
      write(20,25) nang,'angles'
      print*,ntheh,'   ntheh = angles with H'
      print*,nang-ntheh,'   nthe  = angles without H'
      nerr=0
      do k=1,nang
         ki=ntype(iac(ang(1,k)))
         kj=ntype(iac(ang(3,k)))
         do l=1,lnang
            if (ntype(iac(ang(2,k))).ne.lang(2,l)) goto 700
            if ((ki.eq.lang(1,l) .and. kj.eq.lang(3,l)) .or.
     +           (ki.eq.lang(3,l) .and. kj.eq.lang(1,l))) then
               write(20,'(3i5,i10,a5)') (ang(m,k),m=1,3),mh(l),naam
               goto 710
            endif
  700       continue
         enddo
C        Search for entries X  ATOM  X
         do l=1,lnang
            if (ntype(iac(ang(2,k))).ne.lang(2,l)) goto 705
            if (lang(1,l).eq.0 .and. lang(3,l).eq.0) then
               write(20,'(3i5,i10,a5)') (ang(m,k),m=1,3),mh(l),naam
               goto 710
            endif
  705       continue
         enddo 
         nerr=nerr+1
         write(20,'(3i5,i10,a7)') (ang(m,k),m=1,3),0,'  *****'
  710    continue
      enddo
      if (nerr.ne.0)
     +     print*,' ***** ',nerr,' angle-types not found'

C     Write improper dihedrals
      write(20,25) nimp,'improper dihedrals'
      if (nimp.ne.0) then
         print*,' ***** Check assignment of',nimp,' improper dihedrals'
         write(20,*) '***** Change code if needed!'
      endif
      print*,nqhih,'   nqhih = improper dihedrals with H'
      print*,nimp-nqhih,'   nqhi  = improper dihedrals without H'
      nerr=0
      do k=1,nimp
         ki=ntype(iac(imp(1,k)))
         do l=1,lnimp
            if (ki.eq.limp(1,l)) then
               write(20,'(4i5,i10,a9)') (imp(m,k),m=1,4),
     +              mi(l),'  *****'
               goto 750
            endif
         enddo
         nerr=nerr+1
         write(20,'(4i5,i10,a7)') (imp(m,k),m=1,4),0,'  *****'
  750    continue
      enddo
      if (nerr.ne.0)
     +     print*,' ***** ',nerr,' improper-types not found'

C     Write proper dihedrals
      write(20,25) ndih,'torsions'
      print 29
      write(20,29)
      print*,nphih,'   nphih = torsions with H'
      print*,ndih-nphih,'   nphi  = torsions without H'
      nerr=0
      do k=1,ndih
         if (mtout(k).gt.0) then
            write(20,'(4i5,i10,i5)') (dih(m,k),m=1,4),
     +           mtout(k),mult(k)
         else
            nerr=nerr+1
            write(20,'(4i5,i10,i5,a7)')
     +           (dih(m,k),m=1,4),0,mult(k),'  *****'
         endif
      enddo
      if (nerr.ne.0)
     +     print*,' ***** ',nerr,' dihedral-types not found'

C     Write information for special dihedrals
      write(20,25) ndihz,'essential dihedrals'
      do i=1,ndihz
         write(20,73) iome(i),jome(i),kome(i),lome(i)
      enddo
      write(20,25) ndihh,'unimportant dihedrals'
      do i=ndihz+1,ndihz+ndihh
         write(20,73) iome(i),jome(i),kome(i),lome(i)
      enddo

C     Determine and write numbers of reference atoms
c      read(18,*,end=800,err=800) (ireftr(i),i=1,3)
c      print*, ' Reference atoms for rotation matrices:'
      read(18,10,end=800,err=800) regel
      n2=nword(regel)
      do i=1,n2
         call intstr(regel,i,ireftr(i))
      enddo
      print 56, (ireftr(i),i=1,n2)
      goto 770
  800 continue
      do i=1,3
         ireftr(i)=0
      enddo
      do i=1,nrp
         if (panm(i)(1:2).eq.'C1') ireftr(1)=i
         if (panm(i)(1:2).eq.'C2') ireftr(2)=i
         if (panm(i)(1:2).eq.'C3') ireftr(3)=i
      enddo
  770 continue
      write(20,*) ' Reference atoms for rotation matrices:'
      write(20,56) (ireftr(i),i=1,n2)
      do i=1,3
         if (ireftr(i).eq.0) then
            print*,' ***** Reference atom',i,' not found!',
     +           '       Change corresponding zero on topology file'
            write(20,*) ' ***** Reference atom',i,' not found!'
         endif
      enddo
      stop

  900 print*,' ***** No data on special dihedrals'
      stop
  940 print*,' ***** Too many dihedrals',ndih
      stop
  950 print*,' ***** Error on force field file'
      stop

   10 format(a80)
   15 format(3x,16a6)

   24 format('  ***** Reference code',i5,' for atom',i4,2x,a6,
     +     ' in charge group',i4,' replaced by',i5)
   25 format(i5,2x,a)
   26 format('  Charge group',i4,' not in reference list:',
     +     ' atom codes ',40a5)
   28 format('  ***** Code',i5,' for atom',i4,2x,a6,' is not useful')
   29 format
     +     (' Note redundancy between aromatic torsions and impropers!')
   51 format('  ***** Note: Atom',i4,' is not connected to ',
     +     'previous ones and starts a new molecule.')
c   55 format(5i5,a5)
   56 format(4(3i5,5x))
   72 format(20i6)
   73 format(4i5,3x,20i5)
   74 format(7x,'Name',8x,'Type',9x,'Mass',5x,'Ch-cod',2x,'Pol-cod')
   76 format(i4,4x,a5,i6,4x,a5,f10.5,3i8)
   80 format(1x,a80)
      end
