      subroutine congra(n,par,func,nsd,tol,nummer,
     +     step0,deprls,shimax,iprint,emin,ncyc,rmsder)

C     W. T. M. Mooij and B. P. van Eijck, February 1998.
C     Performs conjugate gradient energy minimization of function FUNC
C     with respect to N parameters PAR.
C     Calls subroutine FUNC(n,par,g,en,isubr):
C        EN delivers the value of the function
C        G delivers the negative derivatives to the N parameters PAR
C        ISUBR=1: make pair list(s)
C             =2: use previously made pair list(s)
C     CONGRA calls FUNC first with ISUBR=1. During one minimization
C     no new pair lists must be made to ensure stability (ISUBR=2).
C     If the pair list is updated, discontinuities arise and the
C     derivatives become unreliable.

C     Input:
C        N: number of parameters
C        PAR(1..N): the parameters
C        NSD: maximum number of cycles
C        TOL: stop earlier if RMSDER < TOL
C        NUMMER: structure number (for error messages)  
C        STEP0: initial step in line minimization (see below)
C        DEPRLS: if DEPRLS > 0, test whether a new pair list changes the
C                minimized energy more than DEPRLS. If so, restart.
C        SHIMAX: make a new pair list and restart if any parameter
C                deviates more than SHIMAX from its starting value.
C        IPRINT: regulates print output to screen and unit 16.
C        HALT (unit 15) stops calculation manually
C     Input through common block /AIDT/:
C        NCYCGA: the number of the ab-initio cycle
C        MAXCYC: maximum number of ab-initio cycles
C        IREDO>0: shift too large for parameter IREDO
C             <0: MAXCYC exceeded

C     Output:
C        EMIN: minized energy
C        NCYC: number of cycles
C        PAR(1..N): optimized parameters
C        RMSDER: rms value of G(i), the negative derivatives of EMIN to PAR.
C     Output through common block /AIDT/:
C        IAI=0: calculate intramolecular energy from default force field
C           =1: calculate intramolecular energy and derivatives from GAMESS.
C           =2: calculation through previous call with IAI=1

C     The subroutine relies on a fairly independent piece of code:
C     LINE MINIMIZATION. This will search for the minimium of FUNC in a
C     given search direction ZR, using the negative derivatives G.
C     It starts with given energy EOLD and produces the minimized energy EMIN;
C     PAR is replaced by PAR1 and G by G1.
C     It starts with a given STEP in the direction ZR. To optimize STEP
C     this parameter is divided by FDIV after each minimization, but if the
C     energy minimum is not found within STEP, this parameter is multiplied
C     by FMULT and that multiplication may be repeated NITI times.
C     For optimum performance choose the initial STEP0 in such a way that
C     NITI equals 1 or 2 in the first cycle.
C     Shanno's convergence criteria for line minimization:
C        Component of G1 in de search direction must have decreased
C        sufficiently (criterion EPS1); the energy must have decreased
C        sufficiently (criterion EPS2) and may not be higher than the
C        starting values at the beginning and the end of STEP.

      implicit none
      include 'params.i'
      include 'caidt.i'

C     BFGS method requires stores of inverse Hessian; this requires a large 
C          array HESSIN. Other methods do not require this array.
c      parameter(mxbfgs=500,maxarr=max(mxbfgs,maxpar))

      external func
      double precision a,b,d0,d1,d1g1,d1gs,d2,d21,d3,d31,d4,d5,d6,d7,d8,
     +     deprls,derest,dnew,dold,ea,eb,emin,emin2,eold,eps1,eps2,
     +     fac,fad,fae,fdiv,fmult,g1g,g1g1,gamma,gamma1,gamma2,gnorm,hs
      integer i,icp,iperio,iprev,iprint,j,k,maxarr,mxbfgs,
     +     n,ncyc,niti,npl,npow,nrstrt,nsd,nstijg,nummer
      double precision pg,pr,ptyt,py,rmsder,sg,shift,shim,shimax,
     +     shprev,step,step0,
     +     stpmin,sumd1,sumd2,sy,tol,w,x,ya,yb,yg,ytg,yy,z,zrnorm

      parameter(mxbfgs=maxpar,maxarr=maxpar)
      double precision par(*),g(maxarr),par1(maxarr),g1(maxarr),
     +     zr(maxarr),par0(maxarr),p(maxarr),y(maxarr),deltaE(100),
     +     av(maxarr),bv(maxarr),pt(maxarr),yt(maxarr),s(maxarr),
     +     hessin(mxbfgs,mxbfgs),dg(mxbfgs),hdg(mxbfgs)
      character*5 method
      logical reset

      if (iprint.gt.2) print 32,nsd,tol,step0,deprls,shimax
      if (nsd.eq.0) return
      if (n.le.0) return
      if (n.gt.maxarr) then
         print*,' ***** Increase MAXPAR or MXBFGS in CONGRA'
         stop
      endif

C     Choice of algorithms:
C     BFGS variable metric
      method='BFGS '
C     Polak-Ribiere conjugate gradients
c      method='PR   '
C     Shanno conjugate gradients
c      method='SHAN '
C     Hestenes-Stiefel
c      method='HS   '
C     Fletcher-Reeves conjugate gradients
c      method='FR   '
C     Shanno conjugate gradients, with powell-restarts
c      method='SH-PO'
C     Powell-Beale conjugate gradients ( TINKER LMQN-routine)
c      method='PO-BE'
C     Memoryless Quasi Newton (TINKER LMQN-routine)
c      method='MQN  '

      if (method.eq.'BFGS ') then
         if (n.gt.mxbfgs) then
            print*,' Number of atoms to large for BFGS method',n
            stop
         else
            if (iprint.gt.4) print*,' BFGS Variable Metric'
         endif
      endif
      if (iprint.gt.4) then
         if (method.eq.'PR   ')
     +        print*,' Polak-Ribiere conjugate gradients'
         if (method.eq.'SHAN ')
     +        print*,' Shanno conjugate gradients'
         if (method.eq.'HS   ')
     +        print*,' Hestenes-Stiefel conjugate gradients'
         if (method.eq.'FR   ')
     +        print*,' Fletches-Reeves conjugate gradients'
         if (method.eq.'SH-PO ')
     +        print*,' Shanno conjugate gradients, with Powell restarts'
         if (method.eq.'PO-BE ')
     +        print*,' Powell-Beale conjugate gradients'
      endif

C     SH-PO and PO-BE never restart with SD direction; they do a 'Powell'
C     restart if the search directions are not conjugate enough (Shanno's
C     criterium g1g > 0.2 g1g1), or in any case every NRSTRT iterations
C     (default n).

C     BFGS only restarts with SD every NRSTRT interations (defaults no
C     restart). Others restart only when Shanno's criterium is not satified.

c   90 continue
      iperio=0
      if (method.eq.'PO-BE') nrstrt=n
      if (method.eq.'SH-PO') nrstrt=n
      if (method.eq.'BFGS ') nrstrt=nsd

C     Default values
c      step0=0.005
c      deprls=0.3
c      shimax=0.02
c      step0=0.0005

      shim=shimax
      fmult=1.2d0
      fdiv=1.05d0
      eps2=0.0001d0
      eps1=0.9d0
      derest=0.4d0
      stpmin=step0/10000

C     When the gradient-norm is satisfied, test if energy does not change
C     more than DEPRLS upon making a new pair list
C     When the energy changes more than DEREST upon a making a new
C     pair list, the algoritm is restarted with the gradient vector.

      d0=step0
      ncyc=0
      npow=0
      shprev=0
      iprev=0
      if (iredo.eq.-1000) return

  100 continue
C     Make pair list, reset reference parameters to find the shifts
      call func(n,par,g,eold,1)
C     This implies calling GAMESS if IAI=1
      if (eold.gt.50000) then
         emin=eold
         if (iprint.gt.1) print 42,emin,nummer,1
         goto 710
      endif
      do i=1,n
         par0(i)=par(i)
      enddo

  110 continue
C     Restart search direction to the steepest descent direction
      npl=0
      dold=0
      do i=1,n
         zr(i)=g(i)
         dold=dold+zr(i)**2
      enddo

C     Initialize inverse hessian for BFGS variable metric
      if (method.eq.'BFGS ') then
         do i=1,n
            do j=1,n
               hessin(j,i)=0
            enddo
            hessin(i,i)=1
         enddo
      endif

      d1g1=-dold
      gnorm=sqrt(dold)
      step=d0/gnorm
      if (eold.ge.50000) step=step*100
      icp=7
      if (step.lt.stpmin) goto 700
      rmsder=sqrt(dold/n)
      if (iprint.gt.4) print 40, ncyc,rmsder,step,eold
      if (rmsder.lt.tol) then
C        Derivatives are small enough at (re)start CG
         emin=eold
         if (iprint.gt.1) print 43,nummer
         return
      endif

C     Start LINE MINIMIZATION.
C     Calculate energies (EA, EB) and derivatives (YA, YB) at points
C     A and B along the search direction, starting with A = 0 and B = STEP.
C     If necessary, adjust A and B to find an optimum search range.
C     Find a minimum X in the search range between A and B by determining
C     the minimum of the third-degree polynomial fixed by EA, EB, YA and YB.
  200 continue
      nstijg=0
      ya=0
      do i=1,n
         ya=ya-zr(i)*g(i)
      enddo
      if (ya.ge.0) then
C        No energy gain in the search direction
         if (iprint.gt.3) print*,' ***** ya positive: ',ya
         goto 110
      endif
      a=0
      ea=eold
      b=step
      niti=0
  230 continue
      do i=1,n
         par1(i)=par(i)+b*zr(i)
      enddo
      call func(n,par1,g1,eb,2)
      if (eb.gt.50000) then
         if (iprint.gt.0) print 42,eb,nummer,2
         goto 710
      endif
      yb=0
      do i=1,n
         yb=yb-zr(i)*g1(i)
      enddo

  260 continue

ccc      if (iprint.gt.3) print 50, niti,a,b,ea,eb,ya,yb
      if (yb.lt.0 .and. eb.lt.ea) then
C        There must be a lower minimum further away than B, so shift and
C        double the search range.
         w=b-a
         a=b
         b=a+2*w
         ea=eb
         ya=yb
         niti=niti+1
         goto 230
      endif
      if (eb.gt.ea+10000) then
C        Very asymmetric energy function, decrease search range
         b=(b+a)/2
         if (b-a.lt.stpmin) then
            if (iprint.gt.2) print*,' STEP in LINMIN too small'
            call func(n,par,g,emin,1)
            if (iprint.gt.2) print*,' Energy = ',emin
            if (emin.gt.50000 .and. iprint.gt.1) print 42,emin,nummer,3
            goto 710
         endif
         goto 230
      endif
c      if (abs(b-a).lt.1.0d-10) then
c         print*,' Too close b and a:',a,b
c         call func(n,par,g,emin,1)
c         print*,' Energy = ',emin
c         stop
c      endif
      
C     Now there must be a minimum X in the search range between A and B.
C     Find it and calculate the energy there.
      if (niti.ne.0) then
         step=fmult*step
         d0=fmult*d0
      endif
      z=3*(ea-eb)/(b-a)+ya+yb
      w=z**2-ya*yb
      if (w.le.0) then
         print*,' ***** CONGRA: w < 0!!! This should never happen.'
         write(16,*)' ***** CONGRA:  w < 0!!! This should never happen.'
         print 50, niti,a,b,ea,eb,ya,yb,w
         x=0
      else
         w=sqrt(w)
         x=b-(yb+w-z)/(yb-ya+2*w)*(b-a)
         if (iprint.gt.5) print 50, niti,a,b,ea,eb,ya,yb,w,x/(b-a)
      endif
      do i=1,n
         par1(i)=par(i)+x*zr(i)
         p(i)=x*zr(i)
      enddo
      if (iprint.gt.7) print 80,(par1(i),i=1,n)
      call func(n,par1,g1,emin,2)
      if (emin.gt.50000) then
         if (iprint.gt.1) print 42,emin,nummer,4
         goto 710
      endif
      d1gs=0
      do i=1,n
         d1gs=d1gs-zr(i)*g1(i)
      enddo

C     Check for convergence
      if ((emin.gt.eold+d1g1*x*eps2)
     +     .or.(abs(d1gs).gt.eps1*abs(d1g1))
     +     .or.(((emin.gt.ea) .or. (emin.gt.eb)).and.
     +     ((x.gt.1.01d0*a).and.(x.lt.0.99d0*b)))) then
         if (iprint.gt.4) then
            if (abs(d1gs).gt.eps1*abs(d1g1)) print 72,d1gs,d1g1
            if (emin.gt.eold+d1g1*x*eps2) then
               print 70,emin,eold,d1g1,x
               print 50, niti,a,b,ea,eb,ya,yb,w,x/(b-a)
            endif
            if (emin.gt.ea .or. emin.gt.eb) print 71,emin,ea,eb
         endif

         nstijg=nstijg+1
         if ((x*d1gs).ge.0) then
            b=x
            eb=emin
            yb=d1gs
         else
            a=x
            ea=emin
            ya=d1gs
         endif
         if (nstijg.lt.5) then
C           Try a next interation within the chosen range
            goto 260
         else
C           Restart with new pair list and search direction
            if (npl.ne.0) then
               if (iprint.gt.4) print 35
               goto 100
            endif
C           Unless we had already restarted, then decrease step and try again
            step=step/10
            icp=8
            if (step.lt.stpmin) goto 700
            d0=d0/10
            goto 200
         endif
      endif
      d0=d0/fdiv
C     end of LINE MINIMIZATION
C     Now we have found a lower energy

      ncyc=ncyc+1
C     Calculate RMS gradient, update parameters, calc G1G for new restart
C     Calculate parameter shift (for MQN method)
      g1g1=0
      do i=1,n
         g1g1=g1g1+g1(i)*g1(i)
         s(i)=(par1(i)-par(i))
         par(i)=par1(i)
      enddo
      rmsder=sqrt(g1g1/n)
      if (iprint.gt.4) print 60,niti,ncyc,rmsder,step,emin

      open(unit=87,file='HALT',status='OLD',err=270)
      close(87,status='DELETE')
      print*,' '
      print*,' Stopped because of manual intervention'
      return
  270 continue

      if (iredo.gt.0) then
         ncycga=ncycga+1
         if (ncycga.gt.maxcyc) then
            print*,' Too many ab initio cycles, premature end of CONGRA'
            return
         endif
         print 37,ncycga,emin
         write(16,37) ncycga,emin
         iai=1
         d0=step0
         goto 100
      endif

C     Check whether we are done:
C     1. Too many cycles performed:
      if (ncyc.ge.nsd) then
c         if (iprint.gt.0) then
c            print 36,ncyc
c            return
c         endif
         call func(n,par,g,emin,1)
         if (iprint.gt.1) print 36,ncyc,emin
         return
      endif
C     2. Gradient vector sufficiently small:
      if (rmsder.lt.tol) then
         if (deprls.lt.0) return
C        Check whether the pair list was still good:
         call func(n,par,g1,emin2,1)
         if (emin2.gt.50000) then
            emin=emin2
            if (iprint.gt.1) print 42,emin,nummer,5
            goto 710
         endif
C        Normal end of CONGRA
         if (abs(emin2-emin).lt.deprls) return
         if (iprint.gt.1) then
            print 33,emin2-emin,deprls,nummer
c            write(16,33) emin2-emin,deprls,nummer
         endif
         do i=1,n
            par0(i)=par(i)
         enddo
         iperio=iperio+1
         deltaE(iperio)=emin2-emin
C        Return upon periodic oscillation of the energy
         if (iperio.ge.4) then
            sumd1=deltaE(iperio)-deltaE(iperio-2)
c     +           +deltaE(iperio-4)-deltaE(iperio-6)
            sumd2=deltaE(iperio-1)-deltaE(iperio-3)
c     +           +deltaE(iperio-5)-deltaE(iperio-7)
            if ((sumd1.lt.1e-6).and.(sumd2.lt.1e-6)) then
               if (iprint.gt.0) then
                  print 34,emin2-emin,nummer
                  print*, ' But stopped because of bad convergence  ',
     +                 sumd1,sumd2
               endif
               return
            endif
            if (iperio.gt.20) then
               print 34,emin2-emin,nummer
               print*, ' But stopped because of too many iterations'
               return
            endif
         endif

C        In other cases, continue
         if (abs(emin2-emin).gt.derest) then
C           Restart search direction if the energy changed too much
            eold=emin2
            do k=1,n
               g(k)=g1(k)
            enddo
            goto 110
         else
C           Continue normally with next search direction
            emin=emin2
            goto 400
         endif
      endif

C     Make new pair list if parameters change too much
      do i=1,n
         shift=par(i)-par0(i)
         if (abs(shift).gt.shim) then
            call func(n,par,g1,emin2,1)
            if (emin.gt.50000) then
               emin=emin2
               if (iprint.gt.1) print 42,emin,nummer,6
               goto 710
            endif
            if (iprint.gt.3) print 41, ncyc,rmsder,emin,emin2,i,shift
            do j=1,n
               par0(j)=par(j)
            enddo
            if (iprev.eq.i .and. abs(shift+shprev).lt.0.2d0*shim) then
C              Increase SHIMAX if the pair list oscillates
               shim=2*shim
               if (iprint.gt.1) print 44,i,nummer,shim
            endif
            shprev=shift
            iprev=i
            if (abs(emin2-emin).gt.derest) then
C              New SD search vector if the energy changed too much
               eold=emin2
               do k=1,n
                  g(k)=g1(k)
               enddo
               goto 110
            else
C              Continue normally with next search direction
               emin=emin2
               goto 400
            endif
         endif
      enddo
C     No change in pair list necessary, continue normally

  400 continue
      if (method.eq.'PR   ') then
         pr=0
         do i=1,n
            pr=pr+(g1(i)-g(i))*g1(i)
         enddo
      else if (method.eq.'SHAN ') then
         yy=0
         py=0
         pg=0
         yg=0
         do i=1,n
            y(i)=(g(i)-g1(i))
            yy=yy+y(i)*y(i)
            py=py+y(i)*p(i)
            pg=pg-p(i)*g1(i)
            yg=yg-y(i)*g1(i)
         enddo
      else if (method.eq.'SH-PO') then
         yy=0
         py=0
         pg=0
         yg=0
         do i=1,n
            y(i)=(g(i)-g1(i))
            yy=yy+y(i)*y(i)
            py=py+y(i)*p(i)
            pg=pg-p(i)*g1(i)
            yg=yg-y(i)*g1(i)
         enddo
      else if (method.eq.'HS   ') then
         hs=0
         do i=1,n
            hs=hs+zr(i)*(-g1(i)+g(i))
         enddo
      else if (method.eq.'BFGS ') then
         do i=1,n
            dg(i)=-(g1(i)-g(i))
            hdg(i)=0
         enddo
      else if (method.eq.'PO-BE') then
         yg=0
         py=0
         do i=1,n
            y(i)=(g(i)-g1(i))
            yg=yg-y(i)*g1(i)
            py=py+y(i)*zr(i)
         enddo
      else if (method.eq.'MQN  ') then
         yy=0
         sy=0
         yg=0
         sg=0
         do i=1,n
            s(i)=(par1(i)-par(i))
            y(i)=(g(i)-g1(i))
            yy=yy+y(i)*y(i)
            sy=sy+s(i)*y(i)
            yg=yg-y(i)*g1(i)
            sg=sg-s(i)*g1(i)
         enddo
      endif

C     For all methods:
      g1g=0
      g1g1=0
      do i=1,n
         g1g1=g1g1+g1(i)*g1(i)
C        For restart criterium
         g1g =g1g+g1(i)*g(i)
C        Update derivatives;
         g(i)=g1(i)
      enddo
      dnew=g1g1
      rmsder=sqrt(g1g1/n)
      gnorm=sqrt(g1g1)

C     Update inverse hessian by BFGS algoritm
      if (method.eq.'BFGS ') then
         do j=1,n
            do i=1,n
               hdg(i)=hdg(i) + hessin(i,j)*dg(j)
            enddo
         enddo
         fac=0
         fae=0
         do i=1,n
            fac=fac+dg(i)*zr(i)
            fae=fae+dg(i)*hdg(i)
         enddo
         fac=1/fac
         fad=1/fae
         do i=1,n
            dg(i)=fac*zr(i)-fad*hdg(i)
         enddo
         do j=1,n
            do i=1,n
               hessin(i,j)=hessin(i,j)+fac*zr(i)*zr(j)-fad*hdg(i)*hdg(j)
     +              + fae*dg(i)*dg(j)
            enddo
         enddo
      endif

      npl=npl+1
      npow=npow+1

      d1g1=0
      zrnorm=0
      reset=.false.

C     Set new search direction:

C     If RMS gradient is extremely high, perform another SD step
      if (rmsder.gt.250) then
c         print*,'Another SD step as a result of large gradient'
         eold=emin
         goto 110
      endif

      if (method.eq.'PR   ' .or. method.eq.'FR   '
     +     .or. method.eq.'HS   ') then
         if (method.eq.'FR   ') gamma=dnew/dold
         if (method.eq.'PR   ') gamma=pr/dold
         if (method.eq.'HS   ') gamma=pr/hs
         do i=1,n
            zr(i)=g(i)+gamma*zr(i)
            zrnorm=zrnorm+zr(i)*zr(i)
            d1g1=d1g1-zr(i)*g(i)
         enddo
         if (abs(g1g).ge. 0.2d0*g1g1) reset=.true.
      else if (method.eq.'SHAN ' ) then
         gamma1=(1+yy/py)*pg/py-yg/py
         gamma2=pg/py
         do i=1,n
            zr(i)=g(i)-gamma1*p(i)+gamma2*y(i)
            zrnorm=zrnorm+zr(i)*zr(i)
            d1g1=d1g1-zr(i)*g(i)
         enddo
         if (abs(g1g).ge. 0.2d0*g1g1) reset=.true.
      else if (method.eq.'BFGS ') then
         do i=1,n
            zr(i)=0
            do j=1,n
               zr(i)=zr(i)+hessin(i,j)*g(j)
            enddo
            zrnorm=zrnorm+zr(i)*zr(i)
            d1g1=d1g1-zr(i)*g(i)
         enddo
         if (npl.eq.nrstrt) reset=.true.
      else if (method.eq.'PO-BE') then
         gamma=yg/py
         if (abs(g1g).ge. 0.2d0*g1g1 .or. npl.eq.1 .or.
     +        npow.eq.nrstrt) then
            if (npl.ne.1) print*,'POWELL restart'
            gamma1=0
            ptyt=0
            npow=0
            do i=1,n
               yt(i)=y(i)
               pt(i)=zr(i)
               ptyt=ptyt+yt(i)*pt(i)
            enddo
         else
            ytg=0
            do i=1,n
               ytg=ytg-yt(i)*g(i)
            enddo
            gamma1=ytg/ptyt
         endif
         do i=1,n
            zr(i)=g(i)+gamma*zr(i)+gamma1*pt(i)
            zrnorm=zrnorm+zr(i)*zr(i)
            d1g1=d1g1-zr(i)*g(i)
         enddo

C     If Powell-restart with Shanno's search direction:
      else if (method.eq.'SH-PO') then
         if (abs(g1g).ge. 0.2d0*g1g1 .or. npl.eq.1 .or.
     +        npow.eq.nrstrt) then
            if (npl.ne.1) print*,'POWELL restart'
            npow=0
            d1=0
            d4=0
            gamma1=(1+yy/py)*pg/py-yg/py
            gamma2=pg/py
            do i=1,n
               zr(i)=g(i)-gamma1*p(i)+gamma2*y(i)
               zrnorm=zrnorm+zr(i)*zr(i)
               d1g1=d1g1-zr(i)*g(i)
               yt(i)=y(i)
               pt(i)=p(i)
               d1=d1+pt(i)*yt(i)
               d4=d4+yt(i)*yt(i)
            enddo
         else
            d2=0
            d21=0
            d3=0
            d31=0
            d5=py
            d6=0
            d7=0
            d8=pg
            do i=1,n
               d2=d2+pt(i)*y(i)
               d3=d3+yt(i)*y(i)
               d21=d21-pt(i)*g(i)
               d31=d31-yt(i)*g(i)
            enddo
            do i=1,n
               av(i)=(d1/d4)*y(i)-(d2/d4)*yt(i)+(2*d2/d1-d3/d4)*pt(i)
               bv(i)=-(d1/d4)*g(i)-(d21/d4)*yt(i)+(2*d21/d1-d31/d4)
     +              *pt(i)
            enddo
            do i=1,n
               d6=d6+av(i)*y(i)
               d7=d7-av(i)*g(i)
            enddo
            do i=1,n
               zr(i)=-bv(i)+(d8/d5)*av(i)-((1+d6/d5)*d8/d5-d7/d5)*p(i)
               d1g1=d1g1-g(i)*zr(i)
               zrnorm=zrnorm+zr(i)*zr(i)
            enddo
         endif
      else if (method.eq.'MQN  ') then
         do i=1,n
            zr(i)= g(i) + (y(i)*sg+s(i)*yg)/sy
     +           - (1 + yy/sy)*(s(i)*sg/sy)
            d1g1=d1g1-g(i)*zr(i)
            zrnorm=zrnorm+zr(i)*zr(i)
         enddo
         if (abs(g1g).ge. 0.2d0*g1g1) reset=.true.
      endif

      eold=emin
      dold=dnew
      step=d0/sqrt(zrnorm)
      icp=9
      if (step.lt.1.0d-8) goto 700
      if (reset) goto 110

C     Next iteration: new line minimization
      goto 200

  700 continue
C     Step size has become unacceptably low, abort
      if (iprint.gt.1) print*,
     +     ' ***** STEP in CONGRA too small at checkpoint',icp,nummer
      call func(n,par,g,emin,1)
      if (iprint.gt.1) print*,' Energy is', emin
      if (emin.gt.50000 .and. iprint.gt.1) print 42,emin,nummer,icp
      return

  710 continue
c      print*,' Abnormal end of CONGRA'
      return
C     The next two lines may result in converging minimizations, but with
C         extrememely deformed structures. Is that useful?
c      print*,' ***** CONGRA failed, try to restart!'
c      goto 90

   32 format(' Starting CONGRA,  nsd,tol,step0,deprls,shimax:',
     +      i8,2f10.6,f8.3,f8.3,f8.4)
   33 format(' Converged, but dE too large:',2f11.7,5x,
     +     'Restart with new pair list, structure',i6)
   34 format(' Update pair list needed ,dE = ',f10.6,' structure',i6)
   35 format(' More than 5 iterations in linemin. ',
     +     'Restart with new pair list.')
   36 format(' Maximum number of iterations',i6,' exceeded.',5x,
     +     'Energy recalculated as:',f12.3)
   37 format(' Redo GAMESS from CONGRA, cycle',i4,'     emin',f15.3)
   40 format(' RESTART CG: ncyc, rmsder, step, emin       ',
     +     i5,f10.3,f14.8,f13.3)
   41 format('     NEW PL: ncyc, rmsder, emin1, emin2, nr, shift',
     +     i5,f10.3,f14.3,f13.3,i5,f10.4)
   42 format(' Unreasonable energy',f15.1,' in CONGRA ',
     +     'for structure',i9,' at checkpoint',i3)
   43 format(' ***** Derivatives are small enough at (re)start CG',
     +     '  structure  ',i6)
   44 format(' ***** Oscillation in parameter',i4,' for structure',i9,
     +     6x,'SHIMAX doubled to',f10.4,10x,'Increase ALPHA and HMAX!' )
   50 format(i10,2x,2f12.6,2f13.3,3e11.3,f10.6)
   60 format(7x,' CG: niti, ncyc, rmsder, step, emin',
     +     i5,i7,f11.4,f14.8,f13.3)
   70 format(5x,'Energy increased (or decreased insufficiently)',
     +     3f13.3,e15.3)
   71 format(5x,'Energy larger than ea,eb',3f13.3)
   72 format(5x,'Derivatives decreased insufficiently:',3f13.3)
   80 format(12f10.4)
      end
