      subroutine dlist(istruc,box,x,nrpind,molind,nrptot,inttot,
     +     izoek,iprint,maxtel)

C     Creates the distance list for one crystal structure.
C     Written bij W.T.M. Mooij, adapted by B.P. van Eijck

C     Input through parameters:
C     ISTRUC: the sequence number of the structure
C     BOX(1...6): ax, by, cz, bx, cx, cy
C     X(1...3*NRPTOT): the atom coordinates (nm) for all atoms,
C             including symmetry-expanded ones.
C     NRPIND: the number of independent atoms      = NGR * NRP
C     MOLIND: the number of independent molecules  = NGR * NMOL
C     NRPTOT: the total number of atoms            = NGR * NRP * (NSPGR + 1)
C     INTTOT: the total number of intramolecular distances = NGR * INTRA
C     IZOEK = 0: cluster all structures
C           = 1: search for a particular structure
C     IPRINT regulates output to the screen.

C     Input through common blocks:
C     /NAAM/ gives various constants:
C        CUTOFF (nm): the largest distance included in the list of distances
C        MPOW:   distances are muliplied by MPOW to make them integer.
C        MDIFF:  tolerance DIFF (nm) for equivalence of distances,
C                multiplied by MPOW to make it integer
C        MCUT:   [cutoff distance (nm) - 1.1 * dist],
C                multiplied by MPOW to make it integer
C            The distance list is based on CUTOFF, the actual comparison on
C                this slightly smaller distance.
C        MARGIN: number of contacts per molecule that are allowed not to be
C                found (zero or small).
C     /CLUS/ gives various integer arrays:
C        MOLNR(1...NRP): for each atom, the sequence number of the molecule
C                to which it belongs.
C        TYPE(1...NRP): for each atom, its type. Equivalent atoms (e.g. the
C                carbon atoms C1 and C3 in propane) have the same type.
C        INTRI, INTRJ(1...INTTOT): pairs of atom numbers to indicate
C                intramolecular distances to be included in the distance list,
C                chosen to discriminate between several possible conformations
C        STRLST(ISTRUC): sequence code number of the structure ISTRUC

C     Output through parameters:
C     MAXTEL: the maximum number of distances ever encountered
C             for one structure

C     Output through common block /CLUS/:
C     ZFLAG(ISTRUC): the number of unequivalent molecules
C     SLIST(ISTRUC): the position in the array MDIST before the one where
C              the first distance of the structure ISTRUC is placed.
C     TELLER(ISTRUC,mol=1...ZFLAG(ISTRUC)): the position of the last
C             distance of molecule MOL in the list of distances.
C     MDIST(1...): the array of distances, multiplied by MPOW. The last
C             distance of molecule MOL in the structure ISTRUC can be found
C             in the position SLIST(ISTRUC)+TELLER(ISTRUC,MOL) in this array.

      implicit none
      include 'cdist.i'
      integer i,i1,i2,ilmin,inttot,iover,iprint,
     +     istruc,izoek,j,k,kshi,m,m1,maxtel,mdd,mindd,
     +     molind,molnum,n1,n2,n3,
     +     ni,ni3,nj,nj3,nrpind,nrptot,nsjt,nst
      double precision ax,box,bx,by,cx,cy,cz,d1,d2,dd,
     +     rcutp2,rij2,rijt,x,xij,xij0,yij,yij0,zij,zij0
      integer molflg(maxmol),atflg(maxcon),faults,hits
      integer lstart(maxmol),lend(maxmol)
      dimension box(*),x(*)

      if (nrptot.gt.maxnrp) then
         print*,' ***** Increase MAXNRP',nrptot,maxnrp
         stop
      endif
      if (molind.gt.maxmol) then
         print*,' ***** Increase MAXMOL',molind,maxmol
         stop
      endif
      if (inttot.gt.mxintr) then
         print*,' ***** Increase MXINTR',inttot,mxintr
         stop
      endif

      rcutp2=cutoff**2
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      nsjt=0
      lstart(1)=1
      slist(1)=0
      nst=slist(istruc)
C     NST gives the index of the last atom previously entered in MDIST.
C     Loop over all independent atoms:
      do ni=1,nrpind
         if (type(ni).le.0) goto 400
         molnum=molnr(ni)
         ni3=3*ni-3
C        Loop over all atoms (including symmetry-expanded ones), expand
C           unit cells, and select distances within the CUTOFF radius
C           (no charge groups; cut-off sphere is used for atomic distances).
         do nj=1,nrptot
            if (type(nj).le.0) goto 200
            nj3=3*nj-3
            xij0=x(ni3+1)-x(nj3+1)
            yij0=x(ni3+2)-x(nj3+2)
            zij0=x(ni3+3)-x(nj3+3)
            n3=int((cutoff-zij0)/cz)
            zij=zij0+n3*cz
            n2=int((cutoff-yij0-n3*cy)/by)
            yij=yij0+n2*by+n3*cy
            n1=int((cutoff-xij0-n2*bx-n3*cx)/ax)
            xij=xij0+n1*ax+n2*bx+n3*cx

 150        continue
            if (n1.eq.0 .and. n2.eq.0 .and. n3.eq.0
     +           .and. molnum.eq.molnr(nj)) then
C              This is an intramolecular distance
               do i=1,inttot
                  if (ni.eq.intri(i) .and. nj.eq.intrj(i)) goto 155
               enddo
               goto 160
            endif
 155        continue
            rij2=xij**2+yij**2+zij**2
            if (rij2.lt.rcutp2) then
C              The distance is accepted
               nsjt=nsjt+1
               if (nsjt.gt.maxtel) then
                  maxtel=nsjt
                  if (nsjt.gt.maxcon) then
                     print*,' ***** Number of contacts in one ',
     +                    'structure too big, increase MAXCON',nsjt
                     stop
                  endif
               endif
               if (nsjt+nst.gt.maxcum) then
                  print*,' ***** Cumulative number of contacts ',
     +                 'too big, increase MAXCUM',nsjt+nst,maxcum
                  stop
               endif
               rijt=sqrt(rij2)
               k=nint(mpow*rijt)+mpow*(type(nj)+100*type(ni))
               mdist(nsjt+nst)=k
               if (iprint.gt.3) then
                  dd=mod(k,mpow)/dfloat(mpow)
                  if (abs(dd-rijt).gt.0.0001) then
                     print*,' ***** Error!!',k,mod(k,mpow),rijt,dd-rijt
                     stop
                  endif
                  print 20,molnum,nsjt,rijt,ni,nj,type(ni),type(nj),
     +                 n1,n2,n3,xij,yij,zij,mdist(nsjt+nst)
               endif
            endif
 160        continue
            n1=n1-1
            xij=xij-ax
 170        if (xij.lt.-cutoff) then
               n2=n2-1
               yij=yij-by
 180           if (yij.lt.-cutoff) then
                  n3=n3-1
                  zij=zij-cz
                  if (zij.lt.-cutoff) goto 190
                  n2=int((cutoff-yij0-n3*cy)/by)
                  yij=yij0+n2*by+n3*cy
                  goto 180
               endif
               n1=int((cutoff-xij0-n2*bx-n3*cx)/ax)
               xij=xij0+n1*ax+n2*bx+n3*cx
               goto 170
            endif
            goto 150
 190        continue
 200        continue
         enddo
         lend(molnum)=nsjt
         lstart(molnum+1)=nsjt+1
C        NSJT is the place of the last entry in these arrays for this molecule
 400     continue
      enddo
C     The list of distances has been made.

C     Check for identical surroundings for the different molecules:
C     MOLFLG = 1: molecule is identical to another one
      do i=1,maxmol
         molflg(i)=0
      enddo
      do n1=1,molind
         if (molflg(n1).eq.1) goto 983
         do n2=1,molind
            if ((n2.eq.n1).or.(molflg(n2).eq.1)) goto 982
            faults=0
            do i=1,maxcon
               atflg(i)=0
            enddo
C           Do loop over all the contacts of molecules n1 in structure 1.
C           Contacts within diff from distance cut-off are disregarded
            do i1=lstart(n1),lend(n1)
               m1=mdist(i1+nst)
               if (mod(m1,mpow).gt.mcut) goto 981
               hits=0
               mindd=100000000
C              Select the closest corresponding distance in structure 2.
               do i2=lstart(n2),lend(n2)
                  if (atflg(i2).eq.0) then
                     mdd=abs(m1-mdist(i2+nst))
                     if (iprint.gt.4) then
                        d1=mod(mdist(i1+nst),mpow)/dfloat(mpow)
                        d2=mod(mdist(i2+nst),mpow)/dfloat(mpow)
                        dd=abs(d1-d2)
                        print 23,n1,i1-lstart(n1)+1,d1,
     +                       n2,i2-lstart(n2)+1,d2,dd
                     endif
                     if (mdd.lt.mdiff) then
                        hits=hits+1
                        if (mdd.lt.mindd) then
                           mindd=mdd
                           ilmin=i2
                        endif
                     endif
                  endif
               enddo
               if (hits.eq.0) then
                  if (faults.ge.margin) goto 982
                  faults=faults+1
                  goto 981
               else
                  atflg(ilmin)=1
C                 This contact is now marked as used
               endif
 981           continue
            enddo
C           If the program gets here the molecules are identical
            molflg(n2)=1
 982        continue
         enddo
 983     continue
      enddo

C     Now rearrange the entries in common cluster, such that all independent
C     molecules are in front (and looping over zflag will take you there)
      zflag(istruc)=1
      iover=2
C     First can always stay in its place; write new structure at IOVER.
      do i=2,molind
         if (molflg(i).eq.0) then
            zflag(istruc)=zflag(istruc)+1
            if (iover.ne.i) then
C              Shift block I down to IOVER, using a shift KSHI
               kshi=lstart(i)-lstart(iover)
               lend(iover)=lend(i)-kshi
               do j=lstart(iover),lend(iover)
                  mdist(j+nst)=mdist(j+kshi+nst)
               enddo
               lstart(iover+1)=lend(iover)+1
            endif
            iover=iover+1
         endif
      enddo

      teller(istruc,0)=0
      do i=1,zflag(istruc)
         teller(istruc,i)=lend(i)
      enddo
      slist(istruc+1)=nst+nsjt
      if (iprint.gt.1) then
         if (izoek.eq.0) then
            print 21,istruc,strlst(istruc),zflag(istruc),
     +        nsjt,(lend(m)-lstart(m)+1,m=1,zflag(istruc))
            write(16,21) istruc,strlst(istruc),zflag(istruc),
     +           nsjt,(lend(m)-lstart(m)+1,m=1,zflag(istruc))
         endif
         if (izoek.eq.1) then
            print 22,strlst(istruc),zflag(istruc),
     +        nsjt,(lend(m)-lstart(m)+1,m=1,zflag(istruc))
            write(16,22)strlst(istruc),zflag(istruc),
     +        nsjt,(lend(m)-lstart(m)+1,m=1,zflag(istruc))
         endif
      endif
c      pause

 20   format(2i5,f8.3,2i5,2x,2i5,2x,3i4,3f8.3,i20)
 21   format(' Structure',i7,i8,'  has Z" =',i3,
     +     ' independent molecules with',i5,' distances: ',20i5)
 22   format(' Structure',i8,'  has Z" =',i3,
     +     ' independent molecules with',i5,' distances: ',20i5)
 23   format(2i4,f8.3,4x,2i4,2f8.3)
      end
