      subroutine func2(npar,par,der,enb,isubr)

C     Energy calculation for a crystal of semi-rigid molecules.
C     Put a reference molecule (coordinates XR) into a crystal, applying
C     rotations, translations and hydroxyl internal rotations.
C     The crystallographic parameters are put into BOX, the atomic
C     coordinates into X.
C     Calls energy routine, calculates energy ENB and forces DER.

C     Input through parameters:
C        NPAR: number of parameters
C        PAR(1..NPAR): the cell parameters, centers of gravity,
C                      Euler angles, and hydroxyl dihedral angles.
C        ISUBR=1: calculation through NONBE (make pair list)
C             =2: calculation through LIST (use pair list)
C     Input through common blocks:
C        /PARS/:
C           NRP: number of atoms
C           XR(1..3*NRP): reference coordinates for the free molecule
C           CEL(1..12): ax, by, cz, bx, cx, cy, X, Y, Z, phi, theta, psi
C           IREF(1..12)=0: take BOX from CEL and do not refine
C                      =1: take BOX from PAR and do refine
C        /INDS/:
C           NDIHH: number of hydroxyl groups to be rotated
C           VN(1..NDIHH), NTAL(1..NDIHH) define the potential energy:
C              each dihedral contributes VN(1+cos(NTAL*PAR))
C           IOME, JOME, KOME, NDIHZ: specify the hydroxyl dihedral angles,
C              which are initially rotated about an angle PAR.
C              See procedure INTROT for details.

C     Output through parameters:
C        ENB: nonbonded energy
C        DER(1..NPAR): negative derivatives of ENB to PAR
C     Output through common block:
C        X(1..3*NRP): optimized coordinates

C     Calls subroutines INTROT, NONBE, LIST, and function DIHANG.

      implicit none
      include 'params.i'
      include 'cinds.i'
      include 'cpars.i'
      double precision arot,box,brot,ca,cb,cg,co,der,deuler,dihan,din,
     +     enb,euler,f,omega,par,sa,sb,sg,si,uit,xint
      integer i,i3,isubr,j,j3,k,k3,kpar,m,n,npar
      dimension der(*),par(*),euler(3,3),deuler(3,3,3),box(12),uit(12),
     +     f(3*maxnrp),xint(3*maxnrp),arot(3),brot(3)

      kpar=0
      do i=1,12
         if (iref(i).gt.0) then
            kpar=kpar+1
            box(i)=par(kpar)
         else
            box(i)=cel(i)
         endif
      enddo
      if (npar.ne.kpar+ndihh) then
         print*, ' ***** error in FUNC2:',nrp,npar,kpar,ndihh
         stop
      endif

      do i=1,nrp
         do m=1,3
            xint(3*i-3+m)=xr(3*i-3+m)
         enddo
      enddo

C     Rotate hydroxyl dihedrals about PAR
      do i=1,ndihh
         omega=par(kpar+i)
         co=cos(omega)-1
         si=sin(omega)
         i3=3*iome(ndihz+i)-3
         j3=3*jome(ndihz+i)-3
         k3=3*kome(ndihz+i)-3
         call introt(i3,j3,k3,xint,arot,brot)
         do m=1,3
            xint(i3+m)=xint(i3+m)+si*brot(m)+co*arot(m)
         enddo
      enddo

      ca=cos(box(10))
      sa=sin(box(10))
      cb=cos(box(11))
      sb=sin(box(11))
      cg=cos(box(12))
      sg=sin(box(12))
      euler(1,1)=cg*ca-sg*sa*cb
      deuler(1,1,1)=-cg*sa-sg*ca*cb
      deuler(1,1,2)=       sg*sa*sb
      deuler(1,1,3)=-sg*ca-cg*sa*cb
      euler(1,2)=    cg*sa+sg*ca*cb
      deuler(1,2,1)= cg*ca-sg*sa*cb
      deuler(1,2,2)=      -sg*ca*sb
      deuler(1,2,3)=-sg*sa+cg*ca*cb
      euler(1,3)=    sg*sb
      deuler(1,3,1)= 0
      deuler(1,3,2)= sg*cb
      deuler(1,3,3)= cg*sb
      euler(2,1)=   -sg*ca-cg*sa*cb
      deuler(2,1,1)= sg*sa-cg*ca*cb
      deuler(2,1,2)=       cg*sa*sb
      deuler(2,1,3)=-cg*ca+sg*sa*cb
      euler(2,2)=   -sg*sa+cg*ca*cb
      deuler(2,2,1)=-sg*ca-cg*sa*cb
      deuler(2,2,2)=      -cg*ca*sb
      deuler(2,2,3)=-cg*sa-sg*ca*cb
      euler(2,3)=    cg*sb
      deuler(2,3,1)= 0
      deuler(2,3,2)= cg*cb
      deuler(2,3,3)=-sg*sb
      euler(3,1)=    sa*sb
      deuler(3,1,1)= ca*sb
      deuler(3,1,2)= sa*cb
      deuler(3,1,3)= 0
      euler(3,2)=   -ca*sb
      deuler(3,2,1)= sa*sb
      deuler(3,2,2)=-ca*cb
      deuler(3,2,3)= 0
      euler(3,3)=    cb
      deuler(3,3,1)= 0
      deuler(3,3,2)=-sb
      deuler(3,3,3)= 0
      do i=1,nrp
         j=3*(i-1)
         do m=1,3
            x(j+m)=box(m+6)
            do n=1,3
               x(j+m)=x(j+m)+euler(m,n)*xint(j+n)
            enddo
         enddo
      enddo

      if (isubr.eq.0) return
      if (isubr.eq.1) call nonbe(1,nrp,enb,1,uit,1,box,x,f,0)
      if (isubr.eq.2) call list(1,nrp,enb,1,uit,box,x,f)

      do m=1,3
         uit(m+6)=0
         uit(m+9)=0
         do i=1,nrp
            j=3*(i-1)
            uit(m+6)=uit(m+6)+f(j+m)
            do i3=1,3
               do j3=1,3
                  uit(m+9)=uit(m+9)+f(j+i3)*deuler(i3,j3,m)*xint(j+j3)
               enddo
            enddo
         enddo
      enddo
      k=0
      do i=1,12
         if (iref(i).gt.0) then
            k=k+1
            der(k)=uit(i)
         endif
      enddo

      do i=1,ndihh
         k=kpar+i
         din=ntal(i)*dihan(i+ndihz,x)
         enb=enb+Vn(i)*(1+cos(din))
         der(k)=ntal(i)*Vn(i)*sin(din)
         i3=3*iome(ndihz+i)-3
         j3=3*jome(ndihz+i)-3
         k3=3*kome(ndihz+i)-3
         call introt(i3,j3,k3,x,arot,brot)
         do m=1,3
            der(k)=der(k)+f(i3+m)*brot(m)
         enddo
      enddo
      end
