      subroutine geom(ifirst,nrp,ngr,x,amass,box,nmols,iref,iprint)
C     Calculates geometry parameters from atom coordinates: bonds, angles,
C     impropers, dihedrals, van der Waals distances, cell parameters

C     In a first call (IFIRST=1) these calculations are done for the
C     reference structure (usually the experimental one). A list of
C     van der Waals distances, defined as interatomic distances between 
C     HBMIN and HBMAX, is made.
C     The geometry parameters are stored in OBS(1..NPAR);
C     in a next call (IFIRST>1) the same parameters are calculated for another
C     structure (normally the same one after energy optimization).
C     To represent the discrepancies in one number, the
C     "RMS delta/sigma" is found after a fairly arbitrary
C     assignment of "acceptable" differences SIGMA.

C     Input through parameters:
C        NRP: number of atoms in one group
C        NGR: number of groups
C        X(1..): atom coordinates
C        PANM(1..): atom names
C        AMASS(1..): atomic masses
C        BOX(1..6*NMOLS+6): ax, by, cz, bx, cx, cy,
C           {X, Y, Z, phi, theta, psi (degrees)} (1...NMOLS)
C        NMOLS: the number of molecules
C        IREF(1..12)=0/1: disable/enable refinement of that parameter
C        IPRINT regulates output to unit 16 and to screen.

C     Input through common blocks:
C        /SPGR/ gives space group information:
C           NSPGR, MSP, TSP: see subroutine LEESR
C        /BIND/:
C           NBOND: number of bonds
C           IB, JB (1..): atom numbers in pairs for each bond
C           CB, B0 (1..): force constants and reference bond lengths
C        /HOEK/:
C           NHOEK: number of angles
C           IT, JT, KT (1..): atom numbers for each angle
C           CT, T0 (1..): force constants and reference angles
C        /IMPR/:
C           NIMPR: number of improper dihedrals
C           IQ, JQ, KQ, LQ (1..): atom numbers for each dihedral
C           CQ, Q0 (1..): force constants and reference bond lengths
C        /TORS/:
C           NTORS: number of proper dihedrals
C           IP, JP, KP, LP (1..): atom numbers for each dihedral
C           CP, NP (1..): force constants and periodicities
C        /ATTP/:
C           PANM(1..): atom names in topology file, used if IFIL>0

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cspgr.i'

      integer i,i3,i33,ifirst,ih,ihb1,ihb2,ihbtr,impr,io,iprint,iref,
     +     itx,ity,itz,j,j3,j33,jmol,k,k3,k33,l3,l33,m,maxdat,maxhb,
     +     n,n1,n2,nda,ndev,ngr,nhb,nmols,nop3,npar,nrp,nvdw
      double precision aco,alpha,amass,ap,at,ax,bb,beta,box,bx,by,cc,
     +     cel,cosa,cosb,cosc,cphi,cpsi,csp,cst,ctheta,cx,cy,cz,
     +     d,dr,devmax,dx,dy,dz,eul,euler,euler0,euler1,f,frac,
     +     gamma,grdev,hbmax,hbmin,obs,offd,pi,r,rad,rij,rij2,rkj,
     +     rkj2,rmj2,rnk2,rot1,rot2,rrik,rrmn,s,s1,s2,sangle,
     +     sbond,sbondh,scell,sdev,seul,shoek,shoekh,sigma,simpr,
     +     snp,sphi,spoor,sprop,sproph,spsi,stheta,svdw,szwpt,tol,
     +     x,xexp,xij,xj,xk,xkj,xkl,xmj,xnk,xxij,xxkj,xxkl,
     +     vol1,vol2,yij,ykj,ykl,ymj,ynk,zij,zkj,zkl,zmj,znk,zw
      parameter(maxdat=10000,maxhb=4000)
      dimension obs(maxdat),sigma(20),
     +     ihb1(maxhb),ihb2(maxhb),ihbtr(maxhb,3)
      dimension amass(maxat),box(*),iref(*),x(*),cel(6),
     +     xxij(30),xxkj(3),xxkl(3),sdev(20),ndev(20),grdev(20),
     +     xexp(maxat,3),frac(maxmol,3),
     +     euler(3,3),euler0(maxmol,3,3),euler1(maxmol,3,3)
      character*5 oname(maxhb),otype(maxhb)
      save vol1,obs,ihb1,ihb2,ihbtr,nvdw,frac,euler0,cel,oname,otype
      logical first

      hbmin=0.15d0
      hbmax=0.35d0
c      hbmin=0.23
c      hbmax=0.29
      first=(ifirst.eq.1)
C     Assign acceptable differences:
      sbond=0.001d0
      sigma(1)=sbond
C     Bond lengths without hydrogen atoms
      sbondh=0.03d0
      sigma(2)=sbondh
C     Bond lengths with hydrogen atoms
      shoek=1
      sigma(3)=shoek
C     Bond angles without hydrogen atoms
      shoekh=5
      sigma(4)=shoekh
C     Bond angles with hydrogen atoms
      simpr=1
      sigma(5)=simpr
C     Improper dihedral angles
      sprop=1
      sigma(6)=sprop
C     Torsions without hydrogen atoms
      sproph=10
      sigma(7)=sproph
C     Torsions with hydrogen atoms
      svdw=0.002d0
c      svdw=0.001
      sigma(8)=svdw
C     Non-hydrogen van der Waals distances
      scell=0.01d0
      sigma(9)=scell
C     Cell edges a, b, c (relative!)
      sangle=1
      sigma(10)=sangle
C     Cell angles alpha, beta, gamma
      szwpt=0.002d0
      sigma(11)=szwpt
C     Centers of gravity X, Y, Z
      seul=1
      sigma(12)=seul
C     Euler angles phi, theta, psi

      npar=0
      pi=4*atan(1.0d0)
      rad=180/pi
      tol=1.0d-10
      do i=1,12
         sdev(i)=0
         ndev(i)=0
         grdev(i)=0
      enddo
      devmax=0
      if (.not. first) then
         if (iprint.gt.2) print 20
         write(16,20)
      endif

C     Bond lengths
      if (iprint.gt.2 .and. .not. first) print*, '   Bonds'
      if (.not. first) write(16,*) '   Bonds'
       do k=1,ngr
         do n=1,nbond
            i3=ib(n)
            j3=jb(n)
            if (amass(i3).lt.3 .or. amass(j3).lt.3) then
               ih=1
               s=sbondh
            else
               ih=0
               s=sbond
            endif
            if (abs(s).gt.tol) then
               i3=i3+(k-1)*nrp
               j3=j3+(k-1)*nrp
               i33=3*i3-3
               j33=3*j3-3
               rij2=0
               do m=1,3
                  rij2=rij2+(X(I33+M)-X(J33+M))**2
               enddo
               npar=npar+1
               rij=sqrt(rij2)
               if (first) then
                  obs(npar)=rij
               else
                  d=obs(npar)-rij
                  r=abs(d/s)
                  if (ih.eq.0) then
                     sdev(1)=sdev(1)+r**2
                     ndev(1)=ndev(1)+1
                     if (abs(d).gt.abs(grdev(1))) grdev(1)=d
                  else
                     sdev(2)=sdev(2)+r**2
                     ndev(2)=ndev(2)+1
                     if (abs(d).gt.abs(grdev(2))) grdev(2)=d
                  endif
                  if (r.gt.devmax) devmax=r
                  if (iprint.gt.2) print 30,npar,i3,j3,
     +                 b0(n),ibcod(n),obs(npar),rij,d,s,r
     +                 ,panm(i3),panm(j3),type(i3),type(j3)
                  write(16,30) npar,i3,j3,
     +                 b0(n),ibcod(n),obs(npar),rij,d,s,r
     +                 ,panm(i3),panm(j3),type(i3),type(j3)
               endif
            endif
         enddo
      enddo

C     Bond angles
      if (iprint.gt.2 .and. .not. first) print*, '   Angles'
      if (.not. first) write(16,*) '   Angles'
      do k=1,ngr
         do n=1,nhoek
            i3=it(n)
            j3=jt(n)
            k3=kt(n)
            if (amass(i3).lt.3 .or. amass(k3).lt.3) then
               ih=1
               s=shoekh
            else
               ih=0
               s=shoek
            endif
            if (abs(s).gt.tol) then
               i3=i3+(k-1)*nrp
               j3=j3+(k-1)*nrp
               k3=k3+(k-1)*nrp
               i33=3*i3-3
               j33=3*j3-3
               k33=3*k3-3
               RIJ2=0
               RKJ2=0
               do M=1,3
                  XJ=X(J33+M)
                  XXIJ(M)=X(I33+M)-XJ
                  XXKJ(M)=X(K33+M)-XJ
                  RIJ2=RIJ2+XXIJ(M)**2
                  RKJ2=RKJ2+XXKJ(M)**2
               enddo
               XIJ=XXIJ(1)
               YIJ=XXIJ(2)
               ZIJ=XXIJ(3)
               XKJ=XXKJ(1)
               YKJ=XXKJ(2)
               ZKJ=XXKJ(3)
               RRIK=SQRT(RIJ2*RKJ2)
               CST=(XIJ*XKJ+YIJ*YKJ+ZIJ*ZKJ)/RRIK
               if (CST.GT. 1) CST= 1
               if (CST.LT.-1) CST=-1
               AT=rad*ACOS(CST)
               npar=npar+1
               if (first) then
                  obs(npar)=at
               else
                  d=obs(npar)-at
                  r=abs(d/s)
                  if (ih.eq.0) then
                     sdev(3)=sdev(3)+r**2
                     ndev(3)=ndev(3)+1
                     if (abs(d).gt.abs(grdev(3))) grdev(3)=d
                  else
                     sdev(4)=sdev(4)+r**2
                     ndev(4)=ndev(4)+1
                     if (abs(d).gt.abs(grdev(4))) grdev(4)=d
                  endif
                  if (r.gt.devmax) devmax=r
                  if (iprint.gt.2) print 50,npar,i3,j3,k3,
     +                 t0(n)*rad,ihcod(n),obs(npar),at,d,s,r
     +                 ,panm(i3),panm(j3),panm(k3),
     +                 type(i3),type(j3),type(k3)
                  write(16,50) npar,i3,j3,k3,
     +                 t0(n)*rad,ihcod(n),obs(npar),at,d,s,r
     +                 ,panm(i3),panm(j3),panm(k3),
     +                 type(i3),type(j3),type(k3)
               endif
            endif
         enddo
      enddo

C     Dihedrals (proper and improper)
      do impr=1,2
         if (impr.eq.1) then
            if (abs(simpr).lt.tol) goto 400
            nda=nimpr
            if (iprint.gt.2 .and. .not. first) print*, '   Impropers'
            if (.not. first) write(16,*) '   Impropers'
         endif
         if (impr.eq.2) then
            if (abs(sprop).lt.tol .and. abs(sproph).lt.tol) goto 405
            nda=ntors
            if (iprint.gt.2 .and. .not. first) print*, '   Dihedrals'
            if (.not. first) write(16,*) '   Dihedrals'
         endif
         do k=1,ngr
            do n=1,nda
               if (impr.eq.1) then
                  I3=Iq(n)
                  J3=Jq(n)
                  K3=Kq(n)
                  L3=Lq(n)
               else
                  I3=IP(n)
                  J3=JP(n)
                  K3=KP(n)
                  L3=LP(n)
                  if (amass(i3).lt.3 .or. amass(l3).lt.3) then
                     ih=1
                     s=sproph
                  else
                     ih=0
                     s=sprop
                  endif
               endif
               i3=i3+(k-1)*nrp
               j3=j3+(k-1)*nrp
               k3=k3+(k-1)*nrp
               l3=l3+(k-1)*nrp
               i33=3*i3-3
               j33=3*j3-3
               k33=3*k3-3
               l33=3*l3-3
               do M=1,3
                  XJ=X(J33+M)
                  XK=X(K33+M)
                  XXIJ(M)=X(I33+M)-XJ
                  XXKJ(M)=XK-XJ
                  XXKL(M)=XK-X(L33+M)
               enddo
               XIJ=XXIJ(1)
               YIJ=XXIJ(2)
               ZIJ=XXIJ(3)
               XKJ=XXKJ(1)
               YKJ=XXKJ(2)
               ZKJ=XXKJ(3)
               XKL=XXKL(1)
               YKL=XXKL(2)
               ZKL=XXKL(3)

               XMJ=YIJ*ZKJ-ZIJ*YKJ
               YMJ=ZIJ*XKJ-XIJ*ZKJ
               ZMJ=XIJ*YKJ-YIJ*XKJ
               XNK=YKJ*ZKL-ZKJ*YKL
               YNK=ZKJ*XKL-XKJ*ZKL
               ZNK=XKJ*YKL-YKJ*XKL
               RMJ2=XMJ**2+YMJ**2+ZMJ**2
               RNK2=XNK**2+YNK**2+ZNK**2
               RRMN=SQRT(RMJ2*RNK2)
               rkj=sqrt(xkj**2+ykj**2+zkj**2)
               CSP=(XMJ*XNK+YMJ*YNK+ZMJ*ZNK)/RRMN
               snp=(XKJ*(YMJ*ZNK-ZMJ*YNK)+YKJ*(ZMJ*XNK-XMJ*ZNK)+
     +              ZKJ*(XMJ*YNK-YMJ*XNK))/(rrmn*rkj)
               ap=atan2(snp,csp)*rad
               if (impr.eq.1) then
                  npar=npar+1
                  if (first) then
                     obs(npar)=ap
                  else
                     d=obs(npar)-ap
                     if (d.gt. 180) d=d-360
                     if (d.lt.-180) d=d+360
                     r=abs(d/simpr)
                     sdev(5)=sdev(5)+r**2
                     ndev(5)=ndev(5)+1
                     if (abs(d).gt.abs(grdev(5))) grdev(5)=d
                     if (r.gt.devmax) devmax=r
                     if (iprint.gt.2) print 70,npar,i3,j3,k3,l3,
     +                    rad*q0(n),iicod(n),obs(npar),ap,d,simpr,r
     +                    ,panm(i3),panm(j3),panm(k3),panm(l3),
     +                    type(i3),type(j3),type(k3),type(l3)
                     write(16,70) npar,i3,j3,k3,l3,
     +                    rad*q0(n),iicod(n),obs(npar),ap,d,simpr,r
     +                    ,panm(i3),panm(j3),panm(k3),panm(l3),
     +                    type(i3),type(j3),type(k3),type(l3)
                  endif
               endif
               if (impr.eq.2) then
C                 Count each dihedral only once
                  do j=1,n-1
                     if (ip(n).eq.ip(j) .and. jp(n).eq.jp(j) .and. 
     +                    kp(n).eq.kp(j) .and. lp(n).eq.lp(j)) goto 350
                  enddo  
                  npar=npar+1
                  if (first) then
                     obs(npar)=ap
                  else
                     d=obs(npar)-ap
                     if (d.gt. 180) d=d-360
                     if (d.lt.-180) d=d+360
                     r=abs(d/s)
                     if (ih.eq.0) then
                        sdev(6)=sdev(6)+r**2
                        ndev(6)=ndev(6)+1
                        if (abs(d).gt.abs(grdev(6))) grdev(6)=d
                     else
                        sdev(7)=sdev(7)+r**2
                        ndev(7)=ndev(7)+1
                        if (abs(d).gt.abs(grdev(7))) grdev(7)=d
                     endif
                     if (r.gt.devmax) devmax=r
                     if (iprint.gt.2) print 80,npar,i3,j3,k3,l3,
     +                    cp(n),np(n),itcod(n),obs(npar),ap,d,s,r
     +                    ,panm(i3),panm(j3),panm(k3),panm(l3),
     +                    type(i3),type(j3),type(k3),type(l3)
                     write(16,80) npar,i3,j3,k3,l3,
     +                    cp(n),np(n),itcod(n),obs(npar),ap,d,s,r
     +                    ,panm(i3),panm(j3),panm(k3),panm(l3),
     +                    type(i3),type(j3),type(k3),type(l3)
                  endif
               endif
  350          continue
            enddo
         enddo
  400    continue
      enddo
  405 continue

C     van der Waals distances up to 3.5 Angstrom
      if (iprint.gt.2 .and. .not. first) print*, '   vdWaals '
      if (.not. first) write(16,*) '   vdWaals '
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      if (.not. first) then
         vol2=ax*by*cz/(ngr*(nspgr+1))
      else
         vol1=ax*by*cz/(ngr*(nspgr+1))
      endif
      io=0
      do k=1,ngr
         do i=1,nrp
            if (panm(i)(1:1).ne.'H') then
               io=io+1
               oname(io)=panm(i)(1:5)
               otype(io)=type(i)(1:5)
               j=i+(k-1)*nrp
               do m=1,3
                  xexp(io,m)=x(3*j-3+m)
               enddo
            endif
         enddo
      enddo
      do nop=1,nspgr
         nop3=3*nop-3
         do i=1,io
            k=i+nop*io
            oname(k)=oname(i)
            otype(k)=otype(i)
            xexp(k,1)=msp(nop3+1)*xexp(i,1)+
     +           tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
            xexp(k,2)=msp(nop3+2)*xexp(i,2)+
     +           tsp(nop3+2)*by+tsp(nop3+3)*cy
            xexp(k,3)=msp(nop3+3)*xexp(i,3)+tsp(nop3+3)*cz
         enddo
      enddo

      if (first) then
         nvdw=0
         print 14,hbmin,hbmax
         write(16,14) hbmin,hbmax
         if (iprint.gt.1) print 15
         do i=1,io
            do j=i,io
               do k=0,nspgr
                  do m=1,3
                     xxij(m)=xexp(i,m)-xexp(j+k*io,m)
                  enddo
                  do itx=-4,4
                     do ity=-4,4
                        do itz=-4,4
                           dx=xxij(1)+itx*ax+ity*bx+itz*cx
                           dy=xxij(2)+ity*by+itz*cy
                           dz=xxij(3)+itz*cz
                           rij=sqrt(dx**2+dy**2+dz**2)
                           if (rij.lt.hbmax .and. rij.gt.hbmin) then
                              npar=npar+1
                              obs(npar)=rij
                              nvdw=nvdw+1
                              if (nvdw.gt.maxhb) then
                                 print*, ' ***** Increase maxhb in GEOM'
                                 stop
                              endif
                              ihb1(nvdw)=i
                              ihb2(nvdw)=j+k*io
                              ihbtr(nvdw,1)=itx
                              ihbtr(nvdw,2)=ity
                              ihbtr(nvdw,3)=itz
                              if (iprint.gt.1) print 11,i,j,j+k*io,
     +                             itx,ity,itz,rij,dx,dy,dz
                           endif
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
         print 12,io,nvdw
         write(16,12) io,nvdw
      else
         do nhb=1,nvdw
            i=ihb1(nhb)
            j=ihb2(nhb)
            itx=ihbtr(nhb,1)
            ity=ihbtr(nhb,2)
            itz=ihbtr(nhb,3)
            do m=1,3
               xxij(m)=xexp(i,m)-xexp(j,m)
            enddo
            dx=xxij(1)+itx*ax+ity*bx+itz*cx
            dy=xxij(2)+ity*by+itz*cy
            dz=xxij(3)+itz*cz
            rij=sqrt(dx**2+dy**2+dz**2)
            npar=npar+1
            d=obs(npar)-rij
            r=abs(d/svdw)
            sdev(8)=sdev(8)+r**2
            ndev(8)=ndev(8)+1
            if (abs(d).gt.abs(grdev(8))) grdev(8)=d
            if (r.gt.devmax) devmax=r
            if (iprint.gt.2) print 90,npar,i,j,itx,ity,itz,obs(npar),
     +           rij,d,svdw,r,oname(i),oname(j),otype(i),otype(j)
            write(16,90) npar,i,j,itx,ity,itz,obs(npar),rij,d,
     +           svdw,r,oname(i),oname(j),otype(i),otype(j)
         enddo
      endif

C     Crystallographic cell axes and angles
      if (iprint.gt.2 .and. .not. first) print*, '   Cell parameters'
      if (.not. first) write(16,*) '   Cell parameters'
      bb=sqrt(bx**2+by**2)
      cc=sqrt(cx**2+cy**2+cz**2)
      cosa=(bx*cx+by*cy)/(bb*cc)
      cosb=cx/cc
      cosc=bx/bb
      alpha=rad*acos(cosa)
      beta=rad*acos(cosb)
      gamma=rad*acos(cosc)
      if (.not. first) then
         f=10000*((cel(1)/ax-1)**2+(cel(2)/bb-1)**2+(cel(3)/cc-1)**2)
     +        + (cel(4)-alpha)**2 + (cel(5)-beta)**2 + (cel(6)-gamma)**2
      endif
      cel(1)=ax
      cel(2)=bb
      cel(3)=cc
      cel(4)=alpha
      cel(5)=beta
      cel(6)=gamma
      if (scell.gt.tol) then
         do m=1,3
            if (iref(m).eq.1) then
               npar=npar+1
               if (first) then
                  obs(npar)=cel(m)
               else
                  d=obs(npar)-cel(m)
                  dr=d/obs(npar)
                  r=abs(dr/scell)
                  sdev(9)=sdev(9)+r**2
                  ndev(9)=ndev(9)+1
                  if (abs(dr).gt.abs(grdev(9))) grdev(9)=dr
                  if (r.gt.devmax) devmax=r
                  if (iprint.gt.2) print 10,npar,m,
     +                 obs(npar),cel(m),d,scell,r
                  write(16,10) npar,m,obs(npar),cel(m),d,scell,r
               endif
            endif
         enddo
      endif
      if (sangle.gt.tol) then
         do m=4,6
            if (iref(m).eq.1) then
               npar=npar+1
               if (first) then
                  obs(npar)=cel(m)
               else
                  d=obs(npar)-cel(m)
                  r=abs(d/sangle)
                  sdev(10)=sdev(10)+r**2
                  ndev(10)=ndev(10)+1
                  if (abs(d).gt.abs(grdev(10))) grdev(10)=d
                  if (r.gt.devmax) devmax=r 
                  if (iprint.gt.2) print 13,npar,m,
     +                 obs(npar),cel(m),d,sangle,r
                  write(16,13) npar,m,obs(npar),cel(m),d,sangle,r
               endif
            endif
         enddo
      endif
      if (.not. first) then
         print 17,vol1,vol2,vol1-vol2
         write(16,17) vol1,vol2,vol1-vol2
c         pause
      endif
      
C     Centers of gravity: through differences in fractional coordinates
      if (szwpt.gt.tol) then
         do m=1,3
            xxkl(m)=0
         enddo
         do jmol=1,nmols
            do m=1,3
               xxij(m)=box(6*jmol+m)
            enddo
            call tri(1,0,xxij,1,cosa,cosb,cosc)
            if (first) then
               frac(jmol,1)=xxij(1)/ax
               frac(jmol,2)=xxij(2)/bb
               frac(jmol,3)=xxij(3)/cc
            else
               xxij(1)=frac(jmol,1)*ax-xxij(1)
               xxij(2)=frac(jmol,2)*bb-xxij(2)
               xxij(3)=frac(jmol,3)*cc-xxij(3)
            endif
            do m=7,9
               if (iref(m).eq.1 .or. jmol.gt.1) then
                  npar=npar+1
                  zw=box(6*jmol-6+m)
                  if (first) then
                     obs(npar)=zw
                  else
                     d=xxij(m-6)-xxkl(m-6)
                     r=abs(d/szwpt)
                     sdev(11)=sdev(11)+r**2
                     ndev(11)=ndev(11)+1
                     if (abs(d).gt.abs(grdev(11))) grdev(11)=d
                     if (r.gt.devmax) devmax=r
                     if (iprint.gt.2) print 10,npar,m,
     +                    obs(npar),zw,d,szwpt,r
                     write(16,10) npar,m,obs(npar),zw,d,szwpt,r
                  endif
               elseif (.not. first) then
                  k=m-6
                  xxkl(k)=xxij(k)
                  if (iprint.gt.2) print 63,m,xxij(k)
                  write(16,63) m,xxij(k)
               endif
            enddo
            d=sqrt(xxij(1)**2 + xxij(2)**2 + xxij(3)**2 + 
     +           2*xxij(1)*xxij(2)*cosc + 
     +           2*xxij(2)*xxij(3)*cosa + 2*xxij(3)*xxij(1)*cosb)
            if (.not. first) then
               if (iprint.gt.2) print 62,jmol,d
               write(16,62) jmol,d
               f=f+(100*d)**2
            endif
         enddo
      endif

C     Euler angles: difference between the two rotation angles
      if (seul.gt.tol) then
         do jmol=1,nmols
c            print*,(box(6*jmol+3+m),m=1,3)
            cphi=cos(box(6*jmol-6+10)/rad)
            sphi=sin(box(6*jmol-6+10)/rad)
            ctheta=cos(box(6*jmol-6+11)/rad)
            stheta=sin(box(6*jmol-6+11)/rad)
            cpsi=cos(box(6*jmol-6+12)/rad)
            spsi=sin(box(6*jmol-6+12)/rad)
C           Check for monoatomic molecule [?????]
c            if (cphi+ctheta+cpsi.gt.2) then
c               print*,cphi+ctheta+cpsi
c               goto 500
c            endif
            euler1(jmol,1,1)= cpsi*cphi-spsi*sphi*ctheta
            euler1(jmol,1,2)= cpsi*sphi+spsi*cphi*ctheta
            euler1(jmol,1,3)= spsi*stheta
            euler1(jmol,2,1)=-spsi*cphi-cpsi*sphi*ctheta
            euler1(jmol,2,2)=-spsi*sphi+cpsi*cphi*ctheta
            euler1(jmol,2,3)= cpsi*stheta
            euler1(jmol,3,1)= sphi*stheta
            euler1(jmol,3,2)=-cphi*stheta
            euler1(jmol,3,3)= ctheta
            if (first) then
               rot2=0
               do i=1,3
                  do j=1,3
                     euler0(jmol,i,j)=euler1(jmol,i,j)
                  enddo
               enddo
            else
               spoor=0
               do i=1,3
                  do j=1,3
                     spoor=spoor+euler0(jmol,i,j)*euler1(jmol,i,j)
                  enddo
               enddo
               aco=(spoor-1)/2
C              This is numerically bad because SPOOR is very near to 3
               if (aco.gt.1) aco=1
               rot1=rad*acos(aco)
               do i=1,3
                  do j=1,3
                     euler(i,j)=0
                  enddo
               enddo
               do i=1,3
                  do j=1,3
                     do m=1,3
                        euler(i,j)=euler(i,j)+
     +                       euler0(jmol,i,m)*euler1(jmol,j,m)
                     enddo
                  enddo
               enddo
C              EULER is the rotation from structure 1 to structure 0
               offd=(euler(2,1)-euler(1,2))**2 + 
     +              (euler(3,1)-euler(1,3))**2 +
     +              (euler(2,3)-euler(3,2))**2
               rot2=rad*asin(sqrt(offd)/2)
               f=f+(rot2/2)**2
               write(16,16) spoor,rot1,rot2
               if (iprint.gt.2) print 16,spoor,rot1,rot2
            endif
C           Differences in the individual Euler angles
            do m=10,12
               npar=npar+1
               eul=box(6*jmol-6+m)
               if (first) then
                  obs(npar)=eul
               else
                  d=obs(npar)-eul
                  if (d.gt. 180) d=d-360
                  if (d.lt.-180) d=d+360
                  r=abs(d/seul)
C              Do not count the next contributions: overall rotation is a more
C                 reliable  measure (for theta=0 only PHI + PSI is determined)
c              sdev(11)=sdev(11)+r**2
c              ndev(11)=ndev(11)+1
c              if (abs(d).gt.abs(grdev(11))) grdev(11)=d
                  if (iprint.gt.2) print 60,npar,m,obs(npar),eul,d
                  write(16,60) npar,m,obs(npar),eul,d
               endif
            enddo
C           Rotation angle between before and after minimization
            npar=npar+1
            d=rot2
            r=abs(d/seul)
            sdev(12)=sdev(12)+r**2
            ndev(12)=ndev(12)+1
            if (abs(d).gt.abs(grdev(12))) grdev(12)=d
            if (r.gt.devmax) devmax=r
            if (.not. first) then
               if (iprint.gt.2) print 61,npar,jmol,d,seul,r
               write(16,61) npar,jmol,d,seul,r
            endif
c  500       continue
         enddo
      endif
      n1=0
      n2=0
      s1=0
      s2=0
      do i=1,12
         n1=n1+ndev(i)
         s1=s1+sdev(i)
         if (ndev(i).ne.0) then
            s2=s2+(grdev(i)/sigma(i))**2
            n2=n2+1
            sdev(i)=sqrt(sdev(i)/ndev(i))
         endif
      enddo

C     S1 is (more or less) as defined in the 1999 UPACK publication.
      s1=sqrt(s1/n1)
C     S2 is the rms of the [largest deviation(i) / sigma(i)]
      s2=sqrt(s2/n2)
      if (first) then
         print*,npar,' geometry parameters defined'
         write(16,*) npar,' geometry parameters defined'
      else
         print 40,s1,devmax,n1,f
         print 35
         print 41,(ndev(i),i=1,12)
         print 42,(sdev(i),i=1,12)
         print 43,(grdev(i),i=1,12)
         print 44,(sigma(i),i=1,12)
         write(16,40) s1,devmax,n1,f
         write(16,35)
         write(16,41) (ndev(i),i=1,12)
         write(16,42) (sdev(i),i=1,12)
         write(16,43) (grdev(i),i=1,12)
         write(16,44) (sigma(i),i=1,12)
      endif

   10 format(i6,3x,i4,32x,4f8.4,f10.4)
   11 format(3i5,2x,3i5,f12.5,2x,6f12.5)
   12 format(' Found',i8,' atoms,',i6, ' distances')
   13 format(i6,3x, i4,32x,4f8.2,f10.4)
   14 format(' Searching for van der Waals distances between',2f8.3)
   15 format('  At.1 At.2 + Eq     Cell shifts     Distance       ',
     +     'x-comp      y-comp      z-comp')
   16 format('  Trace',f10.3,'    Euler-rotation: ',2f8.2)
   17 format('    Volume',35x,5f8.4)
   20 format(' Geometry parameters:',11x,'topol    ',
     +     'code   start   final    diff   sigma   dif/sig')
   30 format(i6,3x,2i4,12x,f8.4,i8,4f8.4,f10.4,5x,2a5,14x,2a5)
   35 format(33x,'Bonds  Bonds-H   Angles Angles-H   Improp     ',
     +     'Tors   Tors-H  vdWaals Cell-rel   Angles  Centers   Eulers')
   40 format(' RMS delta/sigma D = ',f10.3,' (max.',f10.3,
     +     ')   from',i6,' comparisons; Williams F =',f10.3)
   41 format(' Number of comparisons       ',20i9)
   42 format(' RMS deviation delta/sigma   ',20f9.2)
   43 format(' Maximum deviation           ',
     +     2f9.4,5f9.2,2f9.4,f9.2,f9.4,f9.2)
   44 format(' Acceptable deviation (sigma)',
     +     2f9.4,5f9.2,2f9.4,f9.2,f9.4,f9.2)
   50 format(i6,3x,3i4,8x,f8.1,i8,4f8.1,f10.4,5x,3a5,9x,3a5)
   60 format(i6,3x, i4,32x,4f8.1,f10.4)
   61 format(i6,10x,'Total shift in Euler angles for molecule',i4,':',
     +     2f8.1,f10.4)
   62 format(16x,'Total shift in center for molecule',i4,':',
     +     6x,2f8.4,f10.4)
   63 format(i13,3x,'Irrelevant coordinate shift',18x,3f8.4)
   70 format(i6,3x,4i4,4x,f8.1,i8,4f8.1,f10.4,5x,4a5,4x,4a5)
   80 format(i6,3x,4i4,f8.3,i4,i8,4f8.1,f10.4,5x,4a5,4x,4a5)
   90 format(i6,3x,2i4,7x,3i4,9x,4f8.4,f10.4,5x,2a5,14x,2a5)
      end
