      subroutine leesr(ispgr,spname,iref,itest,msom,inner)

C     Reads space group data for triclinic, monoclinic, or orthorhombic
C     space groups and selects data for the required space group.

C     Input through parameters:
C     ISPGR: space group number (International Tables). For historic
C        reasons No. 14 refers to P21/a and No. 15 refers to A2/a;
C        the normal settings P21/c and C2/c are found as Nos. 1014 and 1015,
C        repectively.
C        Only triclinic, monoclinic and orthorhombic space groups can be used,
C        but Fdd2 has never been tried and will probably not work.
C     SPNAME: space group name.  This name may contain upper and/or lower case
C        letters and slashes may be omitted, e.g. P21/c =  p21/c = p21C.
C        If ISPGR < 0, SPNAME may also contain the space group number!
C     NOTE: If ISPGR and SPNAME are contradictory, the name has priority.

C---------------------------------------------------------------------------
C     Input from file IN.SPG (unit 11) for every implemented space group;
C     here a - signifies a new line in the input file.
C     -  NAAM: space group name
C     -  IS: space group number
C        NSPGR: number of space group operations (E excluded)
C        IMP: importance number for clustering space groups (not used here)
C     -  MSP(x), TSP(x),    MSP(y), TSP(y),    MSP(z), TSP(z)
C           oblique coordinates are expanded following
C           r'= MSP.r + TSP, so each MSP = +1 or -1 and each TSP = 0 or 0.5.
C     -  IREF(1..12) =  0/1: disables/enables optimization of parameters:
C           cell axes:                1: AX    2: BY      3: CZ
C           cell angles:              4: BX    5: CX      6: CY
C           molecular centers:        7: X     8: Y       9: Z
C           molecular orientations:  10: PHI  11:THETA   12:PSI
C     -  ITEST(1..3): conditions for cell edges:
C           ITEST(1) = 1: enables skipping of cells with AX > BY
C           ITEST(2) = 1: enables skipping of cells with BY > CZ
C           ITEST(3) = 1: enables skipping of cells with AX > CZ
C           This information is only used in the outdated program PACK1.
C---------------------------------------------------------------------------

C     Output through parameters:
C        IREF, ITEST: see above
C        MSOM(m) = sum over MSP(m), compares with IREF.
C        INNER = 0 if the cell has no dipole moment, otherwise it is unchanged
C        ISPGR and SPNAME: changed from input to consistent values, if
C            necessary. If no space group is found, ISPGR is set to -3.

C     Output to common block /SPGR/:
C        NSPGR, MSP, TSP: see above
C        Various information for FDAT

      implicit none
      include 'params.i'
      include 'cspgr.i'
      integer i,ifound,ii,inner,iref,is,ispgr,itest,j,jj,k,m,msom,
     +     nfound,nop3,nspuit
      dimension iref(*),itest(*),msom(3)
      character*8 naam,upcnm,noslnm,ch*1,spname,naamsp
      logical found

C     Remove _ symbols from spname
      j=0
      naamsp=' '
      do i=1,8
         if (spname(i:i).ne.'_') then
            j=j+1
            naamsp(j:j)=spname(i:i)
         endif
      enddo
c      naamsp=spname
      if (spname.ne.'noname' .and. spname.ne.naamsp) 
     +     print*,spname,' renamed to ',naamsp

      if (ispgr.eq.0) then
         nspgr=0
         inner=0
         msom(1)=1
         msom(2)=1
         msom(3)=1
         print*,'No crystal symmetry, free molecule'
         write(16,*) 'No crystal symmetry, free molecule'
         return
      endif
C     Maybe the space group number is in SPNAME
      if (ispgr.le.0) then
         read(spname,70,err=110) i
         print*,' Space group number taken from space group name ',
     +        spname
         ispgr=i
  110    continue
      endif
      open(unit=11,file='in.spg',status='OLD',err=920)
      ifound=0
      nfound=0

  130 continue
C     Read next entry from space group file
      read(11,20,end=150,err=910) naam
      if (naam.eq.'  ') goto 130
      read(11,*,end=910,err=910) is,nspgr,i
      do i=1,nspgr
         j=3*i-3
         read(11,*) (msp(j+m),tsp(j+m),m=1,3)
      enddo
      read(11,*) (iref(m),m=1,12)
      read(11,*) (itest(m),m=1,3)
C     Look for space group number
      if (is.eq.ispgr) then
         ifound=is
      endif
C     Look for first occurrence of space group name
      if (nfound.ne.0) goto 130
C     Change to upper case
      do i=1,8
         ch=naam(i:i)
         k=ichar(ch)
         if (ch.ge.'a' .and. ch.le.'z') k=k-32
         upcnm(i:i)=char(k)
         ch=naamsp(i:i)
         k=ichar(ch)
         if (ch.ge.'a' .and. ch.le.'z') k=k-32
         naamsp(i:i)=char(k)
      enddo
      found=(upcnm.eq.naamsp)
      if (.not. found) then
C        Maybe a slash was absent
         k=0
         do i=1,8
            ch=upcnm(i:i)
            if (ch.ne.'/') then
               k=k+1
               noslnm(k:k)=ch
            endif
         enddo
         found=(noslnm.eq.naamsp)
      endif
      if (found) nfound=is
      goto 130

  150 continue
C     End of space group file reached, check results
      if (ifound.eq.nfound) then
         print*,'LEESR found ',ifound,nfound
C        No space group found!
         if (ifound.eq.0) then
            print 90,naamsp,ispgr
            write(16,90) naamsp,ispgr                
            close(11)
            ispgr=-3
            return
         endif
      elseif (ifound.eq.0) then 
         ispgr=nfound
      elseif (nfound.eq.0) then
         ispgr=ifound
      else
         print*,' ***** Inconsistency between space group number',
     +        ifound,' and space group name'
         print*,' Assume name to be correct: ',naamsp
         ispgr=nfound
      endif

C     The correct space group number ISPGR has been found. Get data.
      rewind(11)
  170 continue
C     Read next entry from space group file
      read(11,20,end=910,err=910) naam
      if (naam.eq.'  ') goto 170
      read(11,*,end=910,err=910) is,nspgr,i
      do i=1,nspgr
         j=3*i-3
         read(11,*) (msp(j+m),tsp(j+m),m=1,3)
      enddo
      read(11,*) (iref(m),m=1,12)
      read(11,*) (itest(m),m=1,3)
      if (is.ne.ispgr) goto 170

C     The space group data have been found.
      close(11)
      if (3*(nspgr+1).gt.mxspgr) then
         print*,' ***** Increase mxspgr',mxspgr
         stop
      endif
      print 40,ispgr,naam
      write(16,40) ispgr,naam

C     Investigate symmetry for dipole moments
      k=0
      do m=1,3
         msom(m)=1
         do nop=1,nspgr
            nop3=3*nop-3
            msom(m)=msom(m)+msp(nop3+m)
         enddo
         if (msom(m).ne.0 .and. iref(m+6).ne.0) then
            print 30,m,msom(m),iref(m+6)
            stop
         endif
         k=k+msom(m)
      enddo
      if (k.eq.0) inner=0
      print 50,(msom(m),m=1,3)
      write(16,50) (msom(m),m=1,3)
      if (inner.eq.0) then
         print 60
         write(16,60)
      endif

C     Create information for FDAT format:
      naamsp=naam
      spname=naam
C     No centrosymmetry
      icentr=2
      do i=1,nspgr
         j=3*i-3
         do m=1,3
            if ((msp(j+m).ne.-1).and.(tsp(j+m).ne.0)) then
               goto 230
            else
C              Centrosymmetric:
               if (m.eq.3) icentr=1
            endif
         enddo
  230    continue
      enddo
      do i=1,3
         mspuit(i)=1
         tspuit(i)=0
      enddo
      nspuit=1
      do i=1,nspgr
         ii=3*i-3
         do j=1,nspuit
            jj=3*j-3
            do m=1,3
               if ((tsp(ii+m).ne.tspuit(jj+m)).or.
     +              (msp(ii+m).ne.-mspuit(jj+m))) goto 270
               if (m.eq.3) goto 290
            enddo
  270       continue
         enddo
         nspuit=nspuit+1
         do k=1,3
            tspuit(3*(nspuit-1)+k)=tsp(ii+k)
            mspuit(3*(nspuit-1)+k)=msp(ii+k)
         enddo
  290    continue
      enddo

C     Monoclinic, triclinic,orthorombic
      if ((iref(4).eq.0).and.(iref(5).eq.0).and.(iref(6).eq.0))
     +     icryst=3
      if ((iref(4).eq.0).and.(iref(6).eq.0).and.(iref(5).gt.0))
     +     icryst=2
      if ((iref(4).gt.0).and.(iref(5).gt.0).and.(iref(6).gt.0))
     +     icryst=1
      return

  910 print*,' ***** Error reading space group file IN.SPG'
      stop
  920 print*,' ***** No space group file IN.SPG present'
      stop
   20 format(a8)
   30 format(' ***** Error in symmetry properties',3i6)
   40 format(' Space group',i5,2x,a10)
   50 format(' Dipole coefficients',3i5)
   60 format(' No calculation of U2-inner')
   70 format(i8)
   90 format(' ***** Space group ',a,'  number',i4,' not found')
      end
