      subroutine leest(nrploc,icoul,iprint)
C     B. P. van Eijck, November 1997.
C     Input files: IN.TP (unit 20), molecular topology
C                  IN.FF (unit 12), force field

C     Input through parameters:
C        ICOUL = 0 disable Coulomb-interactions (set charges to zero)
C                  This includes also the polarizability.
C        IPRINT regulates output to unit 16 and to the screen.

C---------------------------------------------------------------------------
C     Reads file IN.TP which contains a formatted topology file,
C     created by progrsam CODES and/or by hand.
C     Input units: Angstrom, kcal/mole, atomic charge units
C     Output units: nanometers, different charge units, and kJ/mole.
C---------------------------------------------------------------------------
C     A force field file IN.FF must be present:
C     - text
C     - IBUCK = 0: use Lennard-Jones parameters (BUCK set to FALSE)
C                  with Gromos-type combination rule
C     -       =-1: use Lennard-Jones parameters (BUCK set to FALSE)
C                  with Charmm-type combination rule
C             = 1: use Buckingham parameters (BUCK set to TRUE)
C     - CUTOFF radius
C       ALPHAC(nm**-1): Ewald-Coulmb damping by ALPHAC.R
C       ALPHA6: same for Lennard-Jones r**-6 terms
C       HMAXC: Coulomb cutoff radius in reciprocal space (nm**-1)
C       HMAX6: same for Lennard-Jones r**-6 terms
C         If ALPHAC or ALPHA6=0 the corresponding Ewald-summation is skipped
C         If ALPHAC>0 INNER should be set to zero
C       TOLE: omit distance from pair list if contribution > TOLE
C       TOLK: omit vector from reciprocal vector list if contribution > TOLK
C     - NPP = 0: do not use polarization
C           = 1: intermolecular polarization only, neglecting the effect of
C                the induced dipole moments on the field strength.
C           > 1: maximum numbers of cycles in full iterative calculation of
C                polarization with subroutines NONBEP and LISTP.
C                Then the following data are also read here:
C       POLTOL, SCHAAL, GAMPOL, NNN, MMM, NTE, NTTT
C                See NONBEP for a more complete description.
C     - FACT: factor to obtain kJ/mol from input energy units
C       FACR: factor to obtain nanometers from input length units
C     - Optional:
C       SC14C: scaling factor for 1...4 Coulomb interactions
C       SC14LJ: scaling factor for 1...4 Lennard-Jones interactions
C               (if SC14LJ < 0 the absolute value is taken)
C       IDDC = 1: distance dependent dielectric constant
C       EPSIL: dielectric constant = epsil     if IDDC=0
C              dielectric constant = epsil * R if IDDC=1
C              Default value 1.0 in both cases; note: R in Angstrom!
C
C     Now the force field data are read.
C     On each line (possible mixed with blank lines):

C     codes 100-199, bond force constant, bond length
C     codes 200-299, angle force constant, angle value
C     codes 300-399, improper force constant, angle value
C     codes 400-499, torsional force constant, periodicity
C        If the periodicity is negative, the force constant is divided
C           by the multiplicity (which is the number of times that the
C           central atoms occur in torsional angles).
C           This usually presumes equal force constants.
C        If the periodicity is positive, the multiplicity is always 1.
C     codes 500-599, charge
C     If IBUCK=0:
C        atom type 1, atom type 2, C12, C6 [,C12(1...4), C6(1...4)]
C               Codes for the combination of two equal atoms must be present;
C               if other interaction parameters are absent or negative, 
C               they are found from the geometric combination rule.
C     If IBUCK=-1:
C        atom type 1, atom type 2, Emin, Rmin [,Emin(1...4), Rmin(1...4)]
C               Codes for the combination of two equal atoms must be present;
C               other interaction are found from the Charmm combination rule
C               (but note a factor 2 in the arithmetic average).
C        codes 700-799, same for 1...4 interactions
C     In both cases, missing or negative 1...4 parameter are found as follows:
C        if SC14LJ > 0, from diagonal values with the combination rule;
C        if SC14LJ < 0, from the same general off-diagonal element
C     If IBUCK=1:
C        atom type 1, atom type 2, A, B, C
C               Codes for the combination of two equal atoms must be present;
C               if other interaction parameters are absent or negative, 
C               they are found from the geometric combination rule.
C               No special 1..4 interactions in this case.
C         - A: the repulsion-coefficient in kJ/mol and A
C         - B: the repulsion-exponent in A**(-1)
C         - C: the attractie-coefficient in kJ/mol and A
C     codes 800-899, polarizibility ALFPOL; optional: maximum dipole PMAX
C               (see NONBEP for explanation of PMAX)
C---------------------------------------------------------------------------

C     Output trough parameters:
C        NRPLOC: number of atoms in one molecule

C     Output through various common blocks:
C        /NONB/:
C           CUTOFF radius
C           BUCK: TRUE for exponential, FALSE for R^(-12) repulsion
C           DDC: normally FALSE, but TRUE for distance dependent
C                dielectric constant 1/Rij (Angstrom)
C           TOLE,TOLK: see above
C           CG(1..NRP): charges
C           SQ6(1..NRP): square roots of C6 (for Ewald)
C           AMASS(1..NRPLOC): atomic masses
C        /WALD/:
C           ALPHAC, ALPHA6, HMAXC, HMAX6: see above
C        /NONP/ gives information on nonbonded parameters:
C           NCAG: number of charge groups in one molecule
C           IAC(1..NRP): atom codes, taking values 1..Ncod
C           IDO(I)=0: atom I does not contribute to nonbonded energy
C           IEXCL(1..NRP,1..NRP): excluded neighbors.
C           The elements of array IEXCL(i,j) are:
C               =0: intermolecular interaction
C               >0: intramolecular interaction:
C               =1: 1..1, 1..2, 1..3 excluded neighbors
C               =2: 1..4 neighbors
C               =3: 1..5 and further neighbors
C           INC(0..NCAG): charge group pointer list: INC(I) specifies the
C               position of the last atom of the I-th group in the
C               atom sequence.
C               (This algorithm is taken from the GROMOS procedure NONBML)
C           C6(1..Ncod,1..Ncod): attractive C6 parameters
C           CS6(1..Ncod,1..Ncod): same for Lennard-Jones 1..4 interactions
C           C12(1..Ncod,1..Ncod),CS12(1..Ncod,1..Ncod):
C              for Lennard-Jones potential: same for repulsive terms
C              for Buckingham potential: repulsion C12 * exp(-CS12 * R)
C           SC14C, SC14LJ: multiply Lennard-Jones 1..4 Coulomb and other
C              terms with these factors, respectively.
C        /ATTP/:
C           PANM(1..NRPLOC): atom names
C           TYPE(1..NRPLOC): atom types
C        /BIND/:
C           NBOND: number of bonds
C           IB, JB (1..): atom numbers in pairs for each bond
C           CB, B0 (1..): force constants and reference bond lengths
C        /HOEK/:
C           NHOEK: number of angles
C           IT, JT, KT (1..): atom numbers for each angle
C           CT, T0 (1..): force constants and reference angles
C        /IMPR/:
C           NIMPR: number of improper dihedrals
C           IQ, JQ, KQ, LQ (1..): atom numbers for each dihedral
C           CQ, Q0 (1..): force constants and reference bond lengths
C        /TORS/:
C           NTORS: number of proper dihedrals
C           IP, JP, KP, LP (1..): atom numbers for each dihedral
C           CP, NP (1..): force constants and periodicities
C        /POLA/: polarization data, see above
C        /INDS/ gives information on special dihedrals:
C           NDIHZ:  number of dihedral angles that determine the separate
C              conformations
C           NDIHH: number of less essential dihedrals
C           NDIHA=NDIHZ + NDIHH
C           IOME, JOME, KOME, LOME(1..NDIHA):  atomic numbers to define
C              the dihedral
C           IREFTR(1..), the reference atoms (see TRANS)

C     Calls subroutines STREXT, REALST and function NWORD.

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cnonb.i'
      include 'cnonp.i'
      include 'cinds.i'
      include 'cwald.i'
      include 'cpola.i'
      include 'chadd.i'

      integer i,i1,i2,i3,i4,iaci,ibuck,icg,icoul,iddc,idum,ii,iiprev,
     +     im,ios,ioutd,iprint,ireadc,j,j1,j2,jj,jm,k,km,m,m1,m2,maxff,
     +     maxvdw,mul,n,nmol,npt,nratt,nrploc,ntor,nvdw,nword
      double precision aiaj,aij2,c,cha,cpt,da,eij,f12,f6,facb,facq,facr,
     +     fact,ff,pi,rad,rij,totc,totcg
      parameter(maxvdw=300,maxff=900)
      dimension ff(maxff,3)
      character*8 typein(maxnrt),woord,vdw1,vdw2,filenm*9,scr*5
      character*80 regel,titel,fftors(400:499),ffvdw(maxvdw)

C     Read force field file
      do i=400,499
         fftors(i)='notread'
      enddo
      do i=1,maxff
         ff(i,1)=-999
         ff(i,2)=-999
         ff(i,3)=-999
      enddo
      do i=1,12
         ireftr(i)=0
      enddo
      open(unit=12,file='in.ff',status='OLD',err=860)
      read(12,11) regel
      print 28,regel
      write(16,28) regel
      read(12,*) ibuck
      buck=(ibuck.eq.1)
      if (buck) then
         print*,'   Buckingham-potential'
         write(16,*) '   Buckingham-potential'
      else
         if (ibuck.eq.0) then
            print*,'   Lennard-Jones potential,',
     +           ' Gromos-recipe for combination rule'
            write(16,*) '   Lennard-Jones potential,',
     +           ' Gromos-recipe for combination rule'
         else
            print*,'   Lennard-Jones potential,',
     +           ' Charmm-recipe for combination rule'
            write(16,*) '   Lennard-Jones potential,',
     +           ' Charmm-recipe for combination rule'
         endif
      endif
      alphac=0
      alpha6=0
      tole=0
      tolk=0
      read(12,11) regel
      n=nword(regel)
      call realst(regel,1,cutoff)
      if (n.ge.7) then
         call realst(regel,2,alphac)
         call realst(regel,3,alpha6)
         call realst(regel,4,hmaxc)
         call realst(regel,5,hmax6)
         call realst(regel,6,tole)
         call realst(regel,7,tolk)
      endif
      poltol=0.01d0
      schaal=0.1662d0
      gampol=0.85d0
      nnn=3
      mmm=32
      nte=1
      nttt=1
      read(12,11) regel
      n=nword(regel)
      call intstr(regel,1,npp)
      if (n.ge.8) then
         call realst(regel,2,poltol)
         call realst(regel,3,schaal)
         call realst(regel,4,gampol)
         call intstr(regel,5,nnn)
         call intstr(regel,6,mmm)
         call intstr(regel,7,nte)
         call intstr(regel,8,nttt)
      endif
      read(12,*) fact,facr
      iddc=0
      epsil=1
      sc14c=1
      sc14lj=1
      nvdw=0
      read(12,11) regel
      n=nword(regel)
      if (n.eq.0) goto 106
C     Scaling factors for 1...4 interactions
      call realst(regel,1,sc14c)
      call realst(regel,2,sc14lj)
      print 30,sc14c,sc14lj
      write(16,30) sc14c,sc14lj
C     Distance dependent dielectric constant if IDDC=1
      if (n.ge.3) call intstr(regel,3,iddc)
      ddc=(iddc.eq.1)
      if (ddc) print*,'Distance dielectric constant used'
      if (ddc .and. (alphac.gt.0 .or. alpha6.gt.0)) then
         print*,'***** Do not combine Dist-Dielect-Scaling with Ewald'
         stop
      endif
C     Dielectric constant from force field, possibly overridden by input data
      if (n.ge.4) call realst(regel,4,epsil)
      if (ddc) epsil=epsil*10
      write(16,31) epsil
      print 31,epsil
      pi=4*atan(1.0d0)
      rad=180/pi
      facb=fact/(facr**2)
      facq=sqrt(332.0636d0*0.4184d0/epsil)
      f6=fact*(facr**6)
      f12=fact*(facr**12)

C     Read force field codes
  106 read(12,11,end=108,err=108) regel
      n=nword(regel)
      if (n.eq.0) goto 106
      call strext(regel,1,woord)
      read(woord,25,err=107) i
      if (i.ge.400 .and. i.lt.500) then
         fftors(i)=regel
      else
         if (n.gt.4) n=4
         do m=2,n
            call realst(regel,m,ff(i,m-1))
         enddo
      endif
      goto 106
  107 continue
C     Read next line with Van der Waals data
      nvdw=nvdw+1
      if (nvdw.gt.maxvdw) then
         print*,' Increase MAXVDW',nvdw
         stop
      endif
      ffvdw(nvdw)=regel
      goto 106
  108 continue
      close(12)
      print*,nvdw,' lines with van der Waals parameters found'
      write(16,*) nvdw,' lines with van der Waals parameters found'

C     Start reading topology file
      open(unit=20,file='in.tp',status='OLD',err=840)
      read(20,11) titel
      print 29,titel
      write(16,29) titel
      read(20,11) regel
      if (regel(8:17) .eq. 'atom types') then
         print*,' ***** Outdated version of topology file'
         write(16,*) ' ***** Outdated version of topology file'
         ioutd=1
         read(regel,25) nratt
         if (nratt.gt.maxnrt) then
            print*,' ***** Increase maxnrt'
            stop
         endif
         read(20,11) regel
         if (nratt.ne.nword(regel)) then
            print*,' ***** Error in LEEST:',nratt,nword(regel)
            stop
         endif
         do i=1,nratt
            call strext(regel,i,typein(i))
         enddo
         read(20,11) regel
         do i=1,nratt
            read(20,11) regel
         enddo
         read(20,*) nrploc
      elseif (regel(8:16) .eq. 'atoms in ') then
C        New version of topology file and force field file
         ioutd=0
         read(regel,25) nrploc
         nratt=0
      else
         print*,' ***** Error in topology file: ',regel(8:40)
         stop
      endif

C     Read information on atoms
      if (nrploc.gt.maxnrp) then
         print*,'  ***** Increase maxnrp',nrploc
         stop
      endif
      read(20,11) regel
      do i=1,nrploc
         read(20,11) regel
         call intstr(regel,1,j)
         if (i.ne.abs(j)) then
            print*,' ***** Error in atom numbering in LEEST',i,j
            stop
         endif
         call strext(regel,2,panm(i))
         call intstr(regel,3,idum)
         call strext(regel,4,type(i))
         if (ioutd.eq.1) then
            iac(i)=idum
            if (type(i).ne.typein(iac(i))) then
               print*,' ***** Error in atom codes ',
     +              type(i),i,typein(iac(i))
               stop
            endif
         else
            do j=1,i-1
               if (type(j).eq.type(i)) then
                  iac(i)=iac(j)
                  goto 120
               endif
            enddo
            nratt=nratt+1
            if (nratt.gt.maxnrt) then
               print*,' ***** Increase maxnrt'
               stop
            endif
            iac(j)=nratt
            typein(nratt)=type(i)
  120       continue
            if (iac(i).ne.idum) then
               print*,' ***** Inconsistency in atom codes ',
     +              type(i),i,idum,iac(i)
               stop
            endif
         endif
         call realst(regel,5,amass(i))
         call realst(regel,6,c)
C        Determine what kind of charge input there is
         if (i.eq.1) then
            ireadc=1
            if (c.ge.499.5 .and. c.lt.599.5) ireadc=0
         endif
         if (c.ge.499.5 .and. c.lt.599.5) then
            if (ireadc.eq.1) then
               print*,'***** Inconsistent charge input', ireadc
               stop
            endif
            m=nint(c)
            cg(i)=ff(m,1)
            if (cg(i).lt.-900) then
               print*,' ***** Charge code',m,' missing for atom',i
               stop
            endif
         else
            if (ireadc.eq.0) then
               print*,'***** Inconsistent charge input',ireadc
               stop
            endif
            cg(i)=c
            if (abs(cg(i)).gt.100) then
               print*,' ***** Wrong charge for atom',i,cg(i)
               stop
            endif
         endif
         call realst(regel,7,c)
         if (c.ge.799.5 .and. c.lt.899.5) then
            m=nint(c)
            alfpol(i)=ff(m,1)
            pmax(i)=ff(m,2)
            if (cg(i).lt.-900) then
               print*,' ***** Charge code',m,' missing for atom',i
               stop
            endif
         else
            alfpol(i)=c
            pmax(i)=1000000
         endif
         if (abs(alfpol(i)+999).lt.0.001d0) then
c            print*, 'Polarizability ',i,' set to zero'
            alfpol(i)=0
         endif
         if (abs(pmax(i)+999).lt.0.001d0) then
c            print*, 'Maximum dipole ',i,' set to 1000000'
            pmax(i)=1000000
         endif
      enddo
      print*,nratt,' atom types found'
      write(16,*) nratt,' atom types found'

C     Read van der Waals parameters. 
C     There are NRATT atom types and NVDW data lines.

      do i=1,nratt
         do j=1,nratt
            c12(i,j)=-999
            c6(i,j)=-999
            cs12(i,j)=-999
            cs6(i,j)=-999
         enddo
      enddo
      do i=1,nvdw
         regel=ffvdw(i)
         call strext(regel,1,vdw1)
         call strext(regel,2,vdw2)
         j1=0
         j2=0
         do j=1,nratt
            if (vdw1.eq.typein(j)) j1=j
            if (vdw2.eq.typein(j)) j2=j
         enddo
         if (j1.gt.0 .and. j2.gt.0) then
c            print*,i,j1,j2
            n=nword(regel)
            if (buck) then
               if (n.lt.5) then
                  print*, '***** Error in Buckingham data'
                  stop
               endif
               call realst(regel,3,c12(j1,j2))
               call realst(regel,4,cs12(j1,j2))
               call realst(regel,5,c6(j1,j2))
            else
               if (n.lt.4) then
                  print*,' ***** Error in Lennard-Jones data'
                  stop
               endif
               if (n.ge.4) then
                  call realst(regel,3,c12(j1,j2))
                  call realst(regel,4,c6(j1,j2))
               endif
               if (n.ge.6) then
                  call realst(regel,5,cs12(j1,j2))
                  call realst(regel,6,cs6(j1,j2))
               endif
            endif
            c12(j2,j1)=c12(j1,j2)
            c6(j2,j1)=c6(j1,j2)
            cs12(j2,j1)=cs12(j1,j2)
            cs6(j2,j1)=cs6(j1,j2)
         endif
      enddo

C     Take missing 1...4 Lennard-Jones parameters as normal ones
      if (ibuck.lt.1) then
         do i=1,nratt
            if (cs12(i,i).lt.0) cs12(i,i)=c12(i,i)
            if ( cs6(i,i).lt.0)  cs6(i,i)= c6(i,i)
         enddo
      endif
C     Apply combination rules for missing mixed parameters
      do i=1,nratt
         if (c12(i,i).lt.0 .or. c6(i,i).lt.0) then
            print*,' ***** Van der Waals parameter negative:  ',
     +           typein(i),c12(i,i)
            stop
         endif
         do j=1,nratt
            if (buck) then
C              Buckingham potential; Williams combination rule
               if (c6(i,j).lt.0) c6(i,j)=sqrt(c6(i,i)*c6(j,j))
               if (c12(i,j).lt.0) c12(i,j)=sqrt(c12(i,i)*c12(j,j))
               if (cs12(i,j).lt.0) cs12(i,j)=(cs12(i,i)+cs12(j,j))/2
            else
C              Lennard-Jones potential
               if (c12(i,j).lt.0) c12(i,j)=sqrt(c12(i,i)*c12(j,j))
               if (cs12(i,j).lt.0) then
                  if (sc14lj.gt.0) then
                     cs12(i,j)=sqrt(cs12(i,i)*cs12(j,j))
                  else
                     cs12(i,j)=c12(i,j)
                  endif
               endif
               if (ibuck.lt.0) then
C                 Charmm-type combination rule (but note factor 2)
                  if (c6(i,j).lt.0) c6(i,j)=(c6(i,i)+c6(j,j))/2
                  if (cs6(i,j).lt.0) then
                     if (sc14lj.lt.0) then
                        cs6(i,j)=(cs6(i,i)+cs6(j,j))/2
                     else
                        cs6(i,j)=c6(i,j)
                     endif
                  endif
               else
C                 Geometric averages for combination rule
                  if (c6(i,j).lt.0) c6(i,j)=sqrt(c6(i,i)*c6(j,j))
                  if (cs6(i,j).lt.0) then
                     if (sc14lj.gt.0) then
                        cs6(i,j)=sqrt(cs6(i,i)*cs6(j,j))
                     else
                        cs6(i,j)=c6(i,j)
                     endif
                  endif
               endif
            endif
         enddo
      enddo
      sc14lj=abs(sc14lj)
      if (ibuck.eq.-1) then
C        Convert E(ij) and R(ij) to C12(ij) and C6(ij)
         do i=1,nratt
            do j=1,nratt
               eij=c12(i,j)
               rij=c6(i,j)
               c6(i,j)=2*eij*rij**6
               c12(i,j)=eij*rij**12
               eij=cs12(i,j)
               rij=cs6(i,j)
               cs6(i,j)=2*eij*rij**6
               cs12(i,j)=eij*rij**12
            enddo
         enddo
      endif

C     Test van der Waals parameters
      do i=1,nratt
         do j=1,i-1
            aij2=c6(i,j)**2
            aiaj=c6(i,i)*c6(j,j)
            da=abs(aij2-aiaj)
            if (da.gt.0.1d0 .and. da/(aij2+aiaj).gt.0.0001d0) then
               print 21,typein(i),typein(j),aij2,aiaj
               write(16,21) typein(i),typein(j),aij2,aiaj
            endif
         enddo
      enddo

C     Read exclusion matrix
      read(20,11) regel
      read(20,11) regel
      do i=1,nrploc
         read(20,*) (iexcl(j,i),j=1,i)
         do j=1,i
            iexcl(i,j)=iexcl(j,i)
         enddo
         if (icoul.eq.0) then
            cg(i)=0
            alfpol(i)=0
         endif
      enddo

C     Read charge group information
      read(20,*) ncag
      if (ncag.gt.maxcag) then
         print*,' ***** Increase maxcag',ncag,maxcag
         stop
      endif
      read(20,*) (inc(i),i=1,ncag)
      inc(0)=0
      if (ncag.eq.0) then
         print*,' ***** No charge groups'
         stop
      endif
      if (inc(ncag).ne.nrploc) then
         print*, ' ***** Charge group not closed'
         stop
      endif

C     Read bond information
      read(20,*) nbond
      if (nbond.gt.maxbon) then
         print*,' ***** Increase maxbon',nbond
         stop
      endif
      do i=1,nbond
         read(20,11) regel
         n=nword(regel)
         call intstr(regel,1,ib(i))
         call intstr(regel,2,jb(i))
         call strext(regel,n,scr)
         if (scr.eq.'*****') n=n-1
         if (n.eq.4) then
            call realst(regel,3,cb(i))
            call realst(regel,4,b0(i))
c            call realst(regel,5,cha)
         endif
         if (n.eq.3) then
            call intstr(regel,3,m)
            if (m/100.ne.1) then
               print*,' ***** Error in bond code',i,m
               stop
            endif
            ibcod(i)=m
            cb(i)=ff(m,1)*facb
            b0(i)=ff(m,2)
            cha=ff(m,3)
            if (cha.lt.-900) cha=0
         endif
C        Apply bond increments
         if (icoul.ne.0 .and. ireadc.eq.0) then
            cg(ib(i))=cg(ib(i))-cha
            cg(jb(i))=cg(jb(i))+cha
         endif
         if (cb(i).lt.0 .or. b0(i).lt.0) then
            print 39,' ***** Error in bond ',i,m,cb(i),b0(i)
            stop
         endif
      enddo
      if (iprint.gt.2) print*,' Bond information read'

C     Check charges and set total charge to zero if necessary
      if (icoul.eq.1) then
         totc=0
         totcg=0
         icg=1
         do i=1,nrploc
            if (abs(cg(i)).gt.900) then
               print*,' ***** Wrong charge for atom',i,cg(i)
               stop
            endif
            totc=totc+cg(i)
            totcg=totcg+cg(i)
            if (i.eq.inc(icg)) then
c               if (abs(totc).gt.0.001d0) then
                  print 20,icg,totc
                  write(16,20) icg,totc
c               endif
               totc=0
               icg=icg+1
            endif
         enddo
         if (abs(totcg).gt.0.0001d0) then
            print 22, totcg
            write(16,22) totcg
         endif
         do i=1,nrploc
            cg(i)=cg(i)-totcg/nrploc
         enddo
      endif
         
C     Check whether IDO should be set to zero
      do i=1,maxat
         ido(i)=1
      enddo
      do i=1,nrploc
         if (abs(cg(i)).gt.0.001d0) goto 330
         j=iac(i)
         do m=1,nratt
            if (abs(c6(j,m)).gt.0.001d0) goto 330
            if (abs(c12(j,m)).gt.0.001d0) goto 330
            if (.not. buck) then
               if (abs(cs6(j,m)).gt.0.001d0) goto 330
               if (abs(cs12(j,m)).gt.0.001d0) goto 330
            endif
         enddo
         ido(i)=0
         print 24,i
         write(16,24) i
  330    continue
      enddo

C     Read angle information
      read(20,*) nhoek
      if (nhoek.gt.maxang) then
         print*, ' ***** Increase maxang',nhoek
         stop
      endif
      do i=1,nhoek
         read(20,11) regel
         n=nword(regel)
         call intstr(regel,1,it(i))
         call intstr(regel,2,jt(i))
         call intstr(regel,3,kt(i))
         call strext(regel,n,scr)
         if (scr.eq.'*****') n=n-1
         if (n.eq.5) then
            call realst(regel,4,ct(i))
            call realst(regel,5,t0(i))
         endif 
         if (n.eq.4) then
            call intstr(regel,4,m)
            if (m/100.ne.2) then
               print*,' ***** Error in angle code',i,m
               stop
            endif
            ihcod(i)=m
            ct(i)=ff(m,1)*fact
            t0(i)=ff(m,2)
         endif
         if (ct(i).lt.0 .or. t0(i).lt.0) then
            print 39,' ***** Error in angle ',i,m,ct(i),t0(i)
            stop
         endif
      enddo
      if (iprint.gt.2) print*,' Angle information read'

C     Read improper dihedral information
      read(20,*) nimpr
      if (nimpr.ne.0) read(20,11) regel
      if (nimpr.gt.maximp) then
         print*, ' ***** Increase maximp',nimpr
         stop
      endif
      do i=1,nimpr
         read(20,11) regel
         n=nword(regel)
         call intstr(regel,1,iq(i))
         call intstr(regel,2,jq(i))
         call intstr(regel,3,kq(i))
         call intstr(regel,4,lq(i))
         call strext(regel,n,scr)
         if (scr.eq.'*****') n=n-1
         if (n.eq.6) then
            call realst(regel,5,cq(i))
            call realst(regel,6,q0(i))
         endif 
         if (n.lt.6) then
            call intstr(regel,5,m)
            if (m/100.ne.3) then
               print*,' ***** Error in improper code',i,m
               stop
            endif
            iicod(i)=m
            cq(i)=ff(m,1)*fact
            q0(i)=ff(m,2)
         endif
         if (cq(i).lt.0) then
            print 39,' ***** Error in improper ',i,m,cq(i),q0(i)
            stop
         endif
      enddo
      if (iprint.gt.2) print*,' Improper dihedral information read'

C     Read torsional information
      read(20,*) ntor
      read(20,11) regel
      ntors=0
      do i=1,ntor
         read(20,11) regel
         n=nword(regel)
         mul=1
         if (n.lt.5) then
            print*,' ***** Error in torsional code',i,n
            stop
         endif
         call intstr(regel,1,i1)
         call intstr(regel,2,i2)
         call intstr(regel,3,i3)
         call intstr(regel,4,i4)
         call strext(regel,n,scr)
         if (scr.eq.'*****') n=n-1
         if (n.ge.7) then
C              Topology file with values
            ntors=ntors+1
            ip(ntors)=i1
            jp(ntors)=i2
            kp(ntors)=i3
            lp(ntors)=i4
            call intstr(regel,5,mul)
C           The multiplicity has now already been taken care of.
C           So is the number of torsional periodicities.
            if (mul.ne.1) then
               print*,'Multiplicity must be 1!!!'
               mul=1
               print*,' Type ENTER if you wish to continue anyhow'
               read 11, regel
            endif
            call realst(regel,6,cp(ntors))
            call intstr(regel,7,np(ntors))
         endif
         if (n.lt.7) then
C           Topology file with code numbers, multiplicities and
C           perhaps multiple numbers of torsional periodicities.
            call intstr(regel,5,m)
            if (n.eq.6) call intstr(regel,6,mul)
            regel=fftors(m)
            if (regel(1:7).eq.'notread') then
               print*,' ***** Error in torsion ',i,m
               stop
            endif
            n=nword(regel)
            k=n/2
            if (2*k+1.ne.n) then
               print*,' ***** Even number of entries for torsion',m
               stop
            endif
            do j=1,k
               ntors=ntors+1
               if (ntors.gt.maxdih) then
                  print*, ' ***** Increase maxdih',ntors
                  stop
               endif
               itcod(ntors)=m
               ip(ntors)=i1
               jp(ntors)=i2
               kp(ntors)=i3
               lp(ntors)=i4
               call realst(regel,2*j,cpt)
               call intstr(regel,2*j+1,npt)
c               print*,ntorss,i1,i2,i3,i4,mul,cpt,npt
               if (npt.eq.0) then
                  print 23
                  stop
               endif
               cp(ntors)=cpt*fact
               if (npt.lt.0) cp(ntors)=cpt/mul
               np(ntors)=abs(npt)
c               print*,ntors,cp(ntors),np(ntors)
            enddo
         endif
      enddo
      if (iprint.gt.2) print*,' Torsional information read'

C     Read information on special dihedrals
      read(20,*,end=890,err=890) ndihz
      do i=1,ndihz
         read(20,*) iome(i),jome(i),kome(i),lome(i)
      enddo
      read(20,*) ndihh
      print 17,ndihz,ndihh
      write(16,17) ndihz,ndihh
      if (ndihh.gt.maxome) then
         print*, ' ***** Increase maxome'
         stop
      endif
      ndiha=ndihz+ndihh
      if (ndiha.gt.maxome) then
         print*, ' ***** Increase maxome'
         stop
      endif
      do j=1,ndihh
         k=j+ndihz
         read(20,*) i1,i2,i3,i4
         do i=1,ntors
            if ((i1.eq.ip(i) .and. i2.eq.jp(i) .and. i3.eq.kp(i)
     +           .and. i4.eq.lp(i)) .or.
     +           (i4.eq.ip(i) .and. i3.eq.jp(i) .and. i2.eq.kp(i)
     +           .and. i1.eq.lp(i))) then
               if (panm(i1)(1:1).eq.'H') then
                  iome(k)=i1
                  jome(k)=i2
                  kome(k)=i3
                  lome(k)=i4
               elseif (panm(i4)(1:1).eq.'H') then
                  iome(k)=i4
                  jome(k)=i3
                  kome(k)=i2
                  lome(k)=i1
               else
                  print*,' ***** No H atom in dihedral',i1,i2,i3,i4
                  stop
               endif
               goto 440
            endif
         enddo
         print*,' ***** Not found dihedral',i1,i2,i3,i4
         stop
  440    continue
      enddo
      do k=1,ndiha
         if (iome(k).le.0 .or. iome(k).gt.nrploc .or.
     +        jome(k).le.0 .or. jome(k).gt.nrploc .or.
     +        kome(k).le.0 .or. kome(k).gt.nrploc .or.
     +        lome(k).le.0 .or. lome(k).gt.nrploc) then
            print*,' ***** Error in atom number for dihedral',k
            stop
         endif
      enddo
      if (iprint.gt.2) print*,' Special dihedral information read'

C     Read information on reference atoms
      read(20,11,end=890,err=890) regel
      read(20,11,end=890,err=890) regel
      nref=nword(regel)
c      pause
c      if (n.gt.3) n=3
      do i=1,nref
         call intstr(regel,i,ireftr(i))
      enddo
c      if (n.lt.3) ireftr(1)=-ireftr(1)
      do i=1,nref
         if (ireftr(i).lt.1) then
            print*,' Wrong reference atom',i,ireftr(i)
            stop
         endif
      enddo
      print 18,(ireftr(m),m=1,nref)
      write(16,18) (ireftr(m),m=1,nref)
      close(20)
C     End reading topology file

C     Change units of parameters
      do i=1,nratt
         do j=1,nratt
            if (buck) then
               c12(i,j)=c12(i,j)*fact
               cs12(i,j)=cs12(i,j)/facr
               c6(i,j)=c6(i,j)*f6
            else
               c12(i,j)=c12(i,j)*f12
               c6(i,j)=c6(i,j)*f6
               cs12(i,j)=cs12(i,j)*f12
               cs6(i,j)=cs6(i,j)*f6
            endif
         enddo
      enddo
      do i=1,nrploc
         cg(i)=cg(i)*facq
         alfpol(i)=alfpol(i)*facr**3
         iaci=iac(i)
         sq6(i)=sqrt(c6(iaci,iaci))
      enddo
      do i=1,nbond
c         cb(i)=cb(i)*facb
         b0(i)=b0(i)*facr
      enddo
      do i=1,nhoek
c         ct(i)=ct(i)*fact
         t0(i)=t0(i)/rad
      enddo
      do i=1,nimpr
c         cq(i)=cq(i)*fact
         q0(i)=q0(i)/rad
      enddo
      do i=1,ntors
c         cp(i)=cp(i)*fact
      enddo
      do ii=1,nrploc
         do jj=1,nrploc
            sij(ii,jj)=((alfpol(ii)*alfpol(jj)*1000000)**(1/6.0d0))
            if (abs(sij(ii,jj)).lt.1.0d-10) sij(ii,jj)=1
         enddo
      enddo

C     Atom i belongs to molecule INDMOL(i)
      nmol=1
      indmol(0)=0
      indmol(1)=1
      do i=2,nrploc
         if (iexcl(i,i-1).eq.0) nmol=nmol+1
         indmol(i)=nmol
      enddo

C     Read file defining geometrical data on virtual atoms
      if (ichad.eq.2) then
         do i=0,nratt
            c12(i,0)=0
            c12(0,i)=0
            c6(i,0)=0
            c6(0,i)=0
            cs12(i,0)=0
            cs12(0,i)=0
            cs6(i,0)=0
            cs6(0,i)=0
         enddo
         do i=1,maxvir
            do m=1,3
               do m1=1,3
                  do m2=1,3
                     dvir(i,m,m1,m2)=0
                  enddo
               enddo
            enddo
         enddo

         iiprev=1
         nmol=1
         indmol(nrploc+1)=0
         do ii=2,nrploc+1
            if (indmol(ii).ne.indmol(ii-1)) then
C              Start a new molecule, from iiprev to ii-1
               write(filenm,27) 'mol',nmol
   27          format(a3,i2.2,'.def')
               open(48,file=filenm,status='OLD',iostat=ios)
               if (ios.ne.0) then
                  print*,' ***** No file ',filenm,' present'
                  write(16,*) ' ***** No file ',filenm,' present'
                  goto 550
               endif
               
               print*,' Reading non-atomic charge sites from ',filenm
               write(16,*) ' Reading non-atomic charge sites from ',
     +              filenm
  500          continue
               read(48,11,end=540) regel
               n=nword(regel)
               if (n.eq.0) goto 500
               if (n.ne.3 .and. n.ne.5 .and. n.ne.6) then
                  print*,regel
                  print*,' ***** Incorrect line on unit 42'
                  print*,' Type ENTER if you wish to continue anyhow'
                  read 11, regel
                  goto 500
               endif
               nvir=nvir+1
               if (nvir.gt.maxvir) then
                  print*,' ***** Increase maxvir', nvir
                  stop
               endif
               nchad(nvir)=n
               jvir(nvir,3)=0
               call intstr(regel,1,i1)
               call intstr(regel,2,i2)
               call realst(regel,3,pchad(nvir))
               i1=i1+iiprev-1
               i2=i2+iiprev-1
               jvir(nvir,1)=i1
               jvir(nvir,2)=i2
               indvir(nvir)=indmol(i1)
C              Virtual atom i belongs to molecule INDVIR(i)
               if (n.ge.5) then
                  call intstr(regel,4,i3)
                  i3=i3+iiprev-1
                  jvir(nvir,3)=i3
                  call realst(regel,5,qchad(nvir))
               endif
               if (n.eq.6) then
                  call realst(regel,6,schad(nvir))
               endif
               if (n.eq.3) print 14,nvir,i1,i2,pchad(nvir)
               if (n.eq.5) print 14,nvir,i1,i2,pchad(nvir),
     +              i3,qchad(nvir)
               if (n.eq.6) print 14,nvir,i1,i2,pchad(nvir),
     +              i3,qchad(nvir),schad(nvir)
               if (n.eq.3) write(16,14) nvir,i1,i2,pchad(nvir)
               if (n.eq.5) write(16,14) nvir,i1,i2,pchad(nvir),
     +              i3,qchad(nvir)
               if (n.eq.6) write(16,14) nvir,i1,i2,pchad(nvir),
     +              i3,qchad(nvir),schad(nvir)
               goto 500
  540          continue
               nmol=nmol+1
               iiprev=ii
            endif
            close(48)
         enddo

C        End reading mol##.def, expand:
         do m=2,maxvir/nvir
            im=(m-1)*nvir
            do i=1,nvir
               pchad(i+im)=pchad(i)
               qchad(i+im)=qchad(i)
               schad(i+im)=schad(i)
               nchad(i+im)=nchad(i)
               do j=1,3
                  jvir(i+im,j)=jvir(i,j)+(m-1)*nrploc
                  if (jvir(i,j).eq.0) jvir(i+im,j)=0
               enddo
               indvir(i+im)=indvir(i)+(m-1)*indmol(nrploc)
            enddo
         enddo
c         do m=1,3
c  111        format(40i4)
c            print 111,(jvir(i,m),i=1,40)
c         enddo
c         pause
  550    continue
      endif
      ichad=-ichad

C     Expand for identical groups
      do m=2,maxat/nrploc
         im=(m-1)*nrploc
         do i=1,nrploc
            panm(i+im)=panm(i)
            type(i+im)=type(i)
C              Add a number (1...9) to names of atoms in equivalent groups
c               do j=1,8 
c                  if (panm(i)(j:j).eq.' ') then
c                     write(num,16) m-1
c                     panm(i+im)(j:j)=num
c                     goto 480
c                  endif
c               enddo
c 480           continue
            iac(i+im)=iac(i)
            ido(i+im)=ido(i)
            amass(i+im)=amass(i)
            cg(i+im)=cg(i)
            sq6(i+im)=sq6(i)
            alfpol(i+im)=alfpol(i)
            pmax(i+im)=pmax(i)
            indmol(i+im)=indmol(i)+(m-1)*indmol(nrploc)
         enddo
      enddo
C     Expand charge group pointers
      do m=2,maxcag/ncag
         im=(m-1)*nrploc
         km=(m-1)*ncag
         do i=1,ncag
            inc(i+km)=inc(i)+im
         enddo
      enddo
C     Expand dihedral indices
      do m=2,maxmol
         im=(m-1)*nrploc
         jm=(m-1)*ndiha
         do i=1,ndiha
            iome(i+jm)=iome(i)+im
            jome(i+jm)=jome(i)+im
            kome(i+jm)=kome(i)+im
            lome(i+jm)=lome(i)+im
         enddo
      enddo
      return

  840 print 34
      stop
  860 print 35
      stop
  890 print 36
      stop

   11 format(a80)
   14 format(' Virtual atom',i4,3x,2i4,f8.3,i4,2f8.3)
   17 format(i4,' conformation-determining dihedrals without H',
     +     ' and ',i4,' with H')
   18 format(' Reference atoms for rotational matrices:',4(3i4,3x))
   20 format(' Total charge of charge group',i4,f10.3)
   21 format(' ***** i=',a8,' j=',a8,'    C6(ij)**2 =',f10.1,
     +     '  but C6(ii)*C6(jj) =',f10.1)
   22 format(' ***** Total charge not zero:',f8.5,'. Reset to zero!')
   23 format(' ***** Torsional term with frequency zero!',2i5)
   24 format(' Atom',i4,' can be disregarded in NONBE')
   25 format(i5,5x,a5)
   28 format(' LEEST: Reading force field data from: ',a80)
   29 format(' LEEST: Reading topology data from: ',a80)
   30 format(' Scaling factors for 1...4 interactions:',2f8.3)
   31 format(' Dielectric constant',f8.3)
   34 format(' ***** No topology file IN.TP present')
   35 format(' ***** No force field file IN.FF present')
   36 format(' ***** Error in topology file')
   39 format(a,i5,'  code',i5,2x,2f12.4)
      end
