      subroutine list(ngr,nrp,enb,nf,der,box,x,f)
C     Calculates nonbonded crystal energy, forces and virial.
C     The nonbonded distances are selected using a pair list prepared by
C     the subroutine NONBE. Calculates the same quantities as that subroutine.
C     Input through parameters:
C     NGR: number of groups
C     NRP: number of atoms
C     NF rules the calculation of derivatives:
C        NF=0: no derivatives
C        NF=1: calculates forces F and DER
C        NF=2: calculates forces F and virial VIR
C     BOX(1..6): ax, by, cz, bx, cx, cy
C     X(1..): atom coordinates.

C     Input through common blocks:
C     /NBAL/ gives pair list information from NONBE:
C        NPLA: number of pairs in the list
C           The various arrays give information for the calculation of
C           nonbonded pair interactions:
C        IPLA1, IPLA2: coordinates are located in X(IPLA+M) (M=1,2,3)
C        PLJ6, PLJ12, PLJ6EW, PEXP: Lennard-Jones or Buckingham coefficients
C        PCG, PCGEW: Coulomb coefficients
C        SHIFT1..3: fractional shifts for required symmetry operation
C        ISIG1..3: signs for required symmetry operation
C        IEXC: intermolecular or 1..2, 1..3, other intramolecular interaction
C     /POLA/ gives polarization data:
C        NPP=0: no polarization
C        NPP=1: intermolecular polarization with neglect of the effect
C               of the induced dipoles on the electric field.
C        NPP>0: call LISTP for full polarization calculation.
C        ALFA(1..): polarizabilities
C     /NONB/ gives various information:
C        CG(1..NRP): charges
C        EBIG: if any repulsion term exceeds EBIG, stop immediately and
C           set ENB=90000.
C        BUCK: if TRUE, then choose Buckingham potential
C        DDC: if TRUE, use distance-dependent dielectric constant R (A).
C        DRUK: the pressure
C        MSOM(m) = sum over MSP(m), compares with IREF (from LEESR).
C        INNER: do calculation of U2-inner if INNER>0.
C     /WALD/ gives information for Ewald summation. The last symbols C and 6
C        refer to Coulomb terms and attractive r^(-6) terms, respectively.
C        This is only carried out if logical DOEW is TRUE.
C        ALPHAC, ALPHA6: damping factors in nm^(-1).
C        DOEWC, DOEW6: if FALSE, then skip that part of the calculation.

C     Output through parameters:
C     ENB: the nonbonded energy
C     Special cases:
C        ENB = 70000: a cell axis is less than 0.5 Angstrom.
C        ENB = 80000: a distance is amaller than 0.5 A
C        ENB = 90000: a repulsion term exceeds EBIG
C     DER(1..6): the negative derivatives of ENB to BOX(1..6)
C     F(1..3*NGR*NRP): the negative derivatives of ENB to X(1..3*NGR*NRP).
C     Array X: expanded with the symmetry-related molecules.

C     Output through common blocks:
C     /NONB/
C        PMOL(1..3): the cell dipole moment
C        UINNER: the calculated U2-inner
C        EEL: the coulomb energy
C        EDISPA: the attractive dispersion term
C        EDISPR: the repulsive term
C     /WALD/:
C        ECC, EC6: Ewald contribution from constant terms (from KWALD)
C        EKC, EK6: Ewald contribution from reciprocal terms (from KWALD)
C        EDC, ED6: Ewald contribution from direct terms (this subroutine)
C     /POLA/: polarization output if NPP=1:
C        DIP(1..): induced dipole moments
C        EPOL: the polarization energy.
C     /VIRA/:
C        VIRA(1..3,1..3): contribution to virial

C     Calls subroutines LISTP and LISTK.

      implicit none
      include 'params.i'
      include 'cvira.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cwald.i'
      include 'cpola.i'
      include 'chadd.i'

      double precision al6kw,atsqpi,ax,box,bx,by,c6r,cbu,cgij,
     +     cx,cy,cz,d,d2,denuli,denulj,der,df,
     +     dip1,dip2,dip3,dipind,djp1,djp2,djp3,ebu,enb,enul,erfcc,
     +     ewcoc,exa,f,fact,fi,fj,
     +     p,pi,pip,pir,pjr,q,qi,qj,rba,rca,
     +     rij,rij1,rij2,rij3,rij6,rklrij,s,s1,s2,s3,stuk2,
     +     vda,vdr,vel,vol,vp3v,vp3vn,x,xd,xij,xsqc,xsqlj,yd,yij,zd,zij
      integer i,ia3,ib3,ic3,
     +     ii,is1,is2,is3,j,jj,
     +     k,k3,m,m1,m2,n,n3,natoms,nf,ngr,
     +     nrp,nrpt,nvirt
      dimension der(*),box(*),x(*),f(*),rij(3),dipind(3),enul(maxxco),
     +     rba(3),rca(3)
      logical skip,intra

      nrpt=nrp*ngr
      nvirt=nvir*ngr
      natoms=nrpt+nvirt
      if (nvir.gt.0) then
c         noint=.true.
         do i=1,nvirt
            n=nchad(i)
            p=pchad(i)
            q=qchad(i)
            s=schad(i)*10
C           S must have (temporarily?) the dimension A^(-1)
            n3=3*(nrpt+i)-3
            ia3=3*jvir(i,1)-3
            ib3=3*jvir(i,2)-3
            ic3=3*jvir(i,3)-3
            if (n.ge.3) then
               do m=1,3
                  rba(m)=x(ib3+m)-x(ia3+m)
               enddo
            endif
            if (n.ge.5) then
               do m=1,3
                  rca(m)=x(ic3+m)-x(ia3+m)
               enddo
            endif
            if (n.eq.3) then
C              Virtual atom in a bond
               do m=1,3
                  x(n3+m)=x(ia3+m)+p*rba(m)
               enddo
            endif
            if (n.eq.5) then
C              Virtual atom in a plane
               do m=1,3
                  x(n3+m)=x(ia3+m)+p*rba(m)+q*rca(m)
               enddo
            endif
            if (n.eq.6) then
C              Virtual atom in three dimensions
               rij(1)=rba(2)*rca(3)-rba(3)*rca(2)
               rij(2)=rba(3)*rca(1)-rba(1)*rca(3)
               rij(3)=rba(1)*rca(2)-rba(2)*rca(1)
               d2=0
               do m=1,3
                  d=p*rba(m)+q*rca(m)+s*rij(m)
                  d2=d2+d*d
                  x(n3+m)=x(ia3+m)+d
                  rba(m)=s*rba(m)
                  rca(m)=s*rca(m)
                  rij(m)=rba(m)-rca(m)
               enddo
            endif
            do m=1,3
               dvir(i,1,m,m)=1-p-q
               dvir(i,2,m,m)=p
               dvir(i,3,m,m)=q
               if (n.eq.6) then
                  dvir(i,1,3,2)=-rij(1)
                  dvir(i,1,1,3)=-rij(2)
                  dvir(i,1,2,1)=-rij(3)
                  dvir(i,1,2,3)= rij(1)
                  dvir(i,1,3,1)= rij(2)
                  dvir(i,1,1,2)= rij(3)

                  dvir(i,2,3,2)=-rca(1)
                  dvir(i,2,1,3)=-rca(2)
                  dvir(i,2,2,1)=-rca(3)
                  dvir(i,2,2,3)= rca(1)
                  dvir(i,2,3,1)= rca(2)
                  dvir(i,2,1,2)= rca(3)

                  dvir(i,3,2,3)=-rba(1)
                  dvir(i,3,3,1)=-rba(2)
                  dvir(i,3,1,2)=-rba(3)
                  dvir(i,3,3,2)= rba(1)
                  dvir(i,3,1,3)= rba(2)
                  dvir(i,3,2,1)= rba(3)
               endif
            enddo
         enddo
      endif

      if (npp.gt.1) then
         call listp(ngr,nrp,enb,nf,der,box,x,f)
         return
      endif

      if (nf.eq.2) then
         do m=1,3
            do n=1,3
               vira(m,n)=0
            enddo
         enddo
      endif
      do m=1,3
         if (box(m).lt.0.05) then
c            print*,' ***** LIST: Cell axes less than 0.5 Angstrom.'
c            write(16,*) ' ***** LIST: Cell axes less than 0.5 Angstrom.'
            enb=70000
            return
         endif
      enddo

      pi=4*atan(1.0d0)
      atsqpi=2*alphac/sqrt(pi)
      al6kw=alpha6**2
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      edc=0
      ed6=0
      ecc=0
      ec6=0
      ekc=0
      ek6=0
      edispa=0
      edispr=0
      Epol=0

C     Calculate cell dipole moment PMOL; initialize forces F
      uinner=0
      vol=ax*by*cz
      enb=druk*vol
      vp3v=4*pi/(3*vol)
      vp3vn=finner/vol
C     This is twice the energy factor 2 pi / [3 V (nspgr+1)]
      do m=1,3
         p=0
         do i=1,natoms
            p=p+x(3*i-3+m)*msom(m)*cg(i)
         enddo
         uinner=uinner-vp3vn*p*p/2
         pmol(m)=p
         if (inner.eq.0) p=0
         if (nf.gt.0) then
            do i=1,natoms
               f(3*i-3+m)=vp3vn*msom(m)*cg(i)*p
            enddo
         endif
      enddo
C     Initialize field strengths with 4 pi dipole / [3 V]
      do i=1,natoms
         do m=1,3
            enul(3*i-3+m)=vp3v*pmol(m)
         enddo
      enddo
C     Decide whether to include U2-inner in EEL; initialize cell derivatives
      if (inner.ne.0) then
         eel=uinner
      else
         eel=0
      endif
      do m=1,3
         der(m)=(eel-enb)/box(m)
         der(m+3)=0
      enddo

      if (doew) call listk(natoms,nf,der,box,x,f)

      do n=1,npla
         i=ipla1(n)
         skip=(i.lt.0)
         if (skip) i=-(i+1)
         j=ipla2(n)
         intra=(j.lt.0)
         if (intra) j=-(j+1)
         s1=ishift1(n)/2.0d0
         s2=ishift2(n)/2.0d0
         s3=ishift3(n)/2.0d0
         is1=isig1(n)
         is2=isig2(n)
         is3=isig3(n)

         xij=x(i+1)-is1*x(j+1)+s1*ax+s2*bx+s3*cx
         yij=x(i+2)-is2*x(j+2)+s2*by+s3*cy
         zij=x(i+3)-is3*x(j+3)+s3*cz
         rij2=xij**2+yij**2+zij**2
         rij1=sqrt(rij2)
         rij6=rij2*rij2*rij2
         df=0
         if (ichad.eq.0 .and. rij1.lt.0.05d0) then
c            print*,' LIST: short distance: ',i,j,rij1
            enb=80000
            return
         endif

C                    Calculate harmonic terms
c                     b0=0.345
c                     f0=pcg(n)
c                     if (skip) f0=0
c                     vel=1+0.5*f0*(rij1-b0)**2
c                     df=f0*(rij1-b0)/rij1
cc                     print*,n,f0,vel,df
c                    eel=eel+vel
c                     if (b0.gt.-0.1) goto 400

C        Calculate Coulomb energy
C        Distance dependent dielectric constant if DDC=TRUE.
         if (.not. skip) then
            vel=pcg(n)/rij1
            df=-vel/rij2
            if (ddc) then
               vel=vel/rij1
               df=2*df/rij1
            endif
            eel=eel+vel
         endif
         if (doewc) then
C           Calculate Ewald correction
            cgij=pcgew(n)
            rklrij=alphac*rij1
            xsqc=rklrij*rklrij
C           Numerical Recipes for complementary error function:
c            erfc=erfcc(rklrij)
c            t=1/(1.0+0.5*rklrij)
c            erfc=t*exp(-xsqc-1.26551223+t*(1.00002368+
c     +           t*(.37409196+t*(.09678418+t*(-.18628806+
c     +           t*(.27886807+t*(-1.13520398+t*(1.48851587+
c     +           t*(-.82215223+t*.17087277)))))))))
            ewcoc=cgij*(erfcc(rklrij)-1)/rij1
            edc=edc+ewcoc
            exa=0
            if (xsqc.lt.700) exa=exp(-xsqc)
            df=df-(ewcoc+cgij*atsqpi*exa)/rij2
         endif

C        Calculate attractive r^(-6) terms
         if (.not. skip) then
            vda=plj6(n)/rij6
            edispa=edispa-vda
            df=df+6*vda/rij2
         endif
         c6r=plj6ew(n)/rij6
         if (doew6 .and. abs(c6r).gt.1.0d-10) then
C           Calculate Ewald correction
            xsqlj=al6kw*rij2
            exa=0
            if (xsqlj.lt.700) exa=exp(-xsqlj)
            fact = (1 + xsqlj * (1 + xsqlj/2)) * exa - 1
            ed6=ed6-c6r*fact
            df=df+(c6r/rij2)*(6*fact+exa*xsqlj*xsqlj*xsqlj)
         endif

C        Calculate R^(-12) or exponential repulsion
         if (.not. skip) then
            if (buck) then
               cbu=plj12(n)
               ebu=pexp(n)
               vdr=cbu*exp(-ebu*rij1)
               df=df-ebu*vdr/rij1
            else
               vdr=plj12(n)/(rij6*rij6)
               df=df-12*vdr/rij2
            endif
            edispr=edispr+vdr
            if (edispr.gt.ebig) then
c               print*,' LIST: energy > EBIG',rij1,vdr
               enb=90000
               return
            endif
         endif

c 400     continue
C        Calculate derivatives
         xd=xij*df
         yd=yij*df
         zd=zij*df
         f(i+1)=f(i+1)-xd
         f(i+2)=f(i+2)-yd
         f(i+3)=f(i+3)-zd
         f(j+1)=f(j+1)+xd*is1
         f(j+2)=f(j+2)+yd*is2
         f(j+3)=f(j+3)+zd*is3
         if (nf.eq.1) then
C           Calculate derivatives to cell axes
            der(1)=der(1)-xd*s1
            der(2)=der(2)-yd*s2
            der(3)=der(3)-zd*s3
            der(4)=der(4)-xd*s2
            der(5)=der(5)-xd*s3
            der(6)=der(6)-yd*s3
         endif
         if (nf.eq.2) then
C           Calculate virial
            rij(1)=xij
            rij(2)=yij
            rij(3)=zij
            do m=1,3
               do k=1,3
                  vira(m,k)=vira(m,k)+rij(m)*df*rij(k)
               enddo
            enddo
         endif

C        Calculate electric field for intermolecular polarization
         if (npp.eq.1 .and. (.not. intra)) then
            ii=i/3+1
            jj=j/3+1
            qi=cg(ii)
            qj=cg(jj)
            rij3=rij1*rij2
            denuli=qj/rij3
            denulj=qi/rij3
            enul(i+1)=enul(i+1)+xij*denuli
            enul(i+2)=enul(i+2)+yij*denuli
            enul(i+3)=enul(i+3)+zij*denuli
            if (i.ne.j) then
               enul(j+1)=enul(j+1)-xij*denulj*is1
               enul(j+2)=enul(j+2)-yij*denulj*is2
               enul(j+3)=enul(j+3)-zij*denulj*is3
            endif
            r1(n)=xij
            r2(n)=yij
            r3(n)=zij
         endif
      enddo

      if (npp.ne.1) goto 700
C     Calculate polarization energy.
C     Include effects of U-inner on energy and forces (not yet the virial!)
      pip=0
      do m=1,3
         dipind(m)=0
         do i=1,natoms
            j=3*i-3+m
            dip(j)=alfpol(i)*Enul(j)
            Epol=Epol-dip(j)*Enul(j)/2
            dipind(m)=dipind(m)+dip(j)
         enddo
         pip=pip+dipind(m)*pmol(m)
      enddo
      do m=1,3
         der(m)=der(m)-vp3v*pip/box(m)
      enddo
      do m=1,3
         do i=1,natoms
            j=3*i-3+m
            f(j)=f(j)+vp3v*dipind(m)*msom(m)*cg(i)
         enddo
      enddo

C     Calculate intermolecular polarization forces
CCCCC (virial yet to be done!!)
      if (nf.eq.0) goto 700
      if (nf.eq.2) goto 700
      do n=1,npla
         j=ipla2(n)
         if (j.lt.0) goto 690
C        No intramolecular distances
         i=ipla1(n)
         if (i.lt.0) i=-(i+1)
         ii=i/3+1
         jj=j/3+1
         qi=cg(ii)
         qj=cg(jj)
         is1=isig1(n)
         is2=isig2(n)
         is3=isig3(n)
         dip1=dip(i+1)
         dip2=dip(i+2)
         dip3=dip(i+3)
         djp1=dip(j+1)*is1
         djp2=dip(j+2)*is2
         djp3=dip(j+3)*is3
         rij2=r1(n)**2+r2(n)**2+r3(n)**2
         rij1=sqrt(rij2)
         rij3=rij1*rij2

         pjr=djp1*r1(n)+djp2*r2(n)+djp3*r3(n)
         pir=dip1*r1(n)+dip2*r2(n)+dip3*r3(n)
         stuk2=3*(-qj*pir+qi*pjr)/(rij2*rij3)
         fj=qj/rij3
         fi=qi/rij3
         xd=dip1*fj-djp1*fi+stuk2*r1(n)
         yd=dip2*fj-djp2*fi+stuk2*r2(n)
         zd=dip3*fj-djp3*fi+stuk2*r3(n)

         if (i.eq.j) then
            xd=xd/2
            yd=yd/2
            zd=zd/2
         endif
         der(1)=der(1)+xd*ishift1(n)/2
         der(2)=der(2)+yd*ishift2(n)/2
         der(3)=der(3)+zd*ishift3(n)/2
         der(4)=der(4)+xd*ishift2(n)/2
         der(5)=der(5)+xd*ishift3(n)/2
         der(6)=der(6)+yd*ishift3(n)/2
         f(i+1)=f(i+1)+xd
         f(i+2)=f(i+2)+yd
         f(i+3)=f(i+3)+zd
         f(j+1)=f(j+1)-xd*is1
         f(j+2)=f(j+2)-yd*is2
         f(j+3)=f(j+3)-zd*is3
  690    continue
      enddo

  700 continue
      if (nvir.gt.0) then
C        Correct derivatives for non-atomic sites
CCCCC    (virial yet to be done!!)
         do k=1,nvirt
            k3=3*(nrpt+k)-3
            do m=1,3
               j=3*jvir(k,m)-3
               if (j.ge.0) then
                  do m1=1,3
                     do m2=1,3
                        f(j+m1)=f(j+m1)+dvir(k,m,m2,m1)*f(k3+m2)
                     enddo
                  enddo
               endif
            enddo
         enddo
      endif

      if (doewc) eel=eel+ecc+ekc+edc
      if (doew6) edispa=edispa+ec6+ek6+ed6
      enb=enb+eel+edispa+edispr+Epol
      if (nf.eq.2) then
         do m=1,3
            do n=1,3
               vira(m,n)=vira(m,n)/2
            enddo
         enddo
      endif

c      write(94,297) ' List: ',eel,edispa,edispr,enb
c      write(94,298) (der(i),i=1,6)
c      write(95,299) (f(i),i=1,3*nrpt)
c      print 297, '  Ewc: ',ecc,edc,ekc
c      print 297, '  Ew6: ',ec6,ed6,ek6
c      print 297, ' Energie List: ',eel,edispa,edispr,enb
c      print 296,(vira(1,m),m=1,3)
c      print 296,(vira(2,m),m=1,3)
c      print 296,(vira(3,m),m=1,3)
c      print 298, (der(i),i=1,6)
c      print 299, (f(i),i=1,3*nrpt)

c 296  format (3f8.3)
c 297  format(a20,10f12.6)
c 298  format(' Der: ',6f12.6)
c 299  format(6f12.3)
      end
