      subroutine listk(nrp,nf,der,box,x,f)

C     Calculates constant and reciprocal lattice contributions
C     to the Ewald correction on energies, forces and virial.
C     The same quantities are calculated as in KWALD, but it is done using
C     a list of reciprocal lattice vectors prepared by that subroutine.

C     Input through parameters:
C     NRP: the number of atoms in one molecule
C     NF rules the calculation of derivatives:
C        NF=0: no derivatives
C        NF=1: calculates forces F and DER
C        NF=2: calculates forces F and virial VIR
C     BOX(1..6): ax, by, cz, bx, cx, cy
C     X(1..NR3): atom coordinates. X must be big enough to accomodate
C        all symmetry-expanded coordinates.

C     Input through common blocks:
C     /NBKL/ gives reciprocal lattice vector information from KWALD:
C        NPLK: number of reciprocal lattice vectors in the list
C           The various arrays contain:
C        KXL, KYL, KZL: Laue indices of reciprocal lattice vectors
C        CG, SQ6:  Coulomb and R^(-6) coefficients for each atom
C        TEXP: translation component for equivalent position of the atom
C     /SPGR/ gives space group information:
C        NSPGR, MSP, TSP: see subroutine LEESR
C     /WALD/ gives information for Ewald summation. The last symbols C and 6
C        refer to Coulomb terms and attractive r^(-6) terms, respectively.
C        ALPHAC, ALPHA6: damping factors in nm^(-1).
C        HMAXC, HMAX6: maximum reciprocal lattice vectors in nm^(-1).
C        DOEWC, DOEW6: if FALSE, then skip that part of the calculation.

C     Output through parameters:
C     DER(1..6): increased with the negative derivatives of the
C                energy contributions to BOX(1..6)
C     F(1..NR3): increased with the corresponding forces on the atoms
C     X(1..): expanded with the symmetry-related molecules.

C     Output through common blocks:
C     /VIRA/:
C        VIRA(1..3,1..3): increased with twice the contribution to virial
C     /WALD/:
C        ECC, EC6: Ewald contribution from constant terms
C        EKC, EK6: Ewald contribution from reciprocal terms
C        KTOTC, KTOT6: number of reciprocal lattice vectors

      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cvira.i'
      include 'cnbkl.i'
      include 'cwald.i'
      include 'cnonb.i'

      double precision al64,alc4,astx,asty,astz,ax,b,box,bsq6,
     +     bsqc,bsty,bstz,bx,by,cc,ch6,co,cst,cstz,cx,cy,cz,
     +     der,df,dsdc,dsdcst,ebsq,elj0,erfcc,eterm6,etermc,
     +     f,f1,f2,factor,factsp,fsp,h,
     +     phi,pi,pi32,pipi,qco6,qcoc,qsi6,qsic,rij2,rk,
     +     scr,si,sqpi,sqrt6,srco6,srcoc,srsi6,srsic,stco6,stcoc,
     +     stsi6,stsic,sum6,sumc,sumco6,sumcoc,sumsi6,sumsic,
     +     vol,vpk,vpksq,vterm6,vtermc,wh6,whc,x,xij,yij,zij

      integer i,i3,j,k,kvec,kx,ky,kz,m,n,nf,nop3,nr3,nrp,nrsp
      dimension x(*),box(*),f(*),der(*),
     +     qcoc(maxat),qsic(maxat),qco6(maxat),qsi6(maxat),
     +     rk(3),h(3),cst(3,3),dsdc(3,3),dsdcst(3,3),
     +     stcoc(3),stco6(3),stsic(3),stsi6(3),
     +     srcoc(3),srco6(3),srsic(3),srsi6(3)

      nrsp=(nspgr+1)*nrp
      nr3=3*nrp
      fsp=1/(nspgr+1.0d0)
      pi=4*atan(1.0d0)
      sqpi=sqrt(pi)
      pi32=pi*sqpi
      pipi=2*pi
      alc4=4*(alphac**2)
      al64=4*(alpha6**2)
      ekc=0
      ek6=0

C     Calculate reciprocal lattice components CST in orthogonal X,Y,Z system
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      vol=ax*by*cz
      astx=1/ax
      asty=-bx*cz/vol
      astz=(bx*cy-by*cx)/vol
      bsty=1/by
      bstz=-cy*ax/vol
      cstz=1/cz
      cst(1,1)=astx
      cst(1,2)=asty
      cst(1,3)=astz
      cst(2,1)=0
      cst(2,2)=bsty
      cst(2,3)=bstz
      cst(3,1)=0
      cst(3,2)=0
      cst(3,3)=cstz
      sum6=0
      sqrt6=0
      sumc=0
      do i=1,nrp
         if (doewc) sumc=sumc-cg(i)**2
         if (doew6) then
            ch6=sq6(i)**2
            sum6=sum6+ch6
            sqrt6=sqrt6+sq6(i)
         endif
      enddo

C     Expand coordinates using equivalent positions
      if (nspgr.gt.0) then
         do nop=1,nspgr
            nop3=3*nop-3
            do i=1,nrp
               j=3*i-3
               k=j+nop*nr3
               x(k+1)=msp(nop3+1)*x(j+1)+
     +              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
               x(k+2)=msp(nop3+2)*x(j+2)+
     +              tsp(nop3+2)*by+tsp(nop3+3)*cy
               x(k+3)=msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
            enddo
         enddo
      endif
      do i=1,3
         do j=1,3
            dsdcst(i,j)=0
            dsdc(i,j)=0
         enddo
      enddo

C     Calculate constant contributions
      if (doewc) ecc= alphac*sumc / sqpi
      elj0=0
      if (doew6) then
         elj0 = pi32*alpha6**3*sqrt6**2 / (6*vol*fsp)
         ec6= alpha6**6 * sum6 / 12 - elj0
         if (nf.eq.2) then
            do m=1,3
               vira(m,m)=vira(m,m)+elj0
            enddo
         endif
      endif

C     Sum over all vectors selected by KWALD
      do kvec=1,nplk
         kx=kxl(kvec)
         ky=kyl(kvec)
         kz=kzl(kvec)
         xij=kx*astx
         yij=ky*bsty+kx*asty
         zij=kz*cstz+ky*bstz+kx*astz
C        XIJ, YIJ, ZIJ are the X, Y, Z components of the next reciprocal
C        lattice vector, having Laue indices KX, KY, KZ
         rij2=xij**2+yij**2+zij**2

C        Calculation for this reciprocal lattice vector
C        Take only positive KX, correct for omitted vectors by FACTOR.
         if (kx.eq.0) then
            factor=2
            factsp=fsp
         else
            factor=4
            factsp=2*fsp
         endif
         rk(1)=pipi*xij
         rk(2)=pipi*yij
         rk(3)=pipi*zij
         h(1)=2*pipi*kx
         h(2)=2*pipi*ky
         h(3)=2*pipi*kz
         vpksq=rij2*pipi**2
C        H(M) is 4 pi times the Laue index, RK(M) is 2 pi times the RR vector

C        Preparation
         do i=1,nrsp
            phi=x(3*i-2)*rk(1) + x(3*i-1)*rk(2) + x(3*i)*rk(3)
            co=cos(phi)
            si=sin(phi)
            qcoc(i)=cg(i)*co
            qsic(i)=cg(i)*si
            qco6(i)=sq6(i)*co
            qsi6(i)=sq6(i)*si
         enddo
         if (doewc) then
            sumcoc=0
            sumsic=0
            do m=1,3
               srcoc(m)=0
               srsic(m)=0
               stcoc(m)=0
               stsic(m)=0
            enddo
            do i=1,nrsp
               sumcoc=sumcoc+qcoc(i)
               sumsic=sumsic+qsic(i)
               if (nf.eq.1) then
                  do m=1,3
                     j=3*i-3+m
                     srcoc(m)=srcoc(m)+x(j)*qcoc(i)
                     srsic(m)=srsic(m)+x(j)*qsic(i)
                     stcoc(m)=stcoc(m)+texp(j)*qcoc(i)
                     stsic(m)=stsic(m)+texp(j)*qsic(i)
                  enddo
               endif
            enddo
         endif
         if (doew6) then
            sumco6=0
            sumsi6=0
            do m=1,3
               srco6(m)=0
               srsi6(m)=0
               stco6(m)=0
               stsi6(m)=0
            enddo
            do i=1,nrsp
               sumco6=sumco6+qco6(i)
               sumsi6=sumsi6+qsi6(i)
               if (nf.eq.1) then
                  do m=1,3
                     j=3*i-3+m
                     srco6(m)=srco6(m)+x(j)*qco6(i)
                     srsi6(m)=srsi6(m)+x(j)*qsi6(i)
                     stco6(m)=stco6(m)+texp(j)*qco6(i)
                     stsi6(m)=stsi6(m)+texp(j)*qsi6(i)
                  enddo
               endif
            enddo
         endif

C        Calculate energy and virial
         if (doewc) then
            bsqc=vpksq/alc4
            whc=exp(-bsqc)*pipi/(vol*vpksq)
            etermc=factsp*whc*(sumcoc**2+sumsic**2)
            ekc=ekc+etermc
            vtermc=2*(1+bsqc)*etermc/vpksq
            if (nf.eq.2) then
               do m=1,3
                  vira(m,m)=vira(m,m)-etermc
                  do n=1,3
                     vira(m,n)=vira(m,n)+vtermc*rk(m)*rk(n)
                  enddo
               enddo
            endif
         endif

         if (doew6) then
            bsq6=vpksq/al64
            b=sqrt(bsq6)
            vpk=sqrt(vpksq)/(24*vol)
            ebsq=exp(-bsq6)
            scr=sqpi*erfcc(b)-ebsq/b
            wh6=pi32*vpksq*vpk*(scr+ebsq/(2*bsq6*b))
            cc=factsp*(sumco6**2+sumsi6**2)
            eterm6=cc*wh6
            vterm6=3*cc*pi32*vpk*scr
            ek6=ek6-eterm6
            if (nf.eq.2) then
               do m=1,3
                  vira(m,m)=vira(m,m)+eterm6
                  do n=1,3
                     vira(m,n)=vira(m,n)+vterm6*rk(m)*rk(n)
                  enddo
               enddo
            endif
         endif

C        Calculate forces
         if (nf.gt.0) then
            if (doewc) then
               f1=factor*whc*sumcoc
               f2=factor*whc*sumsic
               do i=1,nrp
                  df=f1*qsic(i)-f2*qcoc(i)
                  i3=3*i
                  f(i3-2) = f(i3-2) + df*rk(1)
                  f(i3-1) = f(i3-1) + df*rk(2)
                  f(i3) = f(i3) + df*rk(3)
               enddo
            endif
            if (doew6) then
               f1=factor*wh6*sumco6
               f2=factor*wh6*sumsi6
               do i=1,nrp
                  df=-f1*qsi6(i)+f2*qco6(i)
                  i3=3*i
                  f(i3-2) = f(i3-2) + df*rk(1)
                  f(i3-1) = f(i3-1) + df*rk(2)
                  f(i3) = f(i3) + df*rk(3)
               enddo
            endif
         endif

C        Calculate derivatives to cell parameters
         if (nf.eq.1) then
            if (doewc) then
               do i=1,3
                  do j=i,3
                     dsdcst(i,j)=dsdcst(i,j)+h(i)*
     +                    (-vtermc*rk(j)/2+whc*factsp*
     +                    (sumsic*srcoc(j)-sumcoc*srsic(j)))
                     dsdc(i,j)=dsdc(i,j)+2*rk(i)*factsp*
     +                    whc*(sumsic*stcoc(j)-sumcoc*stsic(j))
                  enddo
               enddo
            endif
            if (doew6) then
               do i=1,3
                  do j=i,3
                     dsdcst(i,j)=dsdcst(i,j)-h(i)*
     +                    (vterm6*rk(j)/2+wh6*factsp*
     +                    (sumsi6*srco6(j)-sumco6*srsi6(j)))
                     dsdc(i,j)=dsdc(i,j)-2*rk(i)*factsp*
     +                    wh6*(sumsi6*stco6(j)-sumco6*stsi6(j))
                  enddo
               enddo
            endif
         endif
      enddo

C     Transform derivatives from reciprocal to real cell axes
      if (nf.eq.1) then
         do i=1,3
            der(i)=der(i)+(ekc+ek6-elj0)/box(i)
            do j=i,3
               do m=1,3
                  do n=1,3
                     dsdc(i,j)=dsdc(i,j)-dsdcst(m,n)*cst(m,i)*cst(j,n)
                  enddo
               enddo
            enddo
         enddo
         der(1)=der(1)-dsdc(1,1)
         der(2)=der(2)-dsdc(2,2)
         der(3)=der(3)-dsdc(3,3)
         der(4)=der(4)-dsdc(1,2)
         der(5)=der(5)-dsdc(1,3)
         der(6)=der(6)-dsdc(2,3)
      endif
c      print*,' listk: ',(der(m),m=1,3)
      end
