      subroutine nonbe(ngr,nrp,enb,nf,der,nl,box,x,f,iprint)
C     Calculates nonbonded crystal energy, forces and virial.
C     B.P. van Eijck, october 1994.

C     One molecule with NRP atoms is augmented with NSPGR symmetry-related
C     ones. The nonbonded distances are selected using a cutoff criterion
C     based on the geometric centers of charge groups.
C     The nonbonded energy ENB is calculated, as well as the forces on the
C     atoms F and the negative derivatives (also called "forces") of ENB
C     to the cell parameters ax, by, cz, bx, cx, cy.
C     It is possible to prepare pair lists, which can be used by other
C     subroutines for quicker and more stable energy minimization.

C     The energy calculation includes Coulomb terms, R^(-6) attractive
C     dispersion terms, and either R^(-12) or exponential repulsion terms.
C     It is calculated on the basis of NGR groups.
C     Ewald summation is possible for Coulomb and dispersion terms.
C     If this is not done, the calculation of U2-inner can be performed.
C     An intermolecular polarization term can be calculated.

C     Input through parameters:
C     NGR: number of groups
C     NRP: number of atoms
C     NF rules the calculation of derivatives:
C        NF=0: no derivatives
C        NF=1: calculates forces F and DER
C        NF=2: calculates forces F and virial VIR
C     NL rules making of a pair lists, which can be used by other subroutines
C        for quicker and more stable calculations in energy minimization:
C        NL=0: no pair list
C        NL=1: pair list for all atoms (for use in LIST)
C        NL=2: pair list for hydrogen atoms (for use in LISTH)
C     BOX(1..6): ax, by, cz, bx, cx, cy
C     X(1..): atom coordinates. X must be big enough to accomodate
C        all symmetry-expanded coordinates.
C     IPRINT regulates output to unit 16 and to screen.

C     Input through common blocks:
C     /POLA/ gives polarization data:
C        NPP=0: no polarization
C        NPP=1: intermolecular polarization with neglect of the effect
C               of the induced dipoles on the electric field.
C        NPP>1: call NONBEP for full polarization calculation.
C        ALFPOL(1..): polarizabilities
C     /NONP/ gives information on nonbonded parameters:
C        NCAG: number of charge groups in one molecule
C        IDO(1..NRP) (0 or 1) disables/enables atoms
C        IAC(1..NRP): atom codes, taking values 1..Ncod
C        IEXCL(1..NRP,1..NRP): excluded neighbors
C        INC(0..NCAG): charge group pointer list: INC(I) specifies the
C           position of the last atom of the I-th group in the atom sequence.
C           (This algorithm is taken from the GROMOS procedure NONBML)
C        C6(1..Ncod,1..Ncod): attractive C6 parameters
C        CS6(1..Ncod,1..Ncod): same for Lennard-Jones 1..4 interactions
C        C12(1..Ncod,1..Ncod),CS12(1..Ncod,1..Ncod):
C           for Lennard-Jones potential: same for repulsive terms
C           for Buckingham potential: repulsion C12 * exp(-CS12 * R)
C        SC14C, SC14LJ: multiply Lennard-Jones 1..4 Coulomb and other
C           terms with these factors, respectively.
C     /NONB/ gives various information:
C        CG(1..NRP): charges
C        EBIG: if any repulsion term exceeds EBIG, stop immediately and
C           set ENB=90000.
C        BUCK: if TRUE, then choose Buckingham potential
C        NOINT: if TRUE, skip intramolecular terms
C        DDC: if TRUE, use distance-dependent dielectric constant R (A).
C        DRUK: the pressure
C        CUTOFF: maximum distance for charge group interactions
C        MSOM(m) = sum over MSP(m), compares with IREF (from LEESR).
C        INNER: do calculation of U2-inner if INNER>0.
C        FINNER: 4*pi/(3.0*(nspgr+1))
C        TOLE: omit atom pairs which contribute less than TOLE to
C              any of the main three energy contributions.
C        EPSIL: the dielectric constant for charges read with ICHAD<0.
C     /SPGR/ gives space group information:
C        NSPGR, MSP, TSP: see subroutine LEESR
C     /INDS/ gives hydroxyl dihedral information (only needed if NL=2):
C        NDIHH: number of hydroxyl dihedrals
C        NDIHZ: number of dihedral that preceeds the first hydroxyl dihedral
C        IOME(1..NDIHH): number of the hydrogen atom that is rotated
C     /WALD/ gives information for Ewald summation. The last symbols C and 6
C        refer to Coulomb terms and attractive r^(-6) terms, respectively.
C        This is only carried out if logical DOEW is TRUE.
C        ALPHAC, ALPHA6: damping factors in nm^(-1).
C        HMAXC, HMAX6: maximum reciprocal lattice vectors in nm^(-1).
C        DOEWC, DOEW6: if FALSE, then skip that part of the calculation.
C     /CHAD/ gives information for virtual atoms.
C        ICHAD rules the use of this feature:
C           ICHAD=0: no virtual atoms
C           ICHAD<0: read charge information from tape 48, 
C                    see below for the precise format
C           ICHAD>0: use this previously found information
C        NVIR: the number of virtual atoms
C        PCHAD, QCHAD, SCHAD(1..NVIR), JVIR(1..NVIR,1..3):
C           the vectors w = r(jvir(2)) - r(jvir(1))
C                   and v = r(jvir(3)) - r(jvir(1)) and R = [v x w]
C           define a virtual atom at vector r(jvir(1)) + p w [+ q v [+ s R]]
C        NCHAD(1..NVIR) defines how many (3, 5, or 6) of these data are used
C        DVIR is a scratch-array for derivatives, initialized in LEEST
C        INDMOL(0..) defines the molecule of each real atom
C        INDVIR(1..NVIR) defines the molecule of each virtual atom
C        NCAGS is the modified number of charge groups.

C     Output through parameters:
C     ENB: the nonbonded energy
C     Special cases:
C        ENB = 60000: a call to KWALD has failed (too many RL vectors)
C        ENB = 70000: a cell axis is less than 0.5 Angstrom.
C        ENB = 80000: a distance is smaller than 0.5 A
C        ENB = 90000: a repulsion term exceeds EBIG
C     DER(1..6): the negative derivatives of ENB to BOX(1..6)
C     F(1..3*NGR*NRP): the negative derivatives of ENB to X(1..3*NGR*NRP).
C     Array X: expanded with the symmetry-related molecules.

C     Output through common blocks:
C     /NBAL/ gives pair list information for LIST:
C        NPLA: number of pairs in the list
C           The various arrays give information for the calculation of
C           nonbonded pair interactions:
C        IPLA1, IPLA2: coordinates are located in X(IPLA+M) (M=1,2,3)
C        PLJ6, PLJ12, PLJ6EW, PEXP: Lennard-Jones or Buckingham coefficients
C        PCG, PCGEW: Coulomb coefficients
C        SHIFT1..3: fractional shifts for required symmetry operation
C        ISIG1..3: signs for required symmetry operation
C        And scratch arrays FLR and ING
C     /NBHL/ gives similar pair list information for LISTH, e.g:
C        NPLH(1..): number of pairs for every hydroxyl H atom.
C     /NONB/
C        PMOL(1..3): the cell dipole moment
C        UINNER: the calculated U2-inner
C        EEL: the coulomb energy
C        EDISPA: the attractive dispersion term
C        EDISPR: the repulsive term
C     /WALD/:
C        ECC, EC6: Ewald contribution from constant terms (from KWALD)
C        EKC, EK6: Ewald contribution from reciprocal terms (from KWALD)
C        EDC, ED6: Ewald contribution from direct terms (this subroutine)
C     /POLA/: polarization output if NPP=1:
C        DIP(1..): induced dipole moments
C        EPOL: the polarization energy.
C     /VIRA/:
C        VIRA(1..3,1..3): contribution to virial

C     Calls subroutines KWALD and NONBEP

      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cvira.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cnbhl.i'
      include 'cinds.i'
      include 'cwald.i'
      include 'cpola.i'
      include 'chadd.i'

      double precision al6kw,atsqpi,ax,box,bx,by,
     +     c12def,c6def,c6r,cbu,cgdef,cgi,cgij,
     +     ch12,ch6,cx,cy,cz,d,d2,de1,de2,de3,denuli,denulj,der,df,
     +     dip1,dip2,dip3,dipind,djp1,djp2,djp3,ebu,enb,enul,eps,erfcc,
     +     ewco6,ewcoc,exa,f,facq,fact,factor,fi,fj,
     +     p,pi,pip,pir,pjr,q,q1,qi,qj,rba,rca,rcutp2,
     +     rij,rij1,rij2,rij3,rij6,rklrij,s,stuk2,totcg,vda,vdr,vel,
     +     velcc,vol,vp3v,vp3vn,x,xd,xij,xij0,xsqc,xsqlj,yd,yij,yij0,
     +     zd,zij,zij0
      integer i,i2,i3,ia,ia3,iaci,iacj,ib,ib3,ibox,ic,ic3,icj,
     +     ii,ii0,ii3,iim,iimax,im,intra,ios,iprint,ireal,
     +     is1,is2,is3,istart,ivirt,j,j3,jd,jj,jj0,jj3,jjm,jjmax,jpt,
     +     k,k3,km,m,m1,m2,mh,n,n1,n2,n3,nag,natoms,nbl,ncagt,nf,ng,ngr,
     +     ni,ni3,nim,nj,nj3,njm,nl,nnj,nop3,nr3,nragt,nrp,nrpt,nsj,
     +     nvirt,nword,nxyz,ncoll
      dimension der(*),box(*),x(*),f(*),rij(3),dipind(3),rba(3),rca(3),
     +     enul(maxxco)
      logical skip
      character regel*100,atn*5

C     Reduce fractional coordinates of separate molecules
C     This is seldom necessary, and it destroys the D-factor in PREP
ccc      nrpt=nrp*ngr
ccc      ax=box(1)
ccc      by=box(2)
ccc      cz=box(3)
ccc      bx=box(4)
ccc      cx=box(5)
ccc      cy=box(6)
ccc      bb=sqrt(bx**2+by**2)
ccc      cc=sqrt(cx**2+cy**2+cz**2)
ccc      cosa=(bx*cx+by*cy)/(bb*cc)
ccc      cosb=cx/cc
ccc      cosc=bx/bb
ccc      call tri(nrpt,0,x,1,cosa,cosb,cosc)
ccc      do i=1,nrpt
ccc         x(3*i-2)=x(3*i-2)/ax
ccc         x(3*i-1)=x(3*i-1)/bb
ccc         x(3*i)=x(3*i)/cc
ccc      enddo
ccc      do imol=1,nmols
ccc         i1=imols(imol)
ccc         i2=imols(imol+1)-1
ccc         do m=1,3
ccc            rij(m)=x(3*i1-3+m)-x(m)
ccc         enddo
ccc         do j=i1,i2
ccc            do m=1,3
ccc               x(3*j-3+m)=x(3*j-3+m)-NINT(rij(m))
ccc            enddo
ccc         enddo
ccc      enddo
ccc      do i=1,nrpt
ccc         x(3*i-2)=x(3*i-2)*ax
ccc         x(3*i-1)=x(3*i-1)*bb
ccc         x(3*i)=x(3*i)*cc
ccc      enddo
ccc      call tri(nrpt,0,x,-1,cosa,cosb,cosc)

      facq=sqrt(332.0636d0*0.4184d0)
      ncagt=ncag*ngr
      ncoll=0
      if (ichad.gt.0) ncagt=ncags
      nrpt=nrp*ngr
      nvirt=nvir*ngr
      natoms=nrpt+nvirt
c      do i=1,2*nrp*ngr
c               print 41,i,(x(3*i-3+m),m=1,3),indmol(i),iac(i),ido(i),
c     +              cg(i)/facq
c      enddo
c      do i=1,3*nvir
c               print*,i,indvir(i)
c      enddo
c      pause

ccc      print*,'NONBE',nop,nspgr
ccc  223 format(3(3i4,f8.4,4x))
ccc      do j=1,nop
ccc         print 223,((mspg(j,m1,m2),m2=1,3),tspg(j,m1),m1=1,3)
ccc      enddo         
ccc      pause
ccc      ngr=1

      if (iprint.gt.1) then
         print 40, ngr,nrp,nvir,ichad,nspgr,npp,cutoff,ebig,druk
         if (noint) print*,
     +        '   No calculation of intramolecular nonbonded energy'
         if (buck) print*,'   Buckingham potential used'
         if (ddc) print*,
     +        '   Distance-dependent dielectric constant used'
         if (inner.gt.0) print*,'   U2-inner included'
         print 90,(box(m),m=1,6)
      endif
      do m=1,3
         if (box(m).lt.0.05d0) then
            print*,' ***** NONBE: Cell axes less than 0.5 Angstrom.'
            write(16,*)' ***** NONBE: Cell axes less than 0.5 Angstrom.'
            enb=70000
            return
         endif
      enddo

      if (nvir.gt.0) then
C        Create NVIRT additional atom sites for each independent molecule
         noint=.true.
         if (nvirt.gt.maxvir) then
            print*,' ***** Increase maxvir for', ngr,' groups:',nvirt
            stop
         endif
         if (iprint.gt.1) 
     +        print*,'Creating',nvirt,' non-atomic charge sites'
         do i=1,nvirt
            n=nchad(i)
            p=pchad(i)
            q=qchad(i)
            s=schad(i)*10
C           S must have (temporarily?) the dimension A^(-1)
            n3=3*(nrpt+i)-3
            i3=3*i-3
            iac(nrpt+i)=0
            do m=1,3
               ibase(nrpt+i,m)=jvir(i,m)
            enddo
            ia3=3*jvir(i,1)-3
            ib3=3*jvir(i,2)-3
            ic3=3*jvir(i,3)-3
            if (iprint.gt.1) then
c               print*,(x(ia3+m),m=1,3)
               if (n.eq.3) print 14,i,jvir(i,1),jvir(i,2),p
               if (n.eq.5) print 14,i,jvir(i,1),jvir(i,2),p,i3,q
               if (n.eq.6) print 14,i,jvir(i,1),jvir(i,2),p,i3,q,s/10
            endif
            if (n.ge.3) then
               do m=1,3
                  rba(m)=x(ib3+m)-x(ia3+m)
               enddo
            endif
            if (n.ge.5) then
               do m=1,3
                  rca(m)=x(ic3+m)-x(ia3+m)
               enddo
            endif
            if (n.eq.3) then
C              Virtual atom in a bond
               do m=1,3
                  x(n3+m)=x(ia3+m)+p*rba(m)
               enddo
               if (iprint.gt.2) print 20,'in bond ',i,(x(n3+m),m=1,3)
            endif
            if (n.eq.5) then
C              Virtual atom in a plane
               do m=1,3
                  x(n3+m)=x(ia3+m)+p*rba(m)+q*rca(m)
               enddo
               if (iprint.gt.2) print 20,'in plane',i,(x(n3+m),m=1,3)
            endif
            if (n.eq.6) then
C              Virtual atom in three dimensions
               rij(1)=rba(2)*rca(3)-rba(3)*rca(2)
               rij(2)=rba(3)*rca(1)-rba(1)*rca(3)
               rij(3)=rba(1)*rca(2)-rba(2)*rca(1)
               d2=0
               do m=1,3
                  d=p*rba(m)+q*rca(m)+s*rij(m)
                  d2=d2+d*d
                  x(n3+m)=x(ia3+m)+d
                  rba(m)=s*rba(m)
                  rca(m)=s*rca(m)
                  rij(m)=rba(m)-rca(m)
               enddo
               if (iprint.gt.2) print 20,'in 3 dim',i,(x(n3+m),m=1,3),
     +              sqrt(d2)
            endif

C           Find derivatives DVIR(k,j,m1,m2) = d[virtual coord k(m1=x,y,z)]
C                / d[real coord j=a,b,c(m2=x,y,z)]
            do m=1,3
               dvir(i,1,m,m)=1-p-q
               dvir(i,2,m,m)=p
               dvir(i,3,m,m)=q
               if (n.eq.6) then
                  dvir(i,1,3,2)=-rij(1)
                  dvir(i,1,1,3)=-rij(2)
                  dvir(i,1,2,1)=-rij(3)
                  dvir(i,1,2,3)= rij(1)
                  dvir(i,1,3,1)= rij(2)
                  dvir(i,1,1,2)= rij(3)

                  dvir(i,2,3,2)=-rca(1)
                  dvir(i,2,1,3)=-rca(2)
                  dvir(i,2,2,1)=-rca(3)
                  dvir(i,2,2,3)= rca(1)
                  dvir(i,2,3,1)= rca(2)
                  dvir(i,2,1,2)= rca(3)

                  dvir(i,3,2,3)=-rba(1)
                  dvir(i,3,3,1)=-rba(2)
                  dvir(i,3,1,2)=-rba(3)
                  dvir(i,3,3,2)= rba(1)
                  dvir(i,3,1,3)= rba(2)
                  dvir(i,3,2,1)= rba(3)
               endif
            enddo
         enddo
      endif

      if (ichad.lt.0) then
C        Read charges on NG groups of NRP real and NVIR virtual atoms.
C        Each group may consist of more than one molecule, 
C           following the topology.
C        Input:
C         - The number of real + virtual atoms = NG*(NRP+NVIR)
C         - One line of text
C        The for each separate molecule: 
C         - atom name, [other data (not used),] charge as last entry
C           First must come the real atoms and then the virtual atoms,
C           whose atom names must be X.
C        If NG does not cover all independent groups, charges are expanded.

         open(48,file='molden.xyz',status='OLD',iostat=ios)
         if (ios.eq.0) then
            print*,' Opening XYZ file from MOLDEN'
         else
            open(48,file='chadd.xyz',status='OLD',iostat=ios)
            if (ios.eq.0) then
               print*,' Opening hand-made XYZ file'
            else
               print*,' ***** No XYZ file present'
               stop
            endif
         endif
         nxyz=0
         ireal=0
         ivirt=0
         eps=sqrt(epsil)
  120    continue
         read(48,21,err=140,end=140) regel
         j=nword(regel)
         if (j.eq.0) goto 120
         if (j.eq.1) call intstr(regel,1,n)
         if (j.gt.1) then
            print*,' Unexpected line on XYZ file:'
            print*, regel
            print*,' Try reading this as a CSSR file.'
            read(48,21,err=130,end=130) regel
            read(48,*,err=130,end=130) nxyz
            read(48,21,err=130,end=130) regel
            print*,nxyz
            do i=1,nxyz
               read(48,21,err=135,end=135) regel
               j=nword(regel)
               call strext(regel,2,atn)
               call realst(regel,j-1,q)
               q=q/eps
               do j=0,ngr
                  if (i+j*nrp.le.nrpt) cg(i+j*nrp)=q
               enddo
               print 44,i,i,atn,q
               write(16,44) i,i,atn,q
            enddo
            goto 140
  130       continue
            print*,' ***** No useful CSSR file found'
            stop
         endif

  127    continue
         read(48,21) regel
         print 23,n,regel
         write(16,23) n,regel
         nxyz=nxyz+n
         do i=1,n
            read(48,21,err=135,end=135) regel
            j=nword(regel)
            if (j.lt.2) goto 135
            call strext(regel,1,atn)
            call realst(regel,j,q)
            q=q/eps
            enul(i)=q
            if (atn.eq.'X') then
               ivirt=ivirt+1
               k=nrpt+ivirt
               cg(k)=q
               sq6(k)=0
C              No dispersion for virtual atoms
               indmol(k)=-indvir(ivirt)
C              Expand since tape 48 may contain insufficient groups
               do j=1,ngr-1
                  cg(k+j*nvir)=cg(k)
                  sq6(k+j*nvir)=sq6(k)
                  indmol(k+j*nvir)=indmol(k)-j*indmol(nrp)
c                  print*,'virt',k,indmol(k),k+j*nvir,
c     +                 indmol(k)-j*indmol(nrp)
               enddo
C              Virtual atoms have a negative molnumber
               m=nchad(ivirt)
               p=pchad(ivirt)
               q1=qchad(ivirt)
               s=schad(ivirt)
               iac(nrpt+i)=0
               ia=jvir(ivirt,1)
               ib=jvir(ivirt,2)
               ic=jvir(ivirt,3)
               if (m.eq.3) print 43,i,ivirt,atn,q,m,ia,ib,p
               if (m.eq.5) print 43,i,ivirt,atn,q,m,ia,ib,p,ic,q1
               if (m.eq.6) print 43,i,ivirt,atn,q,m,ia,ib,p,ic,q1,s
c               if (m.eq.3) write(16,43) i,ivirt,atn,q,m,ia,ib,p
c               if (m.eq.5) write(16,43) i,ivirt,atn,q,m,ia,ib,p,ic,q1
c               if (m.eq.6) write(16,43) i,ivirt,atn,q,m,ia,ib,p,ic,q1,s
            else
               ireal=ireal+1
               do j=0,ngr
                  if (ireal+j*nrp.le.nrpt) cg(ireal+j*nrp)=q
               enddo
               print 44,i,ireal,atn,q
c               write(16,44) i,ireal,atn,q
            endif
         enddo
         write(16,45) (enul(j),j=1,n)

  126    continue
         read(48,21,err=140,end=140) regel
         j=nword(regel)
         if (j.eq.0) goto 126
         if (j.eq.1) call intstr(regel,1,n)
         goto 127
  135    continue
         print*,' ***** Incorrect data on XYZ file.'
         stop
  140    continue
         close(48)

         ng=nxyz/(nrp+nvir)
         print 24,nxyz,ng
         write(16,24) nxyz,ng
         if (nxyz.ne.natoms .or. ng.ne.ngr) then
            print*,' ***** But', nrpt,' atoms in' ,ngr,
     +           ' group(s) were expected.'
            print*,'nrp, nvir =',nrp,nvir
            stop
         endif
         ng=natoms/nxyz
         if (nxyz.lt.natoms) print*,
     +        'Expanding',ng,' times to',natoms,' (virtual) atoms.'
         totcg=0
         do i=1,natoms
            cg(i)=cg(i)*facq
            totcg=totcg+cg(i)
         enddo
         if (abs(totcg).gt.0.0001) then
            print 22,totcg/facq
            write(16,22) totcg/facq
         endif
         do i=1,natoms
            cg(i)=cg(i)-totcg/natoms
         enddo
c  200    continue

C        Expand (and possibly modify) charge group pointers INC
         istart=nrpt+1
C        If no Ewald summation is done, each molecule must be a charge group:
         if (.not. doewc) then
            istart=1
            ncagt=0
         endif
         do i=istart,natoms
            if (indmol(i).ne.indmol(i-1)) then
               inc(ncagt)=i-1
               ncagt=ncagt+1
            endif
         enddo
         inc(ncagt)=natoms
         ncags=ncagt

C        Expand various data for equivalent positions
         do m=1,maxat/natoms
            im=(m-1)*natoms
            do i=1,natoms
               cg(i+im)=cg(i)
               sq6(i+im)=sq6(i)
               indmol(i+im)=indmol(i)+sign((m-1)*indmol(nrpt),indmol(i))
               do m1=1,3
                  ibase(i+im,m1)=ibase(i,m1)+(m-1)*natoms
               enddo
            enddo
         enddo
         do m=2,maxcag/ncagt
            im=(m-1)*natoms
            km=(m-1)*ncagt
            do i=1,ncagt
               inc(i+km)=inc(i)+im
            enddo
         enddo

         if (iprint.gt.2) then
            totcg=0
            do i=1,natoms
               totcg=totcg+cg(i)
               if (indmol(i).gt.0) then
                  print 41,i,(x(3*i-3+m),m=1,3),indmol(i),ido(i),iac(i),
     +                 cg(i)/facq
               else
                  print 41,i,(x(3*i-3+m),m=1,3),indmol(i),ido(i),iac(i),
     +                 cg(i)/facq,(ibase(i,m),m=1,3)
               endif      
            enddo
            print*,' Charge groups ',(inc(m),m=1,ncagt)
            print*,' Total charge',totcg/facq
         endif
         ichad=abs(ichad)
      endif

      if (npp.gt.1) then
         call nonbep(ngr,nrp,enb,nf,der,nl,box,x,f,iprint)
         return
      endif

      if (nl.eq.2) then
         do m=1,ndihh
            nplh(m)=0
         enddo
      endif
      if (nf.eq.2) then
         do m=1,3
            do n=1,3
               vira(m,n)=0
            enddo
         enddo
      endif

      npla=0
      rcutp2=cutoff**2
      nragt=(nspgr+1)*ncagt
      if (nragt.gt.maxcag) then
         print*,' ***** NONBE: Increase maxcag',maxcag,nragt
         stop
      endif
      nr3=3*natoms
      pi=4*atan(1.0d0)
      atsqpi=2*alphac/sqrt(pi)
      al6kw=alpha6**2
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      edc=0
      ed6=0
      ecc=0
      ec6=0
      ekc=0
      ek6=0
      eel=0
      edispa=0
      edispr=0
      ewcoc=0
      ewco6=0
      Epol=0

C     Calculate cell dipole moment PMOL; initialize forces F
      uinner=0
      vol=ax*by*cz
      enb=druk*vol
      vp3v=4*pi/(3*vol)
      vp3vn=finner/vol
C     This is twice the energy factor 2 pi / [3 V (nspgr+1)]
      do m=1,3
         p=0
         do i=1,natoms
            p=p+x(3*i-3+m)*msom(m)*cg(i)
         enddo
         uinner=uinner-vp3vn*p*p/2
         pmol(m)=p
         if (inner.eq.0) p=0
         if (nf.gt.0) then
            do i=1,natoms
               f(3*i-3+m)=vp3vn*msom(m)*cg(i)*p
            enddo
         endif
      enddo
C     Decide whether to include U2-inner in EEL
      if (inner.ne.0) eel=uinner
C     Initialize field strengths with 4 pi dipole / [3 V]
      do i=1,natoms
         do m=1,3
            enul(3*i-3+m)=vp3v*pmol(m)
         enddo
      enddo
C     Iinitialize cell derivatives
      if (nf.eq.1) then
         do m=1,3
            der(m)=(eel-enb)/box(m)
            der(m+3)=0
         enddo
      endif
      if (iprint.gt.0) then
         print 86,uinner
         print 93,(pmol(m),m=1,3)
         print 94,(vp3v*pmol(m),m=1,3)
c         write(16,86) uinner
c         write(16,93) (pmol(m),m=1,3)
c         write(16,94) (vp3v*pmol(m),m=1,3)
      endif

      if (doew) then
C        Calculate reciprocal part of Ewald summation. This includes
C        the expansion with equivalent positions
         call kwald(natoms,nf,der,box,x,f)
         if (ekc.gt.59999) then
            enb=60000
            return
         endif
      else
         if (nspgr.gt.0) then
C           Do expansion with equivalent positions
            do nop=1,nspgr
               nop3=3*nop-3
               do i=1,natoms
                  j=3*i-3
                  k=j+nop*nr3
                  x(k+1)=msp(nop3+1)*x(j+1)+
     +                 tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
                  x(k+2)=msp(nop3+2)*x(j+2)+
     +                 tsp(nop3+2)*by+tsp(nop3+3)*cy
                  x(k+3)=msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
               enddo
            enddo
         endif
      endif
      if (iprint.gt.4) then
         print*,'Coordinates (after expansion):'
         do i=1,natoms*(nspgr+1)
            print 41,i,(x(3*i-3+m),m=1,3),indmol(i),ido(i),iac(i),
     +           cg(i)/facq
         enddo
      endif

C     Calculate the centers of gravity of the charge groups
      i=0
      DO N=1,NRAGT
         DO M=1,3
            rij(M)=X(3*i+M)
         enddo
         i=i+1
         q=cg(i)
         nag=inc(n)-inc(n-1)
         DO J=2,NAG
            DO M=1,3
               RIJ(M)=RIJ(M)+X(3*i+M)
            enddo
            i=i+1
            q=q+cg(i)
         enddo
         DO M=1,3
            flr(3*n-3+m)=rij(m)/nag
         enddo
         m1=indmol(inc(n))
         if (m1.gt.0) ing(m1)=n
         if (m1.lt.0) then
            i2=ing(-m1)
            do m=1,3
               flr(3*n-3+m)=flr(3*i2-3+m)
            enddo
         endif
         if (iprint.gt.2) print 42,n,inc(n),m1,q/facq,
     +        (flr(3*n-3+m),m=1,3)
      enddo

      DO NI=1,ncagt
C        Start the next charge group NI in one asymmetric unit
         if (iprint.gt.8) print 81
         if (iprint.gt.3) print 80,ni
         NI3=3*NI-3
         ii0=inc(ni-1)+1
         iimax=inc(ni)
         DO NJ=ni,NRAGT
C           Start the next charge group NJ, including equivalent positions.
C           Make a list of translations needed to find all charge groups
C           combinations within CUTOFF. The translations are stored in array
C           ING in groups (N1, N2, N3) for every translation 1...NSJ.
            if (iprint.gt.9) print*,' Pair list for charge groups',ni,nj
            NJ3=3*NJ-3
            nsj=0
            xij0=flr(ni3+1)-flr(nj3+1)
            yij0=flr(ni3+2)-flr(nj3+2)
            zij0=flr(ni3+3)-flr(nj3+3)
C           Collect charge group distances within CUTOFF.
            n3=int((cutoff-zij0)/cz)
            zij=zij0+n3*cz
            n2=int((cutoff-yij0-n3*cy)/by)
            yij=yij0+n2*by+n3*cy
            n1=int((cutoff-xij0-n2*bx-n3*cx)/ax)
            xij=xij0+n1*ax+n2*bx+n3*cx
  350       continue
            rij2=xij**2+yij**2+zij**2
C           Always include intramolecular distances
            if (rij2.lt.rcutp2 .or. 
     +           (n1.eq.0 .and. n2.eq.0 .and.n3.eq.0)) then
c     +           abs(indmol(inc(ni))).eq.abs(indmol(inc(nj)))) then
               nsj=nsj+1
               if (iprint.gt.9) print 155,nsj,ni,nj,n1,n2,n3,
     +              xij,yij,zij,sqrt(rij2)
               if (nsj.gt.maxnsj) then
                  print*,' ***** NONBE: Increase maxnsj',nsj
                  stop
               endif
               jpt=3*(nsj-1)
               ing(jpt+1)=n1
               ing(jpt+2)=n2
               ing(jpt+3)=n3
            endif
            n1=n1-1
            xij=xij-ax
  370       if (xij.lt.-cutoff) then
               n2=n2-1
               yij=yij-by
  380          if (yij.lt.-cutoff) then
                  n3=n3-1
                  zij=zij-cz
                  if (zij.lt.-cutoff) goto 390
                  n2=int((cutoff-yij0-n3*cy)/by)
                  yij=yij0+n2*by+n3*cy
                  goto 380
               endif
               n1=int((cutoff-xij0-n2*bx-n3*cx)/ax)
               xij=xij0+n1*ax+n2*bx+n3*cx
               goto 370
            endif
            goto 350
  390       continue
C           End of making list ING(1..NSJ)

C           Start the actual calculation for charge groups NI and NJ
            velcc=0
            IF (nsj.EQ.0) GOTO 480
            ICJ=(NJ-1)/NCAGT
C           ICJ is the number of the equivalent group, 0..NSPGR.
            NNJ=NJ-NCAGT*ICJ
            jj0=inc(nnj-1)+1
            jjmax=inc(nnj)
            if (iprint.gt.4) print 152,ni,ii0,iimax,nj,jj0,jjmax,icj

            do ii=ii0,iimax
C              Start the next atom II, if it is within charge group NI
C              and allowed by IDO
               nim=indmol(ii)
               iim=mod(ii-1,nrp)+1
               if (ido(ii).eq.0) goto 475
               CGI=CG(II)
               IACI=IAC(II)
               II3=3*ii-3

               do jj=jj0,jjmax
C                 Start the next atom JJ, if it is within charge group NJ
C                 and allowed by IDO.
C                 JJ0 and JJMAX were reduced to the non-expanded group
                  njm=indmol(jj+natoms*icj)
                  jjm=mod(jj-1,nrp)+1
C                 NIM and NJM are molecule numbers, negative for virtual atoms.
C                 IIM and JJM indicate the atom numbers in the main group.
                  if (iprint.gt.7) print 92,ni,nj,ii,jj,jj+natoms*icj,
     +                 nim,njm,iim,jjm,ido(jj)
                  if (jj.lt.ii .or. ido(jj).eq.0) goto 470
                  factor=1
                  if (jj.eq.ii) factor=0.5d0
                  jj3=3*jj-3
                  jd=3*icj
                  j3=jj3+natoms*jd
                  if (jd.eq.0) then
                     is1=1
                     is2=1
                     is3=1
                     de1=0
                     de2=0
                     de3=0
                  else
                     is1=msp(jd-2)
                     is2=msp(jd-1)
                     is3=msp(jd)
                     de1=-tsp(jd-2)
                     de2=-tsp(jd-1)
                     de3=-tsp(jd)
                  endif
                  iacj=iac(jj)
                  c6def=c6(iaci,iacj)*factor
                  c12def=c12(iaci,iacj)*factor
                  cgdef=cgi*cg(jj)*factor
                  if (buck) then
                     cbu=c12def
                     ebu=cs12(iaci,iacj)
                     c12def=0
                  endif
                  xij0=x(ii3+1)-x(j3+1)
                  yij0=x(ii3+2)-x(j3+2)
                  zij0=x(ii3+3)-x(j3+3)

                  do ibox=1,nsj
C                    Start the next cell translation from ING
                     ch6=c6def
                     ch12=c12def
                     cgij=cgdef
                     intra=0
                     skip=.false.
                     n1=ing((ibox-1)*3+1)
                     n2=ing((ibox-1)*3+2)
                     n3=ing((ibox-1)*3+3)
                     if (icj.eq.0 .and. 
     +                    n1.eq.0 .and. n2.eq.0 .and. n3.eq.0) then
C                       Possibly an intramolecular distance
                        if (abs(nim).eq.abs(njm)) then
                           intra=1
                           if (njm.gt.0) intra=iexcl(iim,jjm)
                        endif
                        if (intra.eq.2) then
C                          Change and/or scale 1..4 interaction parameters
                           cgij=cgij*sc14c
                           if (.not. buck) then
                              ch12=cs12(iaci,iacj)*factor*sc14lj
                              ch6=cs6(iaci,iacj)*factor*sc14lj
                           endif
                        endif
                        skip=(intra.eq.1 .or. (noint.and.intra.ne.0))
                     endif
C                    Skip interaction for excluded neighbors
C                    Skip intramolecular interaction if NOINT is true.
                     if (.not. doew .and. skip) goto 450
                     xij=xij0+n1*ax+n2*bx+n3*cx
                     yij=yij0+n2*by+n3*cy
                     zij=zij0+n3*cz
                     rij2=xij**2+yij**2+zij**2
                     if (n1.eq.0 .and. n2.eq.0 .and. n3.eq.0 .and.
     +                    ii.eq.jj .and. icj.eq.0) goto 450
                     rij1=sqrt(rij2)
                     if (rij1.lt.1.0e-6) then
                        print*,' ***** Unacceptably small distance ',
     +                       'in NONBE:',ii,jj,rij1
                        enb=80000
c                        return
                        stop
                     endif
                     if (rij1.lt.0.05 .and. (abs(nim).ne.abs(njm))) then
                        if (iprint.gt.0) print*,
     +                       '***** NONBE: short distance: ',ii,jj,rij1
                        enb=80000
                        return
                     endif

C                    Calculate harmonic terms (testcase)
c                     b0=0.345
c                     f0=592.8166*factor
c                     if (skip) f0=0
c                     vel=1+0.5*f0*(rij1-b0)**2
c                     df=f0*(rij1-b0)/rij1
c                     cgij=f0
c                     if (b0.gt.-0.1) goto 400

C                    Calculate Coulomb energy
C                    Distance dependent dielectric constant if DDC=TRUE.
                     vel=cgij/rij1
                     if (skip) vel=0
                     if (nf.gt.0) df=-vel/rij2
                     if (ddc) then
                        vel=vel/rij1
c                        df=2*df/rij1
                        df=-2*vel/rij2
                     endif
                     if (doewc) then
C                       Calculate Ewald correction
                        rklrij=alphac*rij1
                        xsqc=rklrij*rklrij
                        exa=0
                        if (xsqc.lt.700) exa=exp(-xsqc)
                        ewcoc=cgdef*(erfcc(rklrij)-1)/rij1
                        if (nf.gt.0) df=df-(ewcoc+cgdef*atsqpi*exa)/rij2
                     endif

                     if (iaci.eq.0 .or. iacj.eq.0) then
                        vda=0
                        vdr=0
                        ewco6=0
                        goto 400
                     endif
                     
C                    Calculate attractive r^(-6) terms
                     rij6=rij2*rij2*rij2
                     vda=ch6/rij6
                     if (skip) vda=0
                     if (nf.gt.0) df=df+6*vda/rij2
                     if (doew6) then
C                       Calculate Ewald correction
                        c6r=c6def/rij6
                        xsqlj=al6kw*rij2
                        exa=0
                        if (xsqlj.lt.700) exa=exp(-xsqlj)
                        fact = (1 + xsqlj * (1 + xsqlj/2)) * exa - 1
                        ewco6=c6r*fact
                        if (nf.gt.0) df=df+(c6r/rij2)*
     +                       (6*fact+exa*xsqlj*xsqlj*xsqlj)
                     endif

C                    Calculate R^(-12) or exponential repulsion
                     if (skip) then
                        vdr=0
                     else
                        if (buck) then
                           xsqc=ebu*rij1
                           exa=0
                           if (xsqc.lt.700) exa=exp(-xsqc)
                           vdr=cbu*exa
                           if (nf.gt.0) df=df-ebu*vdr/rij1
                        else
                           vdr=ch12/(rij6*rij6)
                           if (nf.gt.0) df=df-12*vdr/rij2
                        endif
                     endif

  400                continue
C                    Throw away contributions smaller than TOLE
                     if (abs(vel+ewcoc).lt.tole .and.
     +                    abs(vda+ewco6).lt.tole .and.
     +                    abs(vdr).lt.tole .and. npp.eq.0) goto 450
c                        print 555,tole,vel,vda,vel+ewcoc,vda+ewco6,vdr
c 555                    format(10f10.4)
                     eel=eel+vel
                     edispa=edispa-vda
                     edispr=edispr+vdr
                     edc=edc+ewcoc
                     ed6=ed6-ewco6
                     if (edispr.gt.ebig) then
                        if (iprint.gt.1) print*,
     +                       'NONBE: energy > EBIG',rij1,vdr
                        enb=90000
                        return
                     endif
                     if (iprint.gt.8) then
                        if (buck) then
                           print 88,n1,n2,n3,xij,yij,zij,rij1,
     +                          cgij,ch6,cbu,ebu,vel,-vda,vdr
                        else
                           print 89,n1,n2,n3,xij,yij,zij,rij1,
     +                          cgij,ch6,ch12,vel,-vda,vdr
                        endif
                     endif
                     velcc=velcc+vel

                     if (abs(vel)+vda+vdr.gt.1.0d6) then
c                     if (abs(vel)+vda+vdr.gt.100) then
                        ncoll=ncoll+1
                        print 880, ncoll,n1,n2,n3
                        print 881, icj,natoms
                        print 882, ii,jj,xij,yij,zij,rij1
                        print 883, vel,-vda,vdr
                        if (ncoll.eq.10) stop
  880                   format('***** Collision',i3,' in cell',3i4)
  881                   format('      Symmetry operator',i4,
     +                       ' on',i6,' atoms')
  882                   format('      Atoms',2i4,'     Distances',
     +                       3f8.3,f10.3)
  883                   format('      Coul, attr, rep',3f15.3)
                     endif

C                    Increase pair list number
                     npla=npla+1
                     if (npla.gt.mxlist) then
                        print*, ' ***** NONBE: Increase mxlist',npla
                        stop
                     endif

C                    Calculate electric field for intermolecular polarization
                     if (npp.eq.1 .and. intra.eq.0) then
                        rij3=rij1*rij2
                        denuli=cg(jj)/rij3
                        denulj=cg(ii)/rij3
                        enul(ii3+1)=enul(ii3+1)+xij*denuli
                        enul(ii3+2)=enul(ii3+2)+yij*denuli
                        enul(ii3+3)=enul(ii3+3)+zij*denuli
                        if (ii3.ne.jj3) then
                           enul(jj3+1)=enul(jj3+1)-xij*denulj*is1
                           enul(jj3+2)=enul(jj3+2)-yij*denulj*is2
                           enul(jj3+3)=enul(jj3+3)-zij*denulj*is3
                        endif
                        r1(npla)=xij
                        r2(npla)=yij
                        r3(npla)=zij
                     endif

C                    Make pair list for LIST (NL=1) and/or for polarization
                     if (npp.eq.1 .or. nl.eq.1) then
                        ipla1(npla)=ii3
                        if (skip) ipla1(npla)=-ii3-1
C                       Negative IPLA1 markes an atom pair to be skipped
                        ipla2(npla)=jj3
                        if (intra.gt.0) ipla2(npla)=-jj3-1
C                       Negative IPLA2 markes any intramolecular atom pair
                        ishift1(npla)=nint(2*(n1+de1))
                        ishift2(npla)=nint(2*(n2+de2))
                        ishift3(npla)=nint(2*(n3+de3))
                        isig1(npla)=is1
                        isig2(npla)=is2
                        isig3(npla)=is3
                        plj6(npla)=ch6
                        plj12(npla)=ch12
                        if (buck) then
                           plj12(npla)=cbu
                           pexp(npla)=ebu
                        endif
                        pcg(npla)=cgij
                        if (doew6) plj6ew(npla)=c6def
                        if (doewc) pcgew(npla)=cgdef
                     endif

C                    Make hydrogen atom pair list for LISTH (NL=2).
C                    No Ewald summation in this case
                     if (nl.eq.2 .and. .not. skip) then
                        do mh=1,ndihh
                           n=3*iome(ndihz+mh)-3
                           if (ii3.eq.n .or. jj3.eq.n) then
                              nplh(mh)=nplh(mh)+1
                              nbl=nplh(mh)
                              if (nbl.gt.maxh) then
                                 print*, ' NONBE: Increase maxh'
                                 stop
                              endif
                              iplh1(mh,nbl)=ii3
                              iplh2(mh,nbl)=jj3
                              shifh1(mh,nbl)=n1+de1
                              shifh2(mh,nbl)=n2+de2
                              shifh3(mh,nbl)=n3+de3
                              isigh1(mh,nbl)=is1
                              isigh2(mh,nbl)=is2
                              isigh3(mh,nbl)=is3
                              pcgh(mh,nbl)=cgij
                           endif
                        enddo
                     endif

                     if (nf.gt.0) then
C                       Calculate derivatives
                        xd=xij*df
                        yd=yij*df
                        zd=zij*df
                        f(ii3+1)=f(ii3+1)-xd
                        f(ii3+2)=f(ii3+2)-yd
                        f(ii3+3)=f(ii3+3)-zd
                        f(jj3+1)=f(jj3+1)+xd*is1
                        f(jj3+2)=f(jj3+2)+yd*is2
                        f(jj3+3)=f(jj3+3)+zd*is3
                        if (nf.eq.1) then
C                          Calculate derivatives to cell axes
                           der(1)=der(1)-xd*(de1+n1)
                           der(2)=der(2)-yd*(de2+n2)
                           der(3)=der(3)-zd*(de3+n3)
                           der(4)=der(4)-xd*(de2+n2)
                           der(5)=der(5)-xd*(de3+n3)
                           der(6)=der(6)-yd*(de3+n3)
c                           print 8888,ii3,jj3,de1+n1,de2+n2,de3+n3,
c     +                          xij,yij,zij,
c     +                          xd*(de2+n2),xd*(de3+n3),yd*(de3+n3)
c 8888                       format(2i3,5x,3f6.2,3f10.5,5x,3f10.5)
                        endif
                        if (nf.eq.2) then
C                          Calculate virial
                           rij(1)=xij
                           rij(2)=yij
                           rij(3)=zij
                           do m=1,3
                              do k=1,3
                                 vira(m,k)=vira(m,k)+rij(m)*df*rij(k)
                              enddo
                           enddo
                        endif
                     endif
  450                continue
                  enddo
  470             continue
               enddo
  475          continue
            enddo
  480       continue
            if (iprint.gt.4) then
               print 142,ni,nj,velcc
               print 141,eel
            endif
c  490       continue
         enddo
      enddo

      if (npp.ne.1) goto 700
C     Calculate polarization energy.
C     Include effects of U-inner on energy and forces (not yet the virial!)
      pip=0
      do m=1,3
         dipind(m)=0
         do i=1,natoms
            j=3*i-3+m
            dip(j)=alfpol(i)*Enul(j)
            Epol=Epol-dip(j)*Enul(j)/2
            dipind(m)=dipind(m)+dip(j)
         enddo
         pip=pip+dipind(m)*pmol(m)
      enddo
      do m=1,3
         der(m)=der(m)-vp3v*pip/box(m)
      enddo
      do m=1,3
         do i=1,natoms
            j=3*i-3+m
            f(j)=f(j)+vp3v*dipind(m)*msom(m)*cg(i)
         enddo
      enddo
      if (iprint.gt.1) print 87,(dipind(m),m=1,3)
      if (iprint.gt.3) then
         print 95
         write(16,95)
         do i=1,natoms
            print 91,i,(dip(3*i-3+m),m=1,3),(enul(3*i-3+m),m=1,3)
            write(16,91) i,(dip(3*i-3+m),m=1,3),
     +           (enul(3*i-3+m),m=1,3)
         enddo
      endif

C     Calculate intermolecular polarization forces
CCCCC (virial yet to be done!!)
      if (nf.eq.0) goto 700
      if (nf.eq.2) goto 700
      do n=1,npla
         j=ipla2(n)
         if (j.lt.0) goto 690
C        No intramolecular distances
         i=ipla1(n)
         if (i.lt.0) i=-(i+1)
         ii=i/3+1
         jj=j/3+1
         qi=cg(ii)
         qj=cg(jj)
         is1=isig1(n)
         is2=isig2(n)
         is3=isig3(n)
         dip1=dip(i+1)
         dip2=dip(i+2)
         dip3=dip(i+3)
         djp1=dip(j+1)*is1
         djp2=dip(j+2)*is2
         djp3=dip(j+3)*is3
         rij2=r1(n)**2+r2(n)**2+r3(n)**2
         rij1=sqrt(rij2)
         rij3=rij1*rij2

         pjr=djp1*r1(n)+djp2*r2(n)+djp3*r3(n)
         pir=dip1*r1(n)+dip2*r2(n)+dip3*r3(n)
         stuk2=3*(-qj*pir+qi*pjr)/(rij2*rij3)
         fj=qj/rij3
         fi=qi/rij3
         xd=dip1*fj-djp1*fi+stuk2*r1(n)
         yd=dip2*fj-djp2*fi+stuk2*r2(n)
         zd=dip3*fj-djp3*fi+stuk2*r3(n)

         if (i.eq.j) then
            xd=xd/2
            yd=yd/2
            zd=zd/2
         endif
         der(1)=der(1)+xd*ishift1(n)/2
         der(2)=der(2)+yd*ishift2(n)/2
         der(3)=der(3)+zd*ishift3(n)/2
         der(4)=der(4)+xd*ishift2(n)/2
         der(5)=der(5)+xd*ishift3(n)/2
         der(6)=der(6)+yd*ishift3(n)/2
         f(i+1)=f(i+1)+xd
         f(i+2)=f(i+2)+yd
         f(i+3)=f(i+3)+zd
         f(j+1)=f(j+1)-xd*is1
         f(j+2)=f(j+2)-yd*is2
         f(j+3)=f(j+3)-zd*is3
  690    continue
      enddo

  700 continue

      if (nvir.gt.0) then
C        Correct derivatives for non-atomic sites
CCCCC    (virial yet to be done!!)
         do k=1,nvirt
            k3=3*(nrpt+k)-3
            do m=1,3
               j=3*jvir(k,m)-3
               if (j.ge.0) then
                  do m1=1,3
                     do m2=1,3
                        f(j+m1)=f(j+m1)+dvir(k,m,m2,m1)*f(k3+m2)
                     enddo
                  enddo
               endif
            enddo
         enddo
      endif

      if (doewc) eel=eel+ecc+ekc+edc
c      print*,'eel+ecc+ekc+edc',eel/ngr,ecc/ngr,ekc/ngr,edc/ngr
      if (doew6) edispa=edispa+ec6+ek6+ed6
      enb=enb+eel+edispa+edispr+Epol
      if (nf.eq.2) then
         do m=1,3
            do n=1,3
               vira(m,n)=vira(m,n)/2
            enddo
         enddo
      endif

      p=ngr
      if (iprint.gt.1) then
         print*,' npla = ',npla
         print 96, enb/p,eel/p,edispa/p,edispr/p,uinner/p,Epol/p
         if (nf.gt.0) print 97, (der(i),i=1,6)
         if (nf.gt.1) then
            print 98, (vira(1,m),m=1,3)
            print 98, (vira(2,m),m=1,3)
            print 98, (vira(3,m),m=1,3)
         endif
         if (nf.gt.0) print 99, (f(i),i=1,3*nrpt)
      endif
   14 format(' Virtual atom',i4,3x,2i4,f8.3,i4,2f8.3)
   20 format(' New atom ',a8,i4,3f10.5,f12.3)
   21 format(a)
   22 format(' ***** Total charge not zero:',f8.3,'. Reset to zero!')
   23 format(' Reading', i4,'  modified charges from file: ',a65)
   24 format(i4,' (virtual) atoms found constituting',i4,' group(s)')
   40 format(' Start NONBE: ngr  nrp  nvir ichad  nspgr  npp  cutoff', 
     +     '        ebig    pressure',/,12x,2i5,2i6,i7,i5,f8.2,2e12.4)
   41 format(i4,3f10.5,3i4,f13.5,8i4)
   42 format(' Charge group ',i3,'  last atom',i4,'   molecule',
     +     i4,'   charge',f10.5,4x,4f10.5)
   43 format(i4,' Charge for virtual atom ',i4,3x,a,f8.5,5x,
     +     i4,3x,2i4,f8.3,i4,2f8.3)
   44 format(i4,' Modified charge for atom',i4,3x,a,f8.5)
   45 format(10f10.5)
   80 format(/,' Start calculation for charge group',i4)
   81 format(12x,'Translation     dx      dy      dz      r',8x,'cgij',
     +     8x,'ch6',10x,'cbu',6x,'ebu',10x,'coul    disp     rep')
   86 format(' U2-inner: ',3f12.3)
   87 format(' Induced molecular dipole moment:     ',3f8.3)
   88 format (12x,3i3,2x,4f8.3,3x,f8.3,2e13.4,f8.3,4x,8f8.3)
   89 format (12x,3i3,2x,4f8.3,3x,f8.3,2e13.4,12x,8f8.3)
   90 format(' box: ',6f10.4)
   91 format(i5,3f9.4,6x,9f8.2)
   92 format(7x,'charge groups NI, NJ:',2i4,8x,
     +     'atoms II, JJ, JJ-exp:',3i4,8x,'molecules:',6i4)
   93 format(' Dipole moment (* 4.0745 for Debye):  ',3f8.3)
   94 format(' Creating field strength:             ',3f8.3)
   95 format('  Induced dipole moments',18x,'from field strengths')
   96 format(' Energy NONBE: ',6f9.3)
   97 format(' - Der:  ',6f12.3)
   98 format(' Virial: ',3f12.3)
   99 format(9f8.1)
  141 format(6x,' Cumulative electrostatic energy so far:',f12.5)
  142 format(6x,' Contribution from charge groups',2i4,f12.5)
  152 format('    Starting charge group combination',i4,' range',
     +     2i4,'  and',i4,' range', 2i4,'  with eq. pos.',8i4)
  155 format(i5,3x,2i5,3x,3i5,10f8.3)
      end
