      subroutine secder(ngr,nrp,enb,box,x,f,hessian,rlist,iprint,
     +     jjsh,n1sh,n2sh,n3sh,msh,sh)

C     Calculates nonbonded crystal energy and second derivatives.
C     B.P. van Eijck, october 1999.

C     One molecule with NRP atoms is augmented with NSPGR symmetry-related
C     ones. The nonbonded distances are selected using a cutoff criterion
C     based on the geometric centers of charge groups.
C     The nonbonded energy ENB is calculated, as well as its second
C     derivatives. It includes Coulomb terms, R^(-6) attractive
C     dispersion terms, and either R^(-12) or exponential repulsion terms.
C     It is calculated on the basis of (NSPGR+1)*NGR groups.
C     Pair quantities calculated are  Bder = (dU/dR) / R and
C        Cder = [(dU/dR) / R - d2U/dR2)] / R^2

C     Polarization is not (yet) possible.
C     Implementation of Ewald summation failed.

C     It is not clear whether or not U2-inner should be included.
C     But the choice matters for polar AND non-polar space groups.

C     Input through parameters:
C     NGR: number of groups
C     NRP: number of atoms
C     BOX(1..6): ax, by, cz, bx, cx, cy
C     X(1..): atom coordinates. X must be big enough to accomodate
C        all symmetry-expanded coordinates.
C     HESSIAN contains bonded contributions
C     IPRINT regulates output to unit 16 and to screen.

C     Input through common blocks:
C     /NONP/ gives information on nonbonded parameters:
C        NCAG: number of charge groups in one molecule
C        IAC(1..NRP): atom codes, taking values 1..Ncod
C        IEXCL(1..NRP,1..NRP): excluded neighbors
C        INC(0..NCAG): charge group pointer list: INC(I) specifies the
C           position of the last atom of the I-th group in the atom sequence.
C           (This algorithm is taken from the GROMOS procedure NONBML)
C        C6(1..Ncod,1..Ncod): attractive C6 parameters
C        CS6(1..Ncod,1..Ncod): same for Lennard-Jones 1..4 interactions
C        C12(1..Ncod,1..Ncod),CS12(1..Ncod,1..Ncod):
C           for Lennard-Jones potential: same for repulsive terms
C           for Buckingham potential: repulsion C12 * exp(-CS12 * R)
C        SC14C, SC14LJ: multiply Lennard-Jones 1..4 Coulomb and other
C           terms with these factors, respectively.
C     /NONB/ gives various information:
C        CG(1..NRP): charges
C        EBIG: if any repulsion term exceeds EBIG, stop immediately and
C           set ENB=90000.
C        BUCK: if TRUE, then choose Buckingham potential
C        NOINT: if TRUE, skip intramolecular terms
C        DDC: if TRUE, use distance-dependent dielectric constant R (A).
C        DRUK: the pressure
C        CUTLV: maximum distance for charge group interactions
C        MSOM(m) = sum over MSP(m), compares with IREF (from LEESR).
C        INNER: do calculation of U2-inner if INNER>0.
C        FINNER: 4*pi/(3.0*(nspgr+1))
C        TOLLV: omit atom pairs which contribute less than TOLLV to
C               Bder*rij**2 and to Cder.
C     /SPGR/ gives space group information:
C        NSPGR, MSP, TSP: see subroutine LEESR
C     /CHAD/ gives information for virtual atoms.
C        ICHAD rules the use of this feature:
C           ICHAD=0: no virtual atoms
C           ICHAD>0: use virtual atoms, with charge information
C                 previously obtained in NONBE with ICHAD<0.
C        NVIR: the number of virtual atoms
C        PCHAD, QCHAD, SCHAD(1..NVIR), JVIR(1..NVIR,1..3):
C           the vectors w = r(jvir(2)) - r(jvir(1))
C                   and v = r(jvir(3)) - r(jvir(1)) and R = [v x w]
C           define a virtual atom at vector r(jvir(1)) + p w [+ q v [+ s R]]
C        NCHAD(1..NVIR) defines how many (3, 5, or 6) of these data are used
C        DVIR is a scratch-array for derivatives, initialized in LEEST
C        INDMOL(0..) defines the molecule of each real atom
C        INDVIR(1..NVIR) defines the molecule of each virtual atom
C        NCAGS is the modified number of charge groups.

C     Output through parameters:
C     ENB: the nonbonded energy
C     Special cases:
C        ENB = 70000: a cell axis is less than 0.5 Angstrom.
C        ENB = 80000: a distance is smaller than 0.5 A
C        ENB = 90000: a repulsion term exceeds EBIG
C     Array X: expanded with the symmetry-related molecules.
C     HESSIAN augmented with nonbonded contributions
C     RLIST(m,1..NPLA)= m-component of vector R(1...NPLA)

C     Output through common blocks:
C     /NBAL/ gives pair list information:
C        NPLA: number of pairs, including vectors to other cells
C        IPLA1(1...NPLA): number of atom (I) in a pair
C        IPLA2(1...NPLA): number of atom (J) in a pair
C        PHI(m1,m2,1..NPLA)=d2U/dr(m1)dr(m2)
C        ISHIFT1..3(1...NPLA): fractional shifts
C        ISHIFT1..3(1...NPLA): fractional shifts
C        And scratch arrays FLR and ING.
C     /NONB/
C        UINNER: the calculated U2-inner
C        EEL: the coulomb energy
C        EDISPA: the attractive dispersion term
C        EDISPR: the repulsive term
      
      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cvira.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbhl.i'
      include 'cinds.i'
      include 'cwald.i'
      include 'cpola.i'
      include 'chadd.i'

      integer i,ia3,iaci,iacj,iacr,ib3,ibasr,ibox,ic3,ici,icj,ii,ii0,
     +     ii3,iim,iimax,iired,im,ind,indmr,inp,intra,iprint,iscr,
     +     j,j1,j2,j3,jj,jj0,jj3,jje,jjm,jjmax,jjsh,jpt,k,k3,kk,kkn,
     +     l,ll,lle,lln,m,m1,m2,m3,m4,msh,n,n1,n1sh,n2,n2sh,n3,n3sh,
     +     nag,natoms,ncagt,ngr,ni,ni3,nim,nj,nj3,njm,nmax,nnj,nop3,
     +     nopi,nopj,npla,npla0,npla1,npla2,npla3,npla4,npla5,nragt,
     +     nrp,nrpt,nrptot,nsj,ntot,nvirt,icq
      double precision al6kw,atsqpi,ax,bder,box,bx,by,c12def,c6def,c6r,
     +     cbu,cder,cgdef,cgi,cgij,cgr,ch12,ch6,cx,cy,cz,d,d2,diag,
     +     ebu,enb,erfc,erfcc,ewco6,ewcoc,exa,f,facq,fact,flr,
     +     ginner,hessian,p,phi,phigr,pi,q,rba,rca,rcutp2,
     +     rij,rij1,rij2,rij4,rij6,rklrij,rlist,s,scr,sh,vda,vdr,vel,
     +     velcc,vol,vp3vn,x,xij,xij0,xr,xsqc,xsqlj,yij,yij0,zij,zij0

C     Common block with pair list data. Note difference with CNBAL.I
      common /nbal/ phi(3,3,mxlist),
     +     ishift1(mxlist),ishift2(mxlist),ishift3(mxlist),
     +     isig1(mxlist),isig2(mxlist),isig3(mxlist),
     +     ipla1(mxlist),ipla2(mxlist),npla
      integer*2 ishift1,ishift2,ishift3,isig1,isig2,isig3,ipla1,ipla2
C     Scratch arrays for nonbe, nonbep and secder
      common /scrt/ flr(3*maxcag),ing(3*maxnsj)
      integer*2 ing
      character*1 yn
      
      dimension diag(maxmat,maxmat,3,3),phigr(0:maxnrp,0:maxnrp,3,3),
     +     hessian(3*maxmat,3*maxmat),ginner(maxmat,3,3)
      dimension box(*),x(*),f(*),rij(3),rba(3),rca(3),
     +     xr(maxxco),cgr(maxat),iacr(maxat),indmr(maxat),
     +     ibasr(maxat,3),scr(3,3),ind(maxxco),
     +     inp(maxcag),iscr(3,2),rlist(3,mxlist)

      logical skip,dew,dewc,dew6

      save icq
      data icq /0/

      dew=doew
      dewc=doewc
      dew6=doew6
C     Implementation of Ewald summation failed. So:
      dew=.false.
      dewc=.false.
      dew6=.false.

      facq=sqrt(332.0636d0*0.4184d0)
      ncagt=ncag*ngr
      nragt=(nspgr+1)*ncagt
      nrpt=nrp*ngr
      nvirt=nvir*ngr
      natoms=nrpt+nvirt
      ntot=natoms*(nspgr+1)
      nrptot=nrpt*(nspgr+1)

      if (iprint.gt.3) then
         print 40, ngr,nrp,nspgr,npp,cutlv,ebig,druk
         if (noint) print*,
     +        '   No calculation of intramolecular nonbonded energy'
         if (buck) print*,'   Buckingham potential used'
         if (ddc) print*,
     +        '   Distance-dependent dielectric constant used'
         if (inner.gt.0) print*,'   U2-inner included'
         print 90,(box(m),m=1,6)
      endif
      do m=1,3
         if (box(m).lt.0.05) then
            print*,' ***** SECDER: Cell axes less than 0.5 Angstrom.'
            write(16,*)
     +           ' ***** SECDER: Cell axes less than 0.5 Angstrom.'
            enb=70000
            return
         endif
      enddo
      open(unit=49,form='UNFORMATTED')

      if (nvir.gt.0) then
C        Create NVIRT additional atom sites for each independent molecule
         noint=.true.
         if (iprint.gt.1) 
     +        print*,'Creating',nvirt,' non-atomic charge sites'
         if (nvirt*(nspgr+1).gt.maxvir) then
            print*,' ***** Increase MAXVIR for', nspgr+1, 
     +           ' equivalent positions:',nvirt
            stop
         endif
         if (ntot.gt.maxmat) then
            print*,' ***** SECDER: Increase MAXMAT:', ntot
            write(16,*) ' ***** SECDER: Increase MAXMAT:', ntot
            stop
         endif
         do i=1,nvirt
            n=nchad(i)
            p=pchad(i)
            q=qchad(i)
            s=schad(i)*10
C           S must have (temporarily?) the dimension A^(-1)
            n3=3*(nrpt+i)-3
            iac(nrpt+i)=0
            do m=1,3
               ibase(nrpt+i,m)=jvir(i,m)
            enddo
            ia3=3*jvir(i,1)-3
            ib3=3*jvir(i,2)-3
            ic3=3*jvir(i,3)-3
            if (n.ge.3) then
               do m=1,3
                  rba(m)=x(ib3+m)-x(ia3+m)
               enddo
            endif
            if (n.ge.5) then
               do m=1,3
                  rca(m)=x(ic3+m)-x(ia3+m)
               enddo
            endif
            if (n.eq.3) then
C              Virtual atom in a bond
               do m=1,3
                  x(n3+m)=x(ia3+m)+p*rba(m)
               enddo
               if (iprint.gt.2) print 20,'in bond ',i,(x(n3+m),m=1,3)
            endif
            if (n.eq.5) then
C              Virtual atom in a plane
               do m=1,3
                  x(n3+m)=x(ia3+m)+p*rba(m)+q*rca(m)
               enddo
               if (iprint.gt.2) print 20,'in plane',i,(x(n3+m),m=1,3)
            endif
            if (n.eq.6) then
C              Virtual atom in three dimensions
               rij(1)=rba(2)*rca(3)-rba(3)*rca(2)
               rij(2)=rba(3)*rca(1)-rba(1)*rca(3)
               rij(3)=rba(1)*rca(2)-rba(2)*rca(1)
               d2=0
               do m=1,3
                  d=p*rba(m)+q*rca(m)+s*rij(m)
                  d2=d2+d*d
                  x(n3+m)=x(ia3+m)+d
                  rba(m)=s*rba(m)
                  rca(m)=s*rca(m)
                  rij(m)=rba(m)-rca(m)
               enddo
               if (iprint.gt.2) print 20,'in 3 dim',i,(x(n3+m),m=1,3),
     +              sqrt(d2)
            endif

C           Find derivatives DVIR(k,j,m1,m2) = d[virtual coord k(m1=x,y,z)]
C                / d[real coord j=a,b,c(m2=x,y,z)]
            do m=1,3
               dvir(i,1,m,m)=1-p-q
               dvir(i,2,m,m)=p
               dvir(i,3,m,m)=q
               if (n.eq.6) then
                  dvir(i,1,3,2)=-rij(1)
                  dvir(i,1,1,3)=-rij(2)
                  dvir(i,1,2,1)=-rij(3)
                  dvir(i,1,2,3)= rij(1)
                  dvir(i,1,3,1)= rij(2)
                  dvir(i,1,1,2)= rij(3)

                  dvir(i,2,3,2)=-rca(1)
                  dvir(i,2,1,3)=-rca(2)
                  dvir(i,2,2,1)=-rca(3)
                  dvir(i,2,2,3)= rca(1)
                  dvir(i,2,3,1)= rca(2)
                  dvir(i,2,1,2)= rca(3)

                  dvir(i,3,2,3)=-rba(1)
                  dvir(i,3,3,1)=-rba(2)
                  dvir(i,3,1,2)=-rba(3)
                  dvir(i,3,3,2)= rba(1)
                  dvir(i,3,1,3)= rba(2)
                  dvir(i,3,2,1)= rba(3)
               endif
            enddo
         enddo
      endif

      npla=0
      npla0=0
      npla1=0
      npla2=0
      npla3=0
      npla4=0
      npla5=0

      rcutp2=cutlv**2
      pi=4*atan(1.0d0)
      atsqpi=2*alphac/sqrt(pi)
      al6kw=alpha6**2
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      edc=0
      ed6=0
      eel=0
      edispa=0
      edispr=0
      ewcoc=0
      ewco6=0      

      uinner=0
      vol=ax*by*cz
      enb=druk*vol*(nspgr+1)
      vp3vn=finner/vol
      if (inner.eq.0) vp3vn=0
C     This is twice the energy factor 2 pi / [3 V (nspgr+1)]
      do m=1,3
         p=pmol(m)
         uinner=uinner-vp3vn*p*p/2
         do i=1,ntot
            f(3*i-3+m)=vp3vn*msom(m)*cg(i)*p
         enddo
      enddo
      if (inner.ne.0) eel=uinner
      if (iprint.gt.3) print 86,uinner

      iscr(1,1)=2
      iscr(1,2)=3
      iscr(2,1)=3
      iscr(2,2)=1
      iscr(3,1)=1
      iscr(3,2)=2
c      do i=1,nvirt
c         do m=1,3
c            print 1111,0,i,i,m,
c     +           ((dvir(i+im,m,m1,m2),m1=1,3),m2=1,3)
c         enddo
c      enddo
      if (nspgr.gt.0) then
C        Do expansion with equivalent positions
         do nop=1,nspgr
            nop3=3*nop-3
            do i=1,natoms
               j=3*i-3
               k=j+nop*3*natoms
               x(k+1)=msp(nop3+1)*x(j+1)+
     +              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
               x(k+2)=msp(nop3+2)*x(j+2)+
     +              tsp(nop3+2)*by+tsp(nop3+3)*cy
               x(k+3)=msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
            enddo
            im=nop*natoms
            do i=1,natoms
               do m=1,3
                  ibase(i+im,m)=ibase(i,m)+im
                  if (ibase(i,m).eq.0) ibase(i+im,m)=0
               enddo
            enddo
            im=nop*nvirt
            do i=1,nvirt
               do m=1,3
                  do m1=1,3
                     do m2=1,3
                        dvir(i+im,m,m1,m2)=dvir(i,m,m1,m2)
     +                       *msp(nop3+m1)*msp(nop3+m2)
c     +                       *msp(nop3+1)*msp(nop3+2)*msp(nop3+3)
C                       That last multiplication was a bad idea ...
                     enddo
                  enddo
c                  print 1111,nop,i,i+im,m,
c     +                 ((dvir(i+im,m,m1,m2),m1=1,3),m2=1,3)
c 1111             format(4i4,12f8.3)
               enddo
            enddo
         enddo
      endif

      if (iprint.gt.4) then
         print*,'Coordinates (after expansion):'
         do i=1,ntot
            if (indmol(i).gt.0) then
               print 41,i,(x(3*i-3+m),m=1,3),0,indmol(i),iac(i),
     +              cg(i)/facq
            else
               print 41,i,(x(3*i-3+m),m=1,3),0,indmol(i),iac(i),
     +              cg(i)/facq,(ibase(i,m),m=1,3)
            endif      
         enddo
      endif

      inc(0)=0
      if (ichad.ne.0) then
C        Collect real and virtual atoms for each molecule in one charge group
         nragt=0
         j=0
  200    continue
         nragt=nragt+1
         do i=1,ntot
            if (abs(indmol(i)).eq.nragt) then
               j=j+1
               ind(j)=i
            endif
         enddo
         inc(nragt)=j
         if (j.lt.ntot) goto 200
      else
         do i=1,ntot
            ind(i)=i
         enddo
      endif

C     Renumber information for the calculation
      do i=1,ntot
         j=ind(i)
         cgr(i)=cg(j)
         iacr(i)=iac(j)
         indmr(i)=indmol(j)
         do m=1,3
            xr(3*i-3+m)=x(3*j-3+m)
            ibasr(i,m)=ibase(j,m)
         enddo
      enddo
      do i=1,ntot
         do m=1,3
            ibase(i,m)=0
         enddo
      enddo
      do i=1,ntot
         j=ind(i)
         do k=1,ntot
            do m=1,3
               if (ibasr(k,m).eq.j) ibase(k,m)=i
            enddo
         enddo
      enddo
c      print*,(inc(i),i=1,nragt)
      ncagt=nragt/(nspgr+1)

      if (iprint.gt.4) then
         print*,'Coordinates (after renumbering)'
         do i=1,ntot
            j=ind(i)
            if (indmr(i).gt.0) then
               print 41,i,(xr(3*i-3+m),m=1,3),j,indmr(i),iacr(i),
     +              cgr(i)/facq
            else
               print 41,i,(xr(3*i-3+m),m=1,3),j,indmr(i),iacr(i),
     +              cgr(i)/facq,(ibase(i,m),m=1,3)
            endif      
         enddo
      endif

      do ii=1,ntot
         do m1=1,3
            do m2=1,3
               ginner(ii,m1,m2)=0
               do jj=1,ntot
                  diag(ii,jj,m1,m2)=0
               enddo
            enddo
            do jj=1,ntot
               if (inner.eq.2 .and. 
     +              indmr(ii).gt.0 .and. indmr(jj).gt.0) then
                  diag(ii,jj,m1,m1)=diag(ii,jj,m1,m1)-
     +                 vp3vn*cgr(ii)*cgr(jj)
               endif
            enddo
         enddo
      enddo

C     Calculate the centers of gravity of the charge groups
      i=0
      DO N=1,NRAGT
         k=0
         q=0
         do m=1,3
            rij(m)=0
         enddo
         nag=inc(n)-inc(n-1)
         DO J=1,NAG
            i=i+1
            if (indmr(i).gt.0) then
               k=k+1
               DO M=1,3
                  RIJ(M)=RIJ(M)+Xr(3*i-3+M)
               enddo
            endif
            q=q+cgr(i)
         enddo
         if (icq.lt.2 .and. abs(q).gt.0.2) then
            print 43,n,q
            icq=1
         endif
         do m=1,3
            flr(3*n-3+m)=rij(m)/k
         enddo
c         if (iprint.gt.2) print 42,n,inc(n),abs(indmr(i)),
c     +        q/facq,(flr(3*n-3+m),m=1,3)
      enddo
      if (icq.eq.1) then
         icq=2
         print*,'LATVIB has no Ewald summation. You may consider ',
     +        'to modify the charge group structure. Continue? (y/n)'
         read*,yn
         if (yn.ne.'y') stop
      endif


      nmax=0
      DO NI=1,nragt
C        Start the next charge group NI, covering all asymmetric units
         ICI=(NI-1)/NCAGT
C        ICI is the number of the equivalent molecule, 0..NSPGR.
         if (iprint.gt.3) print 80,ni
         NI3=3*NI-3
         ii0=inc(ni-1)+1
         iimax=inc(ni)
         DO NJ=1,NRAGT
C           Start the next charge group NJ, including equivalent positions.
C           Make a list of translations needed to find all charge group
C           combinations within CUTLV. The translations are stored in array
C           ING in groups (N1, N2, N3) for every translation 1...NSJ.
            if (iprint.gt.7) print*,' Pair list for charge groups',ni,nj
            NJ3=3*NJ-3
            nsj=0
            xij0=flr(ni3+1)-flr(nj3+1)
            yij0=flr(ni3+2)-flr(nj3+2)
            zij0=flr(ni3+3)-flr(nj3+3)
C           Collect charge group distances within CUTLV.
            n3=int((cutlv-zij0)/cz)
            zij=zij0+n3*cz
            n2=int((cutlv-yij0-n3*cy)/by)
            yij=yij0+n2*by+n3*cy
            n1=int((cutlv-xij0-n2*bx-n3*cx)/ax)
            xij=xij0+n1*ax+n2*bx+n3*cx
  350       continue
            rij2=xij**2+yij**2+zij**2
            if (rij2.lt.rcutp2) then
               nsj=nsj+1
               if (iprint.gt.7) print 155,nsj,ni,nj,n1,n2,n3,
     +              xij,yij,zij,sqrt(rij2)
               if (nsj.gt.maxnsj) then
                  print*,' ***** SECDER: Increase maxnsj',nsj
                  stop
               endif
               jpt=3*(nsj-1)
               ing(jpt+1)=n1
               ing(jpt+2)=n2
               ing(jpt+3)=n3
               if (abs(n1).gt.nmax) nmax=abs(n1)
               if (abs(n2).gt.nmax) nmax=abs(n2)
               if (abs(n3).gt.nmax) nmax=abs(n3)
            endif
            n1=n1-1
            xij=xij-ax
  370       if (xij.lt.-cutlv) then
               n2=n2-1
               yij=yij-by
  380          if (yij.lt.-cutlv) then
                  n3=n3-1
                  zij=zij-cz
                  if (zij.lt.-cutlv) goto 390
                  n2=int((cutlv-yij0-n3*cy)/by)
                  yij=yij0+n2*by+n3*cy
                  goto 380
               endif
               n1=int((cutlv-xij0-n2*bx-n3*cx)/ax)
               xij=xij0+n1*ax+n2*bx+n3*cx
               goto 370
            endif
            goto 350
  390       continue
C           End of making list ING(1 .NSJ)

C           Start the actual calculation for charge groups NI and NJ
            velcc=0
ccccc            IF (nsj.EQ.0) GOTO 490
            ICJ=(NJ-1)/NCAGT
C           ICJ is the number of the equivalent group, 0..NSPGR.
            NNJ=NJ-NCAGT*ICJ
            jj0=inc(nnj-1)+1
            jjmax=inc(nnj)
            if (iprint.gt.4) print 152,ni,ii0,iimax,nj,jj0,jjmax,icj

            do ibox=1,nsj
C              Start the next cell translation from ING
               n1=ing((ibox-1)*3+1)
               n2=ing((ibox-1)*3+2)
               n3=ing((ibox-1)*3+3)
               do ii=0,iimax-ii0
                  do jj=0,jjmax-jj0
                     do m1=1,3
                        do m2=1,3
                           phigr(ii,jj,m1,m2)=0
                        enddo
                     enddo
                  enddo
               enddo
               do ii=ii0,iimax
C                 Start the next atom II, if it is within charge group NI
                  iired=mod(ii-1,natoms)+1
C                 IIRED is reduced to the non-expanded group
                  nim=indmr(ii)
                  iim=ind(mod(ii-1,nrp+nvir)+1)
                  cgi=cgr(iired)
                  iaci=iacr(iired)
                  II3=3*ii-3
C                 NIM and NJM are molecule numbers, negative for virtual atoms.
C                 IIM and JJM indicate the atom numbers in the main group.

                  do jj=jj0,jjmax
C                    Start the next atom JJ, if it is within charge group NJ
C                    JJ0 and JJMAX were reduced to the non-expanded group
                     njm=indmr(jj+natoms*icj)
                     jjm=ind(mod(jj-1,nrp+nvir)+1)
                     jje=jj+natoms*icj
                     if (iprint.gt.7) print 92,ni,nj,ii,jj,jje,
     +                    nim,njm,iim,jjm
                     jj3=3*jj-3
                     j3=jj3+3*icj*natoms
                     iacj=iacr(jj)
                     c6def=c6(iaci,iacj)
                     c12def=c12(iaci,iacj)
                     cgdef=cgi*cgr(jj)
                     if (buck) then
                        cbu=c12def
                        ebu=cs12(iaci,iacj)
                        c12def=0
                     endif
                     ch6=c6def
                     ch12=c12def
                     cgij=cgdef
                     intra=0
                     skip=.false.
                     if (icj.eq.ici .and. 
     +                    n1.eq.0 .and. n2.eq.0 .and. n3.eq.0) then
C                       Possibly an intramolecular distance
                        if (abs(nim).eq.abs(njm)) then
                           intra=1
                           if (njm.gt.0 .and. nim.gt.0)
     +                          intra=iexcl(iim,jjm)
                        endif
                        if (intra.eq.2) then
C                          Change and/or scale 1..4 interaction parameters
                           cgij=cgij*sc14c
                           if (.not. buck) then
                              ch12=cs12(iaci,iacj)*sc14lj
                              ch6=cs6(iaci,iacj)*sc14lj
                           endif
                        endif
                        skip=(intra.eq.1 .or. (noint.and.intra.ne.0))
                     endif
C                    Skip interaction for excluded neighbors
C                    Skip intramolecular interaction if NOINT is true.
                     if (.not. dew .and. skip) goto 450
                     xij=xr(ii3+1)-xr(j3+1)+n1*ax+n2*bx+n3*cx
                     yij=xr(ii3+2)-xr(j3+2)+n2*by+n3*cy
                     zij=xr(ii3+3)-xr(j3+3)+n3*cz

C                    Numerical test SECDER
                     if (jjsh.gt.0 .and. n1.eq.n1sh .and.
     +                    n2.eq.n2sh .and. n3.eq.n3sh) then
                        if (jje.eq.jjsh) then
c                           print*,'changeable atom pair found:',
c     +                          ii,jje,n1,n2,n3,sh
                           if (msh.eq.1) xij=xij-sh
                           if (msh.eq.2) yij=yij-sh
                           if (msh.eq.3) zij=zij-sh
                        endif
                        if (indmr(jje).lt.0) then
                           nop=(jje-1)/natoms
                           kkn=jje-nrpt*(nop+1)
                           do m=1,3
                              if (ibase(jje,m).eq.jjsh) then
C       Nog niet getest! Correct for ind(jje) of zo?
c                                 print*,'with virtual atom',
c     +                                ii,jje,m,ibase(jje,m),kkn,m,msh
                                 xij=xij-sh*dvir(kkn,m,1,msh)
                                 yij=yij-sh*dvir(kkn,m,2,msh)
                                 zij=zij-sh*dvir(kkn,m,3,msh)
                              endif                                   
                           enddo
                        endif
                     endif

                     rij2=xij**2+yij**2+zij**2
c                     if (rij2.lt.1.0d-12) goto 450
                     if (n1.eq.0 .and. n2.eq.0 .and. n3.eq.0 .and.
     +                    ii.eq.jj+natoms*icj) goto 450
                     rij1=sqrt(rij2)
                     rij4=rij2*rij2
                     rij6=rij4*rij2
                     if (rij1.lt.1.0d-6) then
                        print*,' ***** Unacceptably small distance ',
     +                       'in SECDER:',ii,jj,n1,n2,n3,rij1,
     +                       ii3,jj3,j3,icj,nim,njm
                        print*,(xr(ii3+m),m=1,3)
                        print*,(xr(j3+m),m=1,3)
                        enb=80000
                        return
                     endif
                     if (rij1.lt.0.05 .and. (abs(nim).ne.abs(njm))) then
                        if (iprint.gt.1) print*,
     +                       '***** SECDER: short distance: ',ii,jj,
     +                       n1,n2,n3,rij1,ii3,jj3,j3,icj,nim,njm
                        print*,(xr(ii3+m),m=1,3)
                        print*,(xr(j3+m),m=1,3)
                        enb=80000
                        return
                     endif
                     
C                    Calculate harmonic terms (testcase)
c                     b0=0.345
c                     f0=592.8166
c                     if (skip) f0=0
c                     vel=1+0.5*f0*(rij1-b0)**2
c                     Bder=f0*(rij1-b0)/rij1
c                     Cder=(Bder-f0)/rij2
c                     if (b0.gt.-0.1) goto 400

C                    Calculate Coulomb energy
C                    Distance dependent dielectric constant if DDC=TRUE.
                     vel=cgij/rij1
                     if (skip) vel=0
                     Bder=-vel/rij2
                     Cder=-3*vel/rij4
C                    Bder is u'/r, Ader is u", Cder = (Bder - Ader)/rij2
C                    Note: Frenkel's B and C correspond to -Bder and -Cder.
                     if (ddc) then
                        vel=vel/rij1
                        Bder=-2*vel/rij2
                        Cder=-8*vel/rij4
                     endif
                     if (dewc) then
C                       Calculate Ewald correction
                        rklrij=alphac*rij1
                        xsqc=rklrij*rklrij
                        exa=atsqpi*exp(-xsqc)*rij1
                        fact=cgdef/rij1
                        erfc=erfcc(rklrij)
                        ewcoc=(erfc-1)*fact
                        Bder=Bder-(erfc+exa-1)*fact/rij2
                        Cder=Cder-(3*erfc+exa*(3+2*xsqc)-3)*fact/rij4
                     endif

                     if (iaci.eq.0 .or. iacj.eq.0) then
                        vda=0
                        vdr=0
                        ewco6=0
                        goto 400
                     endif

C                    Calculate attractive r^(-6) terms
                     vda=ch6/rij6
                     if (skip) vda=0
                     Bder=Bder+6*vda/rij2
                     Cder=Cder+48*vda/rij4
                     if (dew6) then
C                       Calculate Ewald correction
                        c6r=c6def/rij6
                        xsqlj=al6kw*rij2
                        exa=exp(-xsqlj)
                        fact = (1 + xsqlj * (1 + xsqlj/2)) * exa - 1
                        ewco6=c6r*fact
c                        Bder=Bder+...
c                        Cder=Cder+...
                     endif

C                    Calculate R^(-12) or exponential repulsion
                     if (skip) then
                        vdr=0
                     else
                        if (buck) then
                           vdr=cbu*exp(-ebu*rij1)
                           Bder=Bder-ebu*vdr/rij1
                           Cder=Cder+(Bder-vdr*ebu**2)/rij2
                        else
                           vdr=ch12/(rij6*rij6)
                           Bder=Bder-12*vdr/rij2
                           Cder=Cder-168*vdr/rij4
                        endif
                     endif

  400                continue
                     eel=eel+vel/2
                     edispa=edispa-vda/2
                     edispr=edispr+vdr/2
                     edc=edc+ewcoc/2
                     ed6=ed6-ewco6/2

                     if (edispr.gt.ebig) then
                        if (iprint.gt.1) print*,
     +                       'SECDER: energy > EBIG',rij1,vdr
                        enb=90000
                        return
                     endif
                     if (iprint.gt.7) then
                        if (buck) then
                           print 88,n1,n2,n3,xij,yij,zij,rij1,
     +                          cgij,ch6,cbu,ebu,vel,-vda,vdr,bder,cder
                        else
                           print 89,n1,n2,n3,xij,yij,zij,rij1,
     +                          cgij,ch6,ch12,vel,-vda,vdr,bder,cder
                        endif
                     endif
                     velcc=velcc+vel

C                    Calculate negative derivatives
                     f(ii3+1)=f(ii3+1)-xij*bder
                     f(ii3+2)=f(ii3+2)-yij*bder
                     f(ii3+3)=f(ii3+3)-zij*bder

C                    Throw away contributions smaller than TOLLV
                     if (abs(Cder*rij2).lt.tollv .and.
     +                    abs(Bder).lt.tollv) goto 450

C                    Store results temporarily
                     npla0=npla0+1
                     scr(1,1)=Cder*xij*xij-Bder
                     scr(2,2)=Cder*yij*yij-Bder
                     scr(3,3)=Cder*zij*zij-Bder
                     scr(1,2)=Cder*xij*yij
                     scr(1,3)=Cder*xij*zij
                     scr(2,3)=Cder*yij*zij
                     scr(2,1)=Cder*xij*yij
                     scr(3,1)=Cder*xij*zij
                     scr(3,2)=Cder*yij*zij
c                     write(30,97) ind(ii),ind(jj+natoms*icj),
c     +                    n1,n2,n3,
c     +                    ((scr(m1,m2),m1=1,3),m2=1,3)
C                    Diagonal elements are found from the sum rule:
                     do m1=1,3
                        do m2=1,3
                           phigr(ii-ii0,jj-jj0,m1,m2)=scr(m1,m2)
                           diag(ii,ii,m1,m2)=diag(ii,ii,m1,m2)
     +                          -scr(m1,m2)
                        enddo
                     enddo
  450                continue
                  enddo
C                 End loop over JJ

c  470             continue
               enddo
C              End loop over ii and jj within one box translation

               if (nvir.eq.0) goto 480
C              Construct second derivatives when virtual atoms are present
               do kk=ii0,iimax
                  do ll=jj0,jjmax
                     lle=ll+icj*natoms
                     k=0
                     do m1=1,3
                        do m2=1,3
                           d=phigr(kk-ii0,ll-jj0,m1,m2)
                           if (abs(d).gt.tole + 1.0d-10) k=1
                           scr(m1,m2)=d
                        enddo
                     enddo
                     if (k.eq.1) then

                        if (indmr(kk).lt.0 .and. indmr(ll).gt.0) then
C                          1. Find contribution from PHI(k,j) to PHI(i,j)
C                          Here ii is the basis of kk, and jj=ll
                           do m=1,3
                              ii=ibase(kk,m)
                              if (ii.ne.0) then
                                 k=ind(kk)
                                 nop=(k-1)/natoms+1
                                 kkn=k-nrpt*nop
                                 npla1=npla1+1
                                 do m1=1,3
                                    do m2=1,3
                                       d=0
                                       do m3=1,3
                                          d=d+scr(m3,m2)*
     +                                         dvir(kkn,m,m3,m1)
                                       enddo
                                       phigr(ii-ii0,ll-jj0,m1,m2)=
     +                                      phigr(ii-ii0,ll-jj0,m1,m2)+d
                                    enddo
                                 enddo
c                              write(31,97) ind(ii),ind(lle),ind(kk),m,
c     +                             kkn,((phigr(ii-ii0,ll-jj0,m1,m2),
c     +                             m1=1,3),m2=1,3)
                              endif
                           enddo
                        endif

                        if (indmr(kk).gt.0 .and. indmr(ll).lt.0) then
C                          2. Find contribution from PHI(i,k) to PHI(i,j): 
C                          Here jj is the basis of ll, and ii=kk
                           do m=1,3
                              jj=ibase(ll,m)
                              if (jj.ne.0) then
                                 l=ind(lle)
                                 nop=(l-1)/natoms+1
                                 lln=l-nrpt*nop
                                 npla2=npla2+1
                                 do m1=1,3
                                    do m2=1,3
                                       d=0
                                       do m3=1,3
                                          d=d+scr(m1,m3)*
     +                                         dvir(lln,m,m3,m2)
                                       enddo
                                       phigr(kk-ii0,jj-jj0,m1,m2)=
     +                                      phigr(kk-ii0,jj-jj0,m1,m2)+d
                                    enddo
                                 enddo
c                              write(32,97) ind(kk),ind(jj+icj*natoms),
c     +                             ind(lle),m,
c     +                             lln,((phigr(kk-ii0,jj-jj0,m1,m2),
c     +                             m1=1,3),m2=1,3)
                              endif
                           enddo
                        endif
                        if (indmr(kk).lt.0 .and. indmr(ll).lt.0) then
C                          3. Find contribution from PHI(k,l) to PHI(i,j)
C                          Here ii is the basis of kk, jj is the basis of ll
                           do m=1,3
                              ii=ibase(kk,m)
                              if (ii.ne.0) then
                                 k=ind(kk)
                                 nop=(k-1)/natoms+1
                                 kkn=k-nrpt*nop
                                 do n=1,3
                                    jj=ibase(ll,n)
                                    if (jj.ne.0) then
                                       l=ind(ll)
                                       nop=(l-1)/natoms+1
                                       lln=l-nrpt*nop
                                       npla3=npla3+1
                                       do m1=1,3
                                          do m2=1,3
                                             d=0
                                             do m3=1,3
                                                do m4=1,3
                                                   d=d+scr(m3,m4)*
     +                                                dvir(kkn,m,m3,m1)
     +                                                *dvir(lln,n,m4,m2)
                                                enddo
                                             enddo
                                             phigr(ii-ii0,jj-jj0,m1,m2)=
     +                                            phigr(ii-ii0,jj-jj0,
     +                                            m1,m2)+d
                                          enddo
                                       enddo
c                                    write(33,97) ind(ii),
c     +                                   ind(jj+icj*natoms),
c     +                                   ind(kk),ind(lle),0,
c     +                                   ((phigr(ii-ii0,jj-jj0,m1,m2),
c     +                                   m1=1,3),m2=1,3)
                                    endif
                                 enddo
                              endif
                           enddo
                        endif
                     endif
                  enddo               
               enddo
  480          continue

C              Write results for this box translation
               do ii=ii0,iimax
                  do jj=jj0,jjmax
                     if (indmr(ii).gt.0 .and. indmr(jj).gt.0) then
                        k=0
                        do m1=1,3
                           do m2=1,3
                              d=phigr(ii-ii0,jj-jj0,m1,m2)
                              if (abs(d).gt.tole + 1.0d-10) k=1
                              scr(m1,m2)=d
                           enddo
                        enddo
                        if (k.eq.1) then
C                          Increase pair list number
                           npla=npla+1
                           if (npla.eq.mxlist+1) then
                              print*, ' ***** SECDER: Too many list ',
     +                             'entries, use file'
                           endif
                           ii3=3*ii-3
                           jj3=3*jj-3
                           j3=jj3+3*icj*natoms
                           xij=xr(ii3+1)-xr(j3+1)+n1*ax+n2*bx+n3*cx
                           yij=xr(ii3+2)-xr(j3+2)+n2*by+n3*cy
                           zij=xr(ii3+3)-xr(j3+3)+n3*cz

                           i=ind(ii)
                           j=ind(jj)+icj*natoms
                           nopi=(i-1)/natoms
                           nopj=(j-1)/natoms
                           i=i-nopi*nvirt
                           j=j-nopj*nvirt
                           ii3=3*i-3
                           j3=3*j-3

                           if (npla.le.mxlist) then
                              ipla1(npla)=ii3
                              ipla2(npla)=j3
                              ishift1(npla)=n1
                              ishift2(npla)=n2
                              ishift3(npla)=n3
                              rlist(1,npla)=xij
                              rlist(2,npla)=yij
                              rlist(3,npla)=zij
                              do m1=1,3 
                                 do m2=1,3
                                    phi(m1,m2,npla)=scr(m1,m2)
                                 enddo
                              enddo
c                           write(39,97) i,j,
c     +                          ishift1(npla),ishift2(npla),
c     +                          ishift3(npla),
c     +                          ((phi(m1,m2,npla),m1=1,3),m2=1,3),
c     +                          (rlist(m1,npla),m1=1,3)
                           else
C                             This will slow down the process enormously!
                              write(49) npla,ii3,j3,n1,n2,n3,
     +                             xij,yij,zij,scr
                           endif
                        endif
                     endif
                  enddo
               enddo
            enddo
C           End of group combination NI, NJ

            if (iprint.gt.4) then
               print 142,ni,nj,velcc
               print 141,eel
            endif
  142       format(6x,' Contribution from charge groups',2i4,f12.5)
  141       format(6x,' Cumulative results so far:',f12.5)
c  490       continue
         enddo
      enddo

      do kk=1,ntot
         if (indmr(kk).lt.0) then
            k=ind(kk)
            nop=(k-1)/natoms
            kkn=k-nrpt*(nop+1)
            do m=1,3
               inp(m)=ibase(kk,m)
            enddo

            if (nchad(kkn).eq.6) then
C              4. Find contribution from F(k) to PHI(i,j)
C              Here inp(m), including ii and jj, are the basis of kk
               s=schad(kkn)*10
               nop3=3*nop-3
               if (nop.gt.0) s=s*msp(nop3+1)*msp(nop3+2)*msp(nop3+3)
c               print*,kk,kkn,nop,schad(kkn)*10,s
C              S must have (temporarily?) the dimension A^(-1)
               do n=1,3
                  ii=inp(iscr(n,1))
                  jj=inp(iscr(n,2))
                  npla4=npla4+1
                  do m=1,3
                     m1=iscr(m,1)
                     m2=iscr(m,2)
                     scr(m,m)=0
                     d=f(3*kk-3+m)+vp3vn*pmol(m)*cgr(kk)
                     scr(m1,m2)=-s*d
                     scr(m2,m1)= s*d
                  enddo
                  do m1=1,3
                     do m2=1,3
                        diag(ii,jj,m1,m2)=diag(ii,jj,m1,m2)+scr(m1,m2)
                     enddo
                  enddo
c                  write(34,97) 0,0,kkn,n,11,
c     +                 ((scr(m1,m2),m1=1,3),m2=1,3)

                  npla4=npla4+1
                  do m=1,3
                     m1=iscr(m,1)
                     m2=iscr(m,2)
                     scr(m,m)=0
                     d=f(3*kk-3+m)+vp3vn*pmol(m)*cgr(kk)
                     scr(m1,m2)= s*d
                     scr(m2,m1)=-s*d
                  enddo
                  do m1=1,3
                     do m2=1,3
                        diag(ii,jj,m1,m2)=diag(ii,jj,m1,m2)+scr(m1,m2)
                     enddo
                  enddo
c                  write(34,97) 0,0,kkn,n,22,
c     +                 ((scr(m1,m2),m1=1,3),m2=1,3)
               enddo
            endif

C           5. Find contribution from PHI(k,k) to PHI(i,j)
C           Again inp(m), including ii and jj, are the basis of kk
            do m=1,3
               do n=1,3
                  ii=inp(m)
                  jj=inp(n)
                  if (ii.ne.0 .and. jj.ne.0) then
                     npla5=npla5+1
                     do m1=1,3
                        do m2=1,3
                           d=0
                           do m3=1,3
                              do m4=1,3
                                 d=d+diag(kk,kk,m3,m4)*dvir(kkn,m,m3,m1)
     +                                *dvir(kkn,n,m4,m2)
                              enddo
                           enddo
                           scr(m1,m2)=d
                        enddo
                     enddo
                     do m1=1,3
                        do m2=1,3
                           diag(ii,jj,m1,m2)=diag(ii,jj,m1,m2)
     +                          +scr(m1,m2)
                        enddo
                     enddo
c                     write(35,97) 0,0,kkn,n,m,
c     +                    ((scr(m1,m2),m1=1,3),m2=1,3)
                  endif
               enddo
            enddo
         endif
      enddo

      if (inner.lt.2) goto 800
C     Calculate contribution from U2-inner
      do kk=1,ntot
         if (indmr(kk).lt.0) then
            k=ind(kk)
            nop=(k-1)/natoms
            kkn=k-nrpt*(nop+1)
            do m=1,3
               do m1=1,3
                  do m2=1,3
                     ii=ibase(kk,m)
                     if (ii.ne.0) ginner(ii,m1,m2)=
     +                    ginner(ii,m1,m2)+cgr(kk)*dvir(kkn,m,m2,m1)
                  enddo
               enddo
            enddo
         endif
      enddo

      do ii=1,ntot
         if (indmr(ii).gt.0) then
            do jj=1,ntot
               if (indmr(jj).gt.0) then
                  do m1=1,3
                     do m2=1,3
                        d=0
                        do m=1,3
                           d=d+ginner(ii,m,m1)*ginner(jj,m,m2)
                        enddo
                        diag(ii,jj,m1,m2)=diag(ii,jj,m1,m2)-vp3vn*
     +                       (cgr(ii)*ginner(jj,m1,m2)
     +                       +cgr(jj)*ginner(ii,m2,m1)+d)
                     enddo
                  enddo
               endif
            enddo
         endif
      enddo

  800 continue

C     Restore standard coordinates for LATVIB (only needed for method 1)
      if (nspgr.gt.0) then
C        Do expansion with equivalent positions
         do nop=1,nspgr
            nop3=3*nop-3
            do i=1,nrpt
               j=3*i-3
               k=j+3*nop*nrpt
               x(k+1)=msp(nop3+1)*x(j+1)+
     +              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
               x(k+2)=msp(nop3+2)*x(j+2)+
     +              tsp(nop3+2)*by+tsp(nop3+3)*cy
               x(k+3)=msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
            enddo
         enddo
      endif
      if (iprint.gt.4) then
         print*,'Coordinates (after reduction):'
         do i=1,nrpt*(nspgr+1)
            print 41,i,(x(3*i-3+m),m=1,3)
         enddo
      endif

C     Retain only results for real atoms and add contributions to the hessian
      n=0
      do ii=1,ntot
         i=ind(ii)
         if (indmol(i).gt.0) then
            n=n+1
            j1=3*n-3
            m=0
            do jj=1,ntot
               j=ind(jj)
               if (indmol(j).gt.0) then
                  m=m+1
                  j2=3*m-3
                  do m1=1,3
                     do m2=1,3
                        hessian(j1+m1,j2+m2)=
     +                       hessian(j1+m1,j2+m2)+diag(ii,jj,m1,m2)
                     enddo
                  enddo
               endif
            enddo
         endif
      enddo

      if (nvir.gt.0) then
C        Correct derivatives for non-atomic sites
         do i=1,ntot
            j=ind(i)
            do m=1,3
               xr(3*j-3+m)=f(3*i-3+m)
            enddo
         enddo
         do i=1,3*ntot
            f(i)=xr(i)
         enddo
         do k=1,nvirt
            k3=3*(nrpt+k)-3
            do m=1,3
               j=3*jvir(k,m)-3
               if (j.ge.0) then
                  do m1=1,3
                     do m2=1,3
                        f(j+m1)=f(j+m1)+dvir(k,m,m2,m1)*f(k3+m2)
                     enddo
                  enddo
               endif
            enddo
         enddo
      endif

      if (dewc) eel=eel+edc
      if (dew6) edispa=edispa+ed6
      enb=enb+eel+edispa+edispr

      if (iprint.gt.1) print*,' nmax =',nmax,'       all npla =',
     +     npla,'     ',npla0,npla1,npla2,npla3,npla4,npla5
      p=ngr*(nspgr+1)
      if (iprint.gt.3) then
         print 96,enb/p,eel/p,edc/p,edispa/p,ed6/p,edispr/p,uinner/p
         print 99, (f(i),i=1,3*nrptot)
      endif

      close(49)

   20 format(' New atom ',a8,i4,3f10.5,f12.3)
   40 format(' Start SECDER: ngr, nrp, nspgr, npp, cutlv, ebig,', 
     +     ' pressure: ',4i5,f8.2,2e12.4)
   41 format(i4,3f10.5,3i4,f13.5,8i4)
c   42 format(' Charge group ',i3,'  last atom',i4,'   molecule',
c     +     i4,'   charge',f10.5,4x,4f10.5)
   43 format('***** Charge group',i4,' has charge',f8.3)
   80 format(' Start calculation for charge group',i4)
   86 format(' U2-inner: ',3f8.3)
   88 format (12x,3i3,2x,4f8.3,3x,f8.3,2e13.4,f8.3,4x,3f8.3,2f10.1)
   89 format (12x,3i3,2x,4f8.3,3x,f8.3,2e13.4,12x,3f8.3,2f10.1)
   90 format(' box: ',6f10.4)
   92 format(7x,'charge groups NI, NJ:',2i4,8x,
     +     'atoms II, JJ, JJ-exp:',3i4,8x,'molecules:',6i4)
   96 format(' Energy SECDER: ',10f9.3)
c   97 format(5i4,9f10.3,3f6.3)
   99 format(9f8.1)
  152 format('    Starting charge group combination',i4,' range',
     +     2i4,'  and',i4,' range', 2i4,'  with eq. pos.',8i4)
  155 format(i5,3x,2i5,3x,3i5,10f8.3)
      end
