      SUBROUTINE SHAKE (NR,WINV,NM,NPA,TOL,x,xp,ntc,niter)

C     W.F. VAN GUNSTEREN, GRONINGEN, AUG. 1986

C     Modifications by B. P. van Eijck (lower case):
C     -   The contribution of the constraint forces to the atomic
C         virial is delivered in VIRA through a common block /VIRA/.
C     -   Simplified for crystal structures.

COMMENT   SHAKE WILL SUPPLY CORRECTIONS TO GIVEN COORDINATES XP, SUCH
C     THAT FOR XP A LIST OF CONSTRAINTS WILL BE SATISFIED, EACH
C     GEOMETRICALLY WITHIN A SPECIFIED TOLERANCE TOL. THE CORRECTIONS
C     ARE MADE ALONG VECTORS DERIVED FROM X. WHEN XP=XP(T+DT) RESULTS
C     FROM A NON-CONSTRAINED DYNAMICS TIME STEP DT, STARTING AT X=X(T),
C     THE CORRECTIONS ARE DYNAMICALLY CORRECT UP TO ORDER DT**2.
C     SO THE DYNAMICAL ACCURACY OF SHAKE DEPENDS ON BOTH TOL AND DT,
C     WHEREAS THE GEOMETRICAL ACCURACY ONLY ON TOL.
C     DESCRIPTION OF SHAKE : J. COMP. PHYS. 23 (1977) 327.

C     NR = NUMBER OF ATOMS PER MOLECULE
C     WINV(1..NR) = INVERSE ATOMIC MASSES
C     NM = NUMBER OF MOLECULES
C     NPA = NUMBER OF ATOMS (IN X AND XP) PRECEEDING THE FIRST ATOM
C           OF THE FIRST MOLECULE
C     TOL = RELATIVE GEOMETRICAL TOLERANCE FOR COORDINATE RESETTING
C     X(3*NPA+1..3*(NPA+NM*NR)) = REFERENCE ATOM CARTESIAN COORDINATES
C     XP(3*NPA+1..3*(NPA+NM*NR)) = ATOM CARTESIAN COORDINATES THAT
C                                  WILL BE SHAKEN
C          X- AND Y-AXES LIE ALONG BOX EDGES; CORRESPONDING ATOMS IN
C          X AND XP MUST NOT BE SEPARATED BY PERIODIC TRANSLATIONS
C     NTC = 1 : SHAKE IS NOT PERFORMED, NITER=1
C     NITER = DELIVERED WITH NUMBER OF ITERATIONS

C     IF 1000 OR MORE ITERATIONS APPEAR NECESSARY, OR IF XP DEVIATES
C     TOO MUCH FROM X (DT TOO LARGE), THE SUBR. IS RETURNED WITH A
C     MESSAGE AND NITER=0.

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cvira.i'
      integer i,i3,im,j,j3,k,ll,m,m1,nit,niter,nm,nn,npa,nr,ntc
      double precision acor,diff,rpij2,rrpr,tol,tol2,toler,winv,
     +     x,xh,xij,xp,xpij

      DIMENSION WINV(*),X(*),XP(*),XIJ(3),XPIJ(3),skip(2*maxat)
      LOGICAL SKIP,READY

      IF (NTC.eq.1 .or. nbond.le.0) then
         NITER=1
         RETURN
      endif
      NITER=0
      TOL2=2*TOL
      NN=0

 20   IM=NPA+NN*NR-1
      NN=NN+1
      NIT=0
      DO K=1,NR
         SKIP(K)=.TRUE.
         SKIP(NR+K)=.FALSE.
      enddo
      READY=.FALSE.

 50   IF (READY) GOTO 200
      IF (NIT.GT.1000) GOTO 310
      READY=.TRUE.

      DO LL=1,nbond
         i=ib(ll)
         j=jb(ll)
         IF (SKIP(NR+I).AND.SKIP(NR+J)) GOTO 100
         toler=b0(ll)**2
         I3=3*(I+IM)
         J3=3*(J+IM)
         DIFF=TOLER
         RPIJ2=0
         DO M=1,3
            XPIJ(M)=XP(I3+M)-XP(J3+M)
            RPIJ2=RPIJ2+XPIJ(M)**2
         enddo
         RPIJ2=0
         DO M=1,3
            RPIJ2=RPIJ2+XPIJ(M)**2
         enddo
         DIFF=DIFF-RPIJ2
         IF ( ABS(DIFF).LT.TOLER*TOL2) GOTO 100
         DO M=1,3
            XIJ(M)=X(I3+M)-X(J3+M)
         enddo
         RRPR=0
         DO M=1,3
            RRPR=RRPR+XIJ(M)*XPIJ(M)
         enddo
         IF (RRPR.LT.TOLER*1.d-6) GOTO 320
         ACOR=DIFF/(RRPR*(WINV(I)+WINV(J))*2)
         DO M=1,3
            XH=XIJ(M)*ACOR
            XP(I3+M)=XP(I3+M)+XH*WINV(I)
            XP(J3+M)=XP(J3+M)-XH*WINV(J)
            do m1=1,3
               vira(m,m1)=vira(m,m1)-xh*xij(m1)/dt2
            enddo
         enddo
         SKIP(I)=.FALSE.
         SKIP(J)=.FALSE.
         READY=.FALSE.
 100     continue
      enddo

      NIT=NIT+1
      DO K=1,NR
         SKIP(NR+K)=SKIP(K)
         SKIP(K)=.TRUE.
      enddo
      GOTO 50

 200  NITER=NITER+NIT
      IF (NN.LT.NM) GOTO 20
      NITER=NITER/NN
      RETURN

 310  PRINT 311
      write(16,311)
      NITER=0
      RETURN
 320  PRINT 321,NITER,NIT,NN,LL,I,J
      write(16,321) niter,nit,nn,ll,i,j
      NITER=0
      print 322,i3,(x(i3+m),m=1,3),(xp(i3+m),m=1,3)
      print 322,j3,(x(j3+m),m=1,3),(xp(j3+m),m=1,3)
 311  FORMAT ('COORDINATE RESETTING (SHAKE) WAS NOT ACCOMPLISHED',
     +     ' WITHIN 1000 ITERATIONS')
 321  FORMAT ('COORDINATE RESETTING CANNOT BE ACCOMPLISHED, DEVIATION',
     +     ' IS TOO LARGE',/,'NITER, NIT, NN, LL, I AND J ARE :',7I5)
 322  format (i5,6f10.5)
      END
