      subroutine stedes(n,par,func,nsd,tol,nummer,stdc,
     +     step0,nrest,shimax,iprint,emin,ncyc,rmsder)

C     Performs steepest descent energy minimization of function FUNC
C     with respect to N parameters PAR.
C     Uses subroutine FUNC(n,par,g,en,isubr):
C        EN delivers the value of the function
C        G delivers the negative derivatives to the N parameters PAR
C        ISUBR=1: make pair list(s)
C             =2: use previously made pair list(s)
C     STEDES calls FUNC first with ISUBR=1. During one minimization
C     no new pair lists must be made to ensure stability (ISUBR=2).
C     If the pair list is updated, discontinuities arise and the
C     derivatives become unreliable.
C     The parametershift is STDC*STEP*G/RMSDER, where:
C        G=negative derivative of the energy to the parameter
C        STEP is first STEP0, later it is adjusted:
C             upon increasing energy STEP=STEP/FDIV, else STEP=STEP*MULT
C        RMSDER: rms value of all G*STDC

C     Input:
C        N: number of parameters
C        PAR(1..N): the parameters
C        NSD: maximum number of cycles
C        TOL: stop earlier if energy decreases less than TOL
C        NUMMER: structure number (for error messages)  
C        STDC(1..N): multiplication factor for derivatives
C        STEP0: initial step
C        NREST: make a new pair list and restart after NREST cycles
C        SHIMAX: make a new pair list and restart if any parameter
C                deviates more than SHIMAX from its starting value.
C        IPRINT: regulates print output to screen and unit 16.
C     Input through common block /AIDT/:
C        NCYCGA: the number of the ab-initio cycle
C        MAXCYC: maximum number of ab-initio cycles
C        IREDO>0: shift too large for parameter IREDO

C     Output:
C        EMIN: minized energy
C        NCYC: number of cycles
C        PAR(1..N): optimized parameters
C        RMSDER: rms value of G*STDC, where G(1..N) contains the negative
C                derivatives to the N parameters PAR.
C     Output through common block /AIDT/:
C        IAI=0: calculate intramolecular energy from default force field
C           =1: calculate intramolecular energy and derivatives from GAMESS.
C           =2: calculation through previous call with IAI=1

      implicit none
      include 'params.i'
      include 'caidt.i'
      external func
      double precision dxh,emin,etest,fdiv,fmult,g,
     +     par,par0,par1,rmsder,shimax,stdc,step,step0,tol,winst,zr
      integer i,iprint,n,ncyc,npl,nrest,nsd,nummer
      dimension par(*),stdc(*),
     +     g(maxpar),par1(maxpar),zr(maxpar),par0(maxpar)

      if (iprint.gt.2) print 32,nsd,tol,step0,nrest,shimax
      if (nsd.le.0) return
      if (n.le.0) return
      if (n.gt.maxpar) then
         print*,' ***** Increase MAXPAR in STEDES',n,maxpar
         stop
      endif

C     FMULT: multiplication factor for STEP
C     FDIV: division factor for STEP
      fmult=1.2d0
      fdiv=2

      ncyc=0
      step=step0
  400 continue
      call func(n,par,g,emin,1)
C     This implies calling GAMESS if IAI=1
      if (emin.gt.50000 .and. iprint.gt.1) print 42,emin,nummer,1
      rmsder=0
      do i=1,n
         par0(i)=par(i)
         zr(i)=g(i)*stdc(i)
         rmsder=rmsder+zr(i)*zr(i)
      enddo
      rmsder=sqrt(rmsder/n)
      if (iprint.gt.4) print 40,ncyc,rmsder,step,emin
      if (rmsder.lt.1.0e-30) then
C        Derivatives are zero at (re)start SD
         if (iprint.gt.1) then
            print 43,nummer
c            write(16,43) nummer
         endif
         return
      endif
      npl=0

C     Apply parameter shift and do the calculation
  420 continue
      dxh=step/rmsder
      do i=1,n
         par1(i)=par(i)+zr(i)*dxh
      enddo
c      print 80,(par1(i),i=1,n)
      if (iprint.gt.7) then
         print*,'Coordinates'
         print 80,(par1(i),i=1,n)
      endif
      call func(n,par1,g,etest,2)
      if (iprint.gt.8) then
         print*,'Derivatives'
         print 80,(g(i),i=1,n)
      endif
      if (iprint.gt.1 .and. etest.gt.50000) print 42,etest,nummer,2
      winst=emin-etest
      if (winst.lt.0) then
C        Decrease step if energy increases
         step=step/fdiv
         if (iprint.gt.3) print 70, etest,step
         if (step.lt.step0/1000) then
            if (iprint.gt.3) print*,' ***** STEP in STEDES too small'
C           In single precision hardly any changes
            return
         endif
         goto 420
      endif

C     Now we have found a lower energy
      emin=etest
      rmsder=0
      do i=1,n
         zr(i)=g(i)*stdc(i)
         rmsder=rmsder+zr(i)*zr(i)
         par(i)=par1(i)
      enddo
      rmsder=sqrt(rmsder/n)
      step=step*fmult
      ncyc=ncyc+1
      npl=npl+1
      if (iprint.gt.3) print 50, ncyc,rmsder,step,emin

      if (iredo.gt.0) then
         ncycga=ncycga+1
         if (ncycga.gt.maxcyc) then
            print*,' Too many ab initio cycles, premature end of STEDES'
            return
         endif
         print 37,ncycga,emin
         write(16,37) ncycga,emin
         iai=1
         step=step0
         goto 400
      endif

C     Check whether we are done
      if (ncyc.ge.nsd) then
         if (iprint.gt.1) print 36,ncyc
         return
      endif
C     Normal end of STEDES
      if (abs(winst).lt.tol) then
         if (iprint.gt.1) print*,'Insufficient energy decrease:',
     +        winst,tol
         return
      endif

C     Check whether new pair list is necessary (unless we are entering
C     the final stage, then there may be no time for convergence)
      if (npl.eq.nrest) goto 400
      if (ncyc.ge.nsd-n) goto 420
      do i=1,n
         if (abs(par(i)-par0(i)).gt.shimax) then
            if (iprint.gt.3) print 41, ncyc,rmsder,step,etest,
     +           i, par(i)-par0(i)
            goto 400
         endif
      enddo
      goto 420

   32 format(' Starting STEDES,  nsd,tol,step0,nrest,shimax: ',
     +      i6,2f10.6,i4,f8.3)
   36 format(' Maximum number of iterations exceeded',i6)
   37 format(' SD: Redo GAMESS from STEDES, cycle',i4,'     emin',f15.3)
   40 format(' RESTART SD: ncyc, rmsder, step, emin       ',
     +     i5,f10.3,f14.8,f13.3)
   41 format('     NEW PL: ncyc, rmsder, step, emin, shift',
     +     i5,f10.3,f14.8,f13.3,i5,f10.4)
   42 format(' Unreasonable energy',f15.1,' in STEDES ',
     +     'for structure',i6,' at checkpoint',i3)
   43 format(' ***** Derivatives are zero at restart SD, structure',i6)
   50 format(12x,' SD: ncyc, rmsder, step, emin',i12,f10.3,f14.8,f13.3)
   70 format(13x,'Energy increases to ',f15.3,8x,
     +     'Therefore step decreased to',f10.6)
   80 format(12f10.4)
      end
