      subroutine trans(nrp,x,ireftr,xcm,xtr,phi,theta,psi)

C     Transforms atomic coordinates to a reference axes system.
C     Input:
C         NRP: number of atoms
C         X(1..3*NRP): atomic coordinates
C         IREFTR(1..3): reference atoms

C     The reference system is chosen as follows:
C        The three atoms IREFTR(1..3) define the x,y-plane, and
C        the two atoms IREFTR(1..2) define the x-axis:
C           Calculate vectors v1=r1-r2 and v4=r3-r2
C           Find the x-axis by normalizing v1
C           Find the z-axis from v3=[v1*v4] and normalizing v3
C           Find the y-axis from v2=[v3*v1]
C        If symmetry is present, the z-axis should be the unique axis.
C        If any IREFTR is too large or negative, IREFTR(1..3) is reset.
C     Output:
C         XCM(1..3): average coordinates (H atoms excluded)
C         XTR(1..3*NRP): coordinates in reference system
C         PHI, THETA, PSI: Euler angles (degrees) corresponding to TRMAT

C     Calls function RANDOM.

      implicit none
      integer i,i1,i2,i3,iprint,ireftr,m,n,nrp,nz
      double precision c,costh,phi,psi,rad,rsq1,rsq3,
     +     theta,trmat,v1,v2,v3,v4,x,xcm,xtr
      dimension x(*),xtr(*),xcm(*),ireftr(*),
     +     trmat(3,3),v1(3),v2(3),v3(3),v4(3)
      save 
      data iprint /4/

      if (iprint.gt.7) then
         iprint=0
         print*,' ***** No further information from TRANS'
         write(16,*) ' ***** No further information from TRANS'
      endif
      rad=45/atan(1.0d0)
C     Determine center of atoms
      nz=0
      do m=1,3
         xcm(m)=0
      enddo
      do i=1,nrp
         nz=nz+1
         do m=1,3
            xcm(m)=xcm(m)+x(3*i-3+m)
         enddo
      enddo
      if (nz.gt.0) then
         do m=1,3
            xcm(m)=xcm(m)/nz
         enddo
      endif

C     Determine the axes system
      i1=ireftr(1)
      i2=ireftr(2)
      i3=ireftr(3)
      if (iprint.gt.2) then
         print*,' Starting TRANS with',nrp,' atoms'
         print*,' Local reference coordinates',i1,i2,i3
         write(16,*) ' Starting TRANS with',nrp,' atoms'
         write(16,*) ' Local reference coordinates',i1,i2,i3
         iprint=iprint+1
      endif
      do m=1,3
         if (ireftr(m).gt.nrp .or. ireftr(m).le.0) then
            i1=1
            i2=nrp
            i3=(nrp+1)/2
            if (iprint.gt.2) then
               iprint=iprint+1
               print 20,nrp,i1,i2,i3
               write(16,20) nrp,i1,i2,i3
            endif
            goto 200
         endif
      enddo
 200  continue

      do m=1,3
         v1(m)=x(3*i1-3+m)-x(3*i2-3+m)
         v4(m)=x(3*i3-3+m)-x(3*i2-3+m)
      enddo
 220  continue
      v3(1)=v1(2)*v4(3)-v1(3)*v4(2)
      v3(2)=v1(3)*v4(1)-v1(1)*v4(3)
      v3(3)=v1(1)*v4(2)-v1(2)*v4(1)
      rsq1=0
      rsq3=0
      do m=1,3
         rsq1=rsq1+v1(m)**2
         rsq3=rsq3+v3(m)**2
      enddo
      if (rsq1.lt.1.0e-8) then
         if (iprint.gt.0) then
            iprint=iprint+1
            print*,' ***** TRANS: no reference distance' ,rsq1
            write(16,*) ' ***** TRANS: no reference distance' ,rsq1
            print 10,i1,(x(3*i1-3+m),m=1,3)
            print 10,i2,(x(3*i2-3+m),m=1,3)
            print 10,i3,(x(3*i3-3+m),m=1,3)
         endif
         return
      endif
      if (rsq3.lt.1.0e-8) then
         if (iprint.gt.3) then
            iprint=iprint+1
            print*,' ***** TRANS: atoms nearly linear:',rsq3
            write(16,*) ' ***** TRANS: atoms nearly linear:',rsq3
            print 10,i1,(x(3*i1-3+m),m=1,3)
            print 10,i2,(x(3*i2-3+m),m=1,3)
            print 10,i3,(x(3*i3-3+m),m=1,3)
         endif
C        Assign arbitrary coordinates to atom 3, hopefully not colinear
         v4(1)=1.1d0
         v4(2)=1.2d0
         v4(3)=1.3d0
         goto 220
      endif
      do m=1,3
         v1(m)=v1(m)/sqrt(rsq1)
         v3(m)=v3(m)/sqrt(rsq3)
      enddo
      v2(1)=v3(2)*v1(3)-v3(3)*v1(2)
      v2(2)=v3(3)*v1(1)-v3(1)*v1(3)
      v2(3)=v3(1)*v1(2)-v3(2)*v1(1)

C     Determine the transformation matrix
      do m=1,3
         trmat(m,1)=v1(m)
         trmat(m,2)=v2(m)
         trmat(m,3)=v3(m)
      enddo

C     Transform coordinates to reference axes
      do i=1,nrp
         do m=1,3
            c=0
            do n=1,3
               c=c+trmat(n,m)*(x(3*i-3+n)-xcm(n))
            enddo
            xtr(3*i-3+m)=c
         enddo
      enddo

C     Determine the Euler angles
      costh=trmat(3,3)
      if (abs(costh).gt.1) costh=sign(1.0d0,costh)
      theta=acos(costh)*rad
      if (abs(costh).le.0.9999) then
         psi=atan2(trmat(1,3),trmat(2,3))*rad
         phi=atan2(trmat(3,1),-trmat(3,2))*rad
      else
         phi=0
         psi=atan2(trmat(1,2)*costh,trmat(2,2)*costh)*rad
C        This is an arbitrary choice, only (PHI + PSI) is determinable.
      endif
      if (phi.lt.-0.1) phi=phi+360
      if (psi.lt.-0.1) psi=psi+360

      if (iprint.gt.2) then
         print*,' End TRANS'
      endif
 10   format(i5,3f10.5)
 20   format(' TRANS: Reset for a molecule with',i3,' atoms',6i4)
      end
